@@ -2,6 +2,7 @@
 #undef EAZEL_SERVICES
 #undef ENABLE_NLS
 #undef ENABLE_PROFILER
+#undef ENABLE_INSTALLER
 #undef HAVE_CATGETS
 #undef HAVE_GETTEXT
 #undef HAVE_LC_MESSAGES
@@ -22,3 +23,8 @@
 #undef bzopen
 #undef bzread
 #undef bzwrite
+#undef HAVE_DEV
+#undef HAVE_VOL_DEV
+#undef HAVE_VOL
+#undef HAVE_ETC_MNTTAB
+#undef HAVE_PROC_MOUNTS
