@@ -96,13 +96,17 @@ main (int argc, char *argv[])
 		nautilus_make_warnings_and_criticals_stop_in_debugger
 			(G_LOG_DOMAIN, g_log_domain_glib, "Gdk", "Gtk", "GnomeVFS", "GnomeUI", "Bonobo", "ORBit", NULL);
 	}
+	
+	/* Disable session manager connection */
+	gnome_client_disable_master_connection ();
+
+	gnomelib_register_popt_table (oaf_popt_options, oaf_get_popt_table_name ());
+	orb = oaf_init (argc, argv);
 
 	/* Initialize libraries. */
-        gnome_init_with_popt_table ("nautilus-adapter", VERSION, 
-				    argc, argv,
-				    oaf_popt_options, 0, NULL); 
+        gnome_init ("nautilus-adapter", VERSION, 
+		    argc, argv); 
 	g_thread_init (NULL);
-	orb = oaf_init (argc, argv);
 	gnome_vfs_init ();
 	bonobo_init (orb, CORBA_OBJECT_NIL, CORBA_OBJECT_NIL);
 
