@@ -139,7 +139,7 @@ nautilus_adapter_destroy (GtkObject *object)
 
 	g_free (adapter->details);
 	
-	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
+	NAUTILUS_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
 NautilusAdapter *
