@@ -1924,6 +1924,22 @@ static char
 	} else {
 		i=V(c[0],c[1]);
 		switch (i) {
+		case V('`',' '):
+			out_html(change_to_font('B'));
+			out_html("`");
+			trans_char(c,'"','\a');
+			c=c+j;
+			if (*c!='\n')
+				c++;
+			c=scan_troff(c, 1, NULL);
+			out_html("'");
+			out_html(change_to_font('R'));
+			out_html(NEWLINE);
+			if (fillout)
+				curpos++;
+			else
+				curpos=0;
+			break;
 		case V('a','b'):
 			h=c+j;
 			while (*h && *h !='\n')
@@ -3695,10 +3711,16 @@ main(int argc, char **argv)
 
 		if(output[0])
 		  infh = gzopen(output, "r");
+
 	      }
 	  }
-	if(!infh)
-	  return 3;
+	if(!infh) {
+		printf("<HTML><HEAD><TITLE>Document not found</TITLE>\n"
+		       "</HEAD><BODY>The document \"%s\" couldn't be found. It may have been removed from your system.\n"
+		       "</BODY></HTML>\n", infile);
+
+		return 3;
+	}
 
 	buf=read_man_page();
 	if (!buf) {
