@@ -39,9 +39,11 @@ gboolean mozilla_events_is_key_return (gpointer dom_event);
 
 char *mozilla_events_get_href_for_event (gpointer dom_event);
 
+#if 0
 gboolean mozilla_events_is_in_form_POST_submit (gpointer dom_event);
 
 gboolean mozilla_events_is_url_in_iframe (GtkMozEmbed *embed, const char *uri);
+#endif
 
 #ifdef __cplusplus
 }
