@@ -37,14 +37,17 @@
 
 #include <libnautilus/libnautilus.h>
 #include <libnautilus-extensions/nautilus-background.h>
+#include <libnautilus-extensions/nautilus-ctree.h>
 #include <libnautilus-extensions/nautilus-file.h>
 #include <libnautilus-extensions/nautilus-file-utilities.h>
 #include <libnautilus-extensions/nautilus-glib-extensions.h>
 #include <libnautilus-extensions/nautilus-gtk-extensions.h>
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
+#include <libnautilus-extensions/nautilus-image.h>
 #include <libnautilus-extensions/nautilus-metadata.h>
 #include <libnautilus-extensions/nautilus-string.h>
 #include <libnautilus-extensions/nautilus-theme.h>
+#include <libnautilus-extensions/nautilus-viewport.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtklabel.h>
 #include <gnome.h>
@@ -86,13 +89,8 @@ static void rpm_view_load_location_callback      (NautilusView         *view,
 static void nautilus_rpm_view_verify_package_callback (GtkWidget *widget,
 				   		  NautilusRPMView *rpm_view);
 
-static void file_selection_callback              (GtkCList             *clist,
-                                                  int                   row,
-                                                  int                   column,
-                                                  GdkEventButton       *event,
-                                                  NautilusRPMView      *rpm_view);
-static void go_to_button_callback                (GtkWidget            *widget,
-                                                  NautilusRPMView      *rpm_view);
+static void file_activation_callback (NautilusCTree *ctree, NautilusCTreeNode *node, int column,
+                                      NautilusRPMView *rpm_view);
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusRPMView, nautilus_rpm_view, GTK_TYPE_EVENT_BOX)
 
@@ -119,6 +117,7 @@ nautilus_rpm_view_initialize (NautilusRPMView *rpm_view)
 	GtkWidget *icon_title_box, *title_box;
 	GtkTable *table;
   	char *default_icon_path;
+        GtkWidget *viewport;
   	
   	static gchar *list_headers[] = { N_("Package Contents") };
 	
@@ -137,7 +136,7 @@ nautilus_rpm_view_initialize (NautilusRPMView *rpm_view)
 
 	rpm_view->details->current_uri = NULL;
 	rpm_view->details->background_connection = 0;
-	rpm_view->details->selected_file = -1;
+	rpm_view->details->selected_file = NULL;
 	
 	/* set up the default background color */
   	background = nautilus_get_widget_background (GTK_WIDGET (rpm_view));
@@ -154,9 +153,9 @@ nautilus_rpm_view_initialize (NautilusRPMView *rpm_view)
 	gtk_widget_show (icon_title_box);
 	gtk_box_pack_start (GTK_BOX (rpm_view->details->package_container), icon_title_box, FALSE, FALSE, 0);	
 	
-	/* allocate a pixwidget to hold the icon */
+	/* allocate a nautilus_image to hold the icon */
 	default_icon_path = nautilus_theme_get_image_path ("gnome-pack-rpm.png");
-	rpm_view->details->package_image = gnome_pixmap_new_from_file (default_icon_path);
+	rpm_view->details->package_image = nautilus_image_new (default_icon_path);
 	g_free (default_icon_path);
 	gtk_widget_show (rpm_view->details->package_image);
 	gtk_box_pack_start (GTK_BOX (icon_title_box), rpm_view->details->package_image, FALSE, FALSE, 8);	
@@ -188,7 +187,7 @@ nautilus_rpm_view_initialize (NautilusRPMView *rpm_view)
 	table = GTK_TABLE(gtk_table_new(4, 4, FALSE));
 	gtk_widget_set_usize (GTK_WIDGET (table), 420, -1);
 	
-  	temp_widget = gtk_label_new(_("Size: "));
+  	temp_widget = gtk_label_new(_("Installed Size: "));
 	gtk_label_set_justify (GTK_LABEL (temp_widget), GTK_JUSTIFY_RIGHT);
 	gtk_misc_set_alignment(GTK_MISC(temp_widget), 1.0, 0.5);
 	gtk_table_attach(table, temp_widget, 0,1, 1,2, GTK_FILL, GTK_FILL, 0,0);
@@ -327,51 +326,55 @@ nautilus_rpm_view_initialize (NautilusRPMView *rpm_view)
 	gtk_container_add (GTK_CONTAINER (rpm_view->details->package_verify_button), temp_widget); 	
 	gtk_box_pack_start(GTK_BOX (temp_box), rpm_view->details->package_verify_button,
 				 FALSE, FALSE, 4);		
-	gtk_widget_show(rpm_view->details->package_verify_button);
+        /* FIXME - 6974, 5123 This feature is buggy and is therefore
+         * being disabled for 1.0. It should either be turned on and
+         * fixed after 1.0 or should be left to the planned 1.2
+         * rewrite.
+         gtk_widget_show(rpm_view->details->package_verify_button);
+        */
 
         gtk_signal_connect (GTK_OBJECT (rpm_view->details->package_verify_button), 
                             "clicked", 
                             GTK_SIGNAL_FUNC (nautilus_rpm_view_verify_package_callback), 
                             rpm_view);
-		
+
 	/* add the list of files contained in the package */
 
   	temp_widget = gtk_scrolled_window_new(NULL, NULL);
-  	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(temp_widget),
-				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+  	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(temp_widget),
+                                        GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    	gtk_box_pack_start (GTK_BOX (rpm_view->details->package_container), temp_widget, TRUE, TRUE, 0);	
-  	gtk_widget_show(temp_widget);
- 
-  	rpm_view->details->package_file_list = gtk_clist_new_with_titles(1, list_headers);
-  	gtk_clist_column_titles_passive(GTK_CLIST(rpm_view->details->package_file_list));
-  	gtk_container_add (GTK_CONTAINER (temp_widget), rpm_view->details->package_file_list);	
-  	gtk_widget_show(rpm_view->details->package_file_list);
+  	gtk_widget_show (temp_widget);
+
+        /* use a tree list now */
+        rpm_view->details->package_file_tree = nautilus_ctree_new_with_titles (1, 0, list_headers);
+        gtk_container_add (GTK_CONTAINER (temp_widget), rpm_view->details->package_file_tree);
+        gtk_signal_connect (GTK_OBJECT (rpm_view->details->package_file_tree),
+                            "tree-activate-row", GTK_SIGNAL_FUNC (file_activation_callback), rpm_view);
+        nautilus_ctree_set_line_style (NAUTILUS_CTREE (rpm_view->details->package_file_tree), NAUTILUS_CTREE_LINES_NONE);
+        gtk_widget_show (rpm_view->details->package_file_tree);
 
- 	gtk_signal_connect (GTK_OBJECT (rpm_view->details->package_file_list),
-                            "select_row", GTK_SIGNAL_FUNC (file_selection_callback), rpm_view);
-	
 	/* add an hbox for buttons that operate on the package list */
-	temp_box = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX (rpm_view->details->package_container), temp_box, FALSE, FALSE, 4);
-	gtk_widget_show(temp_box);
+	temp_box = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (rpm_view->details->package_container), temp_box, FALSE, FALSE, 4);
+	gtk_widget_show (temp_box);
 		
-	/* add the file go-to button */
-
-	rpm_view->details->go_to_button = gtk_button_new();		    
-	temp_widget = gtk_label_new (_("Go to selected file"));
-	gtk_widget_show (temp_widget);
-	gtk_container_add (GTK_CONTAINER (rpm_view->details->go_to_button), temp_widget); 	
-	gtk_box_pack_start(GTK_BOX (temp_box), rpm_view->details->go_to_button,
-				 FALSE, FALSE, 2);		
-
-	gtk_signal_connect (GTK_OBJECT(rpm_view->details->go_to_button), "clicked", GTK_SIGNAL_FUNC(go_to_button_callback), rpm_view);	
-	gtk_widget_set_sensitive(rpm_view->details->go_to_button, FALSE);
-	gtk_widget_show(rpm_view->details->go_to_button);
-	
 	/* add the description */
-	rpm_view->details->package_description = gtk_label_new (_("Description"));	
-	gtk_box_pack_start (GTK_BOX (rpm_view->details->package_container), rpm_view->details->package_description,
-				FALSE, FALSE, 8);	
+        temp_widget = gtk_scrolled_window_new (NULL, NULL);
+        gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (temp_widget),
+                                        GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+        viewport = nautilus_viewport_new (NULL, NULL);
+	gtk_viewport_set_shadow_type (GTK_VIEWPORT (viewport), GTK_SHADOW_NONE);
+        gtk_container_add (GTK_CONTAINER (temp_widget), viewport);
+	background = nautilus_get_widget_background (viewport);
+	nautilus_background_set_color (background, RPM_VIEW_DEFAULT_BACKGROUND_COLOR);
+        gtk_widget_show (viewport);
+        gtk_box_pack_start (GTK_BOX (rpm_view->details->package_container), temp_widget, TRUE, TRUE, 8);
+        gtk_widget_show (temp_widget);
+
+	rpm_view->details->package_description = gtk_label_new (_("Description"));
+        gtk_label_set_justify (GTK_LABEL (rpm_view->details->package_description), GTK_JUSTIFY_LEFT);
+        gtk_container_add (GTK_CONTAINER (viewport), rpm_view->details->package_description);
 	gtk_widget_show (rpm_view->details->package_description);
 	
 	/* prepare ourselves to receive dropped objects */
@@ -395,7 +398,7 @@ nautilus_rpm_view_destroy (GtkObject *object)
 #ifdef EAZEL_SERVICES
         pack = (PackageData *) gtk_object_get_data (GTK_OBJECT (rpm_view), "packagedata");
         if (pack) {
-                packagedata_destroy (pack, TRUE);
+                gtk_object_unref (GTK_OBJECT (pack));
         }
 
 	if (rpm_view->details->root_client) {
@@ -419,7 +422,7 @@ nautilus_rpm_view_destroy (GtkObject *object)
 	g_free (rpm_view->details->package_name);
 	g_free (rpm_view->details);
 
-	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
+	NAUTILUS_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
 /* Component embedding support */
@@ -429,38 +432,106 @@ nautilus_rpm_view_get_nautilus_view (NautilusRPMView *rpm_view)
 	return rpm_view->details->nautilus_view;
 }
 
-/* callback to handle file selection in the file list view */
-static void 
-file_selection_callback(GtkCList * clist, int row, int column, GdkEventButton * event, NautilusRPMView* rpm_view)
-{
-	gtk_widget_set_sensitive(rpm_view->details->go_to_button, rpm_view->details->package_installed);
-	rpm_view->details->selected_file = row;
-}
-
-/* callback to handle the go to file button */
-static void 
-go_to_button_callback (GtkWidget * widget, NautilusRPMView *rpm_view)
+static void
+file_activation_callback (NautilusCTree *ctree, NautilusCTreeNode *node, int column, NautilusRPMView *rpm_view)
 {
 	char *path_name;
-	
-	gtk_clist_get_text (GTK_CLIST(rpm_view->details->package_file_list), 
-			    rpm_view->details->selected_file, 0, &path_name);
-	nautilus_view_open_location_in_this_window
-                (rpm_view->details->nautilus_view, path_name);
+        char *path_item;
+        char *new_name;
+
+        path_name = g_strdup ("");
+        while (1) {
+                nautilus_ctree_node_get_pixtext (ctree, node,
+                                                 0, &path_item, NULL, NULL, NULL);
+                if (strcmp (path_item, "/") != 0) {
+                        new_name = g_strdup_printf ("/%s%s", path_item, path_name);
+                        g_free (path_name);
+                        path_name = new_name;
+                }
+
+                if (NAUTILUS_CTREE_ROW (node)->parent == NULL) {
+                        break;
+                }
+                node = NAUTILUS_CTREE_ROW (node)->parent;
+        }
+
+	nautilus_view_open_location_in_this_window (rpm_view->details->nautilus_view, path_name);
 }
 
 static void
-add_to_clist (char *name,
-              NautilusRPMView *view)
+add_to_filename_tree (char *name, NautilusRPMView *view)
 {
-        gtk_clist_append (GTK_CLIST(view->details->package_file_list), &name);
+        char **path;
+        GNode *node, *next_node;
+        gboolean found;
+        int i;
+
+        path = g_strsplit (name, "/", 0);
+        node = view->details->filename_tree;
+        for (i = 0; path[i] != NULL; i++) {
+                if (path[i][0] == '\0') {
+                        continue;
+                }
+                found = FALSE;
+                for (next_node = g_node_first_child (node); next_node != NULL;
+                     next_node = g_node_next_sibling (next_node)) {
+                        if (strcmp (path[i], (char *) next_node->data) == 0) {
+                                /* hit! */
+                                found = TRUE;
+                                node = next_node;
+                                break;
+                        }
+                }
+                if (! found) {
+                        next_node = g_node_new (g_strdup (path[i]));
+                        g_node_insert (node, -1, next_node);
+                        node = next_node;
+                }
+        }
+}
+
+static void
+fill_filename_tree_int (GtkWidget *ctree, NautilusCTreeNode *parent, GNode *node)
+{
+        GNode *child;
+        NautilusCTreeNode *me;
+        char *text[1];
+
+        for (child = g_node_first_child (node); child != NULL; child = g_node_next_sibling (child)) {
+                text[0] = (char *) child->data;
+                me = nautilus_ctree_insert_node (NAUTILUS_CTREE (ctree), parent, NULL, text,
+                                                 0, NULL, NULL, NULL, NULL,
+                                                 (child->children == NULL) ? TRUE : FALSE,
+                                                 TRUE);
+                fill_filename_tree_int (ctree, me, child);
+        }
+}
+
+static void
+fill_filename_tree (NautilusRPMView *rpm_view)
+{
+        NautilusCTreeNode *top;
+        char *text[] = { "/" };
+
+        top = nautilus_ctree_insert_node (NAUTILUS_CTREE (rpm_view->details->package_file_tree),
+                                          NULL, NULL, text,
+                                          0, NULL, NULL, NULL, NULL,
+                                          FALSE, TRUE);
+        fill_filename_tree_int (rpm_view->details->package_file_tree, top, rpm_view->details->filename_tree);
+}
+
+static gboolean
+filename_node_free (GNode *node)
+{
+        g_free (node->data);
+        return FALSE;
 }
 
 /* here's where we do most of the real work of populating the view with info from the package */
 /* open the package and copy the information, and then set up the appropriate views with it */
 /* FIXME bugzilla.eazel.com 725: use gnome-vfs to open the package */
 
-static void 
+static gboolean
 nautilus_rpm_view_update_from_uri (NautilusRPMView *rpm_view, const char *uri)
 {
 	char *temp_str;
@@ -471,7 +542,7 @@ nautilus_rpm_view_update_from_uri (NautilusRPMView *rpm_view, const char *uri)
 		
 	/* load the standard icon as the default */
 	default_icon_path = nautilus_theme_get_image_path ("gnome-pack-rpm.png");
-    	gnome_pixmap_load_file (GNOME_PIXMAP (rpm_view->details->package_image), default_icon_path);
+    	nautilus_image_set_pixbuf_from_file_name (NAUTILUS_IMAGE (rpm_view->details->package_image), default_icon_path);
         g_free (default_icon_path);
 
 #ifdef EAZEL_SERVICES	       
@@ -481,15 +552,26 @@ nautilus_rpm_view_update_from_uri (NautilusRPMView *rpm_view, const char *uri)
                                                                         PACKAGE_FILL_EVERYTHING);
 	g_free (path_name);
 
-        /* Set the file list */
-        gtk_clist_freeze (GTK_CLIST (rpm_view->details->package_file_list));
-        gtk_clist_clear (GTK_CLIST (rpm_view->details->package_file_list));              
+	if (rpm_view->details->package == NULL) {
+		return FALSE;
+	}
+
+        /* set the file tree */
+        gtk_clist_freeze (GTK_CLIST (rpm_view->details->package_file_tree));
+        gtk_clist_clear (GTK_CLIST (rpm_view->details->package_file_tree));
+        if (rpm_view->details->filename_tree != NULL) {
+                g_node_traverse (rpm_view->details->filename_tree, G_PRE_ORDER, G_TRAVERSE_ALL, -1,
+                                 (GNodeTraverseFunc)filename_node_free, NULL);
+                g_node_destroy (rpm_view->details->filename_tree);
+        }
+        rpm_view->details->filename_tree = g_node_new (g_strdup ("/"));
+        g_list_foreach (rpm_view->details->package->provides, (GFunc)add_to_filename_tree, rpm_view);
+        fill_filename_tree (rpm_view);
         temp_str = g_strdup_printf(_("Package Contents: %d files"), 
                                    g_list_length (rpm_view->details->package->provides));
-        g_list_foreach (rpm_view->details->package->provides, (GFunc)add_to_clist, rpm_view);
-        gtk_clist_set_column_title (GTK_CLIST(rpm_view->details->package_file_list), 0, temp_str);
+        gtk_clist_set_column_title (GTK_CLIST(rpm_view->details->package_file_tree), 0, temp_str);
         g_free(temp_str);                
-        gtk_clist_thaw(GTK_CLIST(rpm_view->details->package_file_list));
+        gtk_clist_thaw(GTK_CLIST(rpm_view->details->package_file_tree));
                 
         temp_str = g_strdup_printf(_("Package \"%s\" "), rpm_view->details->package->name);
         gtk_label_set_text (GTK_LABEL (rpm_view->details->package_title), temp_str);
@@ -550,12 +632,23 @@ nautilus_rpm_view_update_from_uri (NautilusRPMView *rpm_view, const char *uri)
 	}
 	if (is_installed != 0) {
 		gtk_widget_show (rpm_view->details->package_uninstall_button);
-                gtk_widget_show (rpm_view->details->package_verify_button);
+                /* FIXME - 6974, 5123 This feature is buggy and is therefore
+                 * being disabled for 1.0. It should either be turned on and
+                 * fixed after 1.0 or should be left to the planned 1.2
+                 * rewrite.
+                 gtk_widget_show (rpm_view->details->package_verify_button);
+                */
         } else {
                 gtk_widget_hide (rpm_view->details->package_uninstall_button);
-                gtk_widget_hide (rpm_view->details->package_verify_button);
+                /* FIXME - 6974, 5123 This feature is buggy and is therefore
+                 * being disabled for 1.0. It should either be turned on and
+                 * fixed after 1.0 or should be left to the planned 1.2
+                 * rewrite.
+                 gtk_widget_hide (rpm_view->details->package_verify_button);
+                */
         }	
 #endif /* EAZEL_SERVICES */              
+	return TRUE;
 }
 
 char*
@@ -576,12 +669,12 @@ nautilus_rpm_view_get_view (NautilusRPMView *view)
         return view->details->nautilus_view;
 }
 
-void
+gboolean
 nautilus_rpm_view_load_uri (NautilusRPMView *rpm_view, const char *uri)
 {
         g_free(rpm_view->details->current_uri);
         rpm_view->details->current_uri = g_strdup (uri);	
-        nautilus_rpm_view_update_from_uri(rpm_view, uri);
+        return nautilus_rpm_view_update_from_uri(rpm_view, uri);
 }
 
 static void
@@ -590,8 +683,11 @@ rpm_view_load_location_callback (NautilusView *view,
                                  NautilusRPMView *rpm_view)
 {
         nautilus_view_report_load_underway (rpm_view->details->nautilus_view);
-        nautilus_rpm_view_load_uri (rpm_view, location);
-        nautilus_view_report_load_complete (rpm_view->details->nautilus_view);
+        if (nautilus_rpm_view_load_uri (rpm_view, location)) {
+        	nautilus_view_report_load_complete (rpm_view->details->nautilus_view);
+	} else {
+		nautilus_view_report_load_failed (rpm_view->details->nautilus_view);
+	}
 }
 
 static gboolean
