@@ -46,6 +46,7 @@ typedef struct {
 	GList *errors;
 	GList *path;
 	GList *packs;
+	GList *handled;
 } GetErrorsForEachData;
 
 #ifdef EIP_DEBUG
@@ -60,7 +61,61 @@ eazel_install_problem_debug_attempts (int *key,
 #endif
 
 static void
-get_detailed_messages_foreach (PackageData *pack, GetErrorsForEachData *data)
+get_detailed_messages_breaks_foreach (PackageBreaks *breakage, GetErrorsForEachData *data)
+{
+	PackageData *previous_pack = NULL;
+	PackageData *top_pack = NULL;
+	GList **errors = &(data->errors);
+	char *top_name = NULL;
+	char *previous_name = NULL;
+	char *package_broken_name = NULL;;
+
+	if (data->path) {
+		previous_pack = PACKAGEDATA(data->path->data);
+		previous_name = packagedata_get_readable_name (previous_pack);
+		top_pack = PACKAGEDATA(g_list_last (data->path)->data);
+		if (top_pack == previous_pack) {
+			top_pack = NULL;
+		} else {
+			top_name = packagedata_get_readable_name (top_pack);
+		}
+	}
+
+	package_broken_name = packagedata_get_readable_name (packagebreaks_get_package (breakage));
+
+	if (IS_PACKAGEFILECONFLICT (breakage)) {
+		char *message;
+
+		if (top_pack != NULL) {
+			message = g_strdup_printf ("Conflict between %s (required by %s) and %s", 
+						   previous_name,
+						   top_name,
+						   package_broken_name);
+		} else {
+			message = g_strdup_printf ("Conflict between %s and %s", 
+						   previous_name,
+						   package_broken_name);
+		}
+		(*errors) = g_list_append (*errors, message);
+	} else if (IS_PACKAGEFEATUREMISSING (breakage)) {
+		PackageFeatureMissing *missing = PACKAGEFEATUREMISSING (breakage);
+		missing = NULL;
+	} else {
+		char *message;
+		
+		message = g_strdup_printf ("Feature dependency between %s and %s", 
+					   previous_name,
+					   package_broken_name);
+		(*errors) = g_list_append (*errors, message);
+	}
+
+	g_free (top_name);
+	g_free (previous_name);
+	g_free (package_broken_name);
+}
+
+static void
+get_detailed_messages_foreach (GtkObject *foo, GetErrorsForEachData *data)
 {
 	char *message = NULL;
 	char *required = NULL;
@@ -69,10 +124,22 @@ get_detailed_messages_foreach (PackageData *pack, GetErrorsForEachData *data)
 	GList **errors = &(data->errors);
 	PackageData *previous_pack = NULL;
 	PackageData *top_pack = NULL;
+	PackageData *pack = NULL;
+
+	if (IS_PACKAGEDATA (foo)) {
+		pack = PACKAGEDATA (foo);
+	} else if (IS_PACKAGEBREAKS (foo)) {
+		pack = packagebreaks_get_package (PACKAGEBREAKS (foo));
+	} else if (IS_PACKAGEDEPENDENCY (foo)) {
+		pack = PACKAGEDEPENDENCY (foo)->package;
+	} else {
+		g_assert_not_reached ();
+	}
+
 
 	if (data->path) {
-		previous_pack = (PackageData*)(data->path->data);
-		top_pack = (PackageData*)(g_list_last (data->path)->data);
+		previous_pack = PACKAGEDATA(data->path->data);
+		top_pack = PACKAGEDATA(g_list_last (data->path)->data);
 		if (top_pack == previous_pack) {
 			previous_pack = NULL;
 		}
@@ -81,26 +148,27 @@ get_detailed_messages_foreach (PackageData *pack, GetErrorsForEachData *data)
 	}
 	required = packagedata_get_readable_name (pack);
 
+	if (g_list_find (data->handled, pack)) { return; }
+
 	switch (pack->status) {
 	case PACKAGE_UNKNOWN_STATUS:
 		break;
+	case PACKAGE_CANCELLED:
+		if (required_by) {
+			if (pack->modifies!=NULL) {
+				message = g_strdup_printf (_("%s was cancelled"), required);
+			}
+		}
+		break;
 	case PACKAGE_SOURCE_NOT_SUPPORTED:
 		message = g_strdup_printf (_("%s is a source package, which is not yet supported"), 
 					   required);
 		break;
 	case PACKAGE_FILE_CONFLICT:
-		if (required_by && top_name) {
-			message = g_strdup_printf (_("%s had a file conflict with %s which %s required"), 
-						   required, 
-						   required_by, 
-						   top_name);
-		} else if (top_name) {
-			message = g_strdup_printf (_("%s had a file conflict with %s"), 
-						   required, top_name);
-		}
+		/* this will be reported below, when parsing the PackageBreaks */
 		break;
 	case PACKAGE_DEPENDENCY_FAIL:
-		if (pack->soft_depends || pack->hard_depends) {
+		if (pack->depends) {
 
 		} else {
 			if (previous_pack && previous_pack->status == PACKAGE_BREAKS_DEPENDENCY) {
@@ -122,13 +190,12 @@ get_detailed_messages_foreach (PackageData *pack, GetErrorsForEachData *data)
 		}
 		break;
 	case PACKAGE_INVALID:
+		message = g_strdup_printf (_("%s is damaged"), required);
 		break;
 	case PACKAGE_CANNOT_OPEN:
 		if (previous_pack) {
-			if (previous_pack->status == PACKAGE_DEPENDENCY_FAIL) {
-				message = g_strdup_printf (_("%s requires %s, which could not be found on the server"), 
-							   required_by,required);
-			}
+			message = g_strdup_printf (_("%s requires %s, which could not be found on the server"), 
+						   required_by,required);
 		} else {
 			message = g_strdup_printf (_("%s could not be found on the server"), 
 						   required);
@@ -137,47 +204,64 @@ get_detailed_messages_foreach (PackageData *pack, GetErrorsForEachData *data)
 	case PACKAGE_PARTLY_RESOLVED:
 		break;
 	case PACKAGE_ALREADY_INSTALLED:
-		message = g_strdup_printf (_("%s is already installed"), required);
+		if (pack->modifies==NULL) {
+			message = g_strdup_printf (_("%s is already installed"), required);
+		}
 		break;
 	case PACKAGE_CIRCULAR_DEPENDENCY: 
-		if (previous_pack->status == PACKAGE_CIRCULAR_DEPENDENCY) {
-			if (g_list_length (data->path) >= 3) {
-				PackageData *causing_package = (PackageData*)((g_list_nth (data->path, 1))->data);
-				char *cause = packagedata_get_readable_name (causing_package);
-				message = g_strdup_printf ("%s and %s are mutexed because of %s", 
-							   required_by,
-							   required, 
-							   cause);
-				g_free (cause);
-			} else {
-				message = g_strdup_printf ("%s and %s exclude each other, but were both needed",
-							   required_by,
-							   required);
-			}
-		} 
+		if (previous_pack == NULL) {
+			message = g_strdup_printf ("%s depends on itself. Possibly SoftCat is updating...",
+						   required);
+		} else {
+			if (previous_pack->status == PACKAGE_CIRCULAR_DEPENDENCY) {
+				if (strcmp (required_by, required)==0) {
+					message = g_strdup_printf ("%s depends on itself. Possibly SoftCat is updating...",
+								   required);					
+				} else {
+					if (g_list_length (data->path) >= 3) {
+						PackageData *causing_package = PACKAGEDATA((g_list_nth (data->path, 1))->data);
+						char *cause = packagedata_get_readable_name (causing_package);
+						message = g_strdup_printf ("%s and %s are mutexed because of %s", 
+									   required_by,
+									   required, 
+									   cause);
+						g_free (cause);
+					} else {
+						message = g_strdup_printf ("%s and %s exclude each other, but were both needed",
+									   required_by,
+									   required);
+					}
+				}
+			} 
+		}
 		break;
 	case PACKAGE_RESOLVED:
 		break;
+	case PACKAGE_PACKSYS_FAILURE:
+		message = g_strdup_printf ("Cannot access the local package system");
+		break;
 	}
 
 	if (message != NULL) {
 		(*errors) = g_list_append (*errors, message);
-	} else {
+	} else if (pack->status == PACKAGE_CANCELLED) {
 		switch (pack->modify_status) {
 		case PACKAGE_MOD_UNTOUCHED:
 			break;
-		case PACKAGE_MOD_UPGRADED:
-			break;
 		case PACKAGE_MOD_DOWNGRADED:
-			message = g_strdup_printf (_("%s, which is newer, is already installed"), 
+			message = g_strdup_printf (_("%s, which is newer, needs downgrade and downgrade is not enabled"), 
 						   required);
 			break;
+		case PACKAGE_MOD_UPGRADED:
+			message = g_strdup_printf (_("%s, which is older, needs upgrade and upgrade is not enabled"), 
+						   required);						     
+			break;
 		case PACKAGE_MOD_INSTALLED:
 			break;
 		case PACKAGE_MOD_UNINSTALLED:
 			break;
 		}
-
+		
 		if (message != NULL) {
 			(*errors) = g_list_append (*errors, message);
 		}
@@ -189,18 +273,20 @@ get_detailed_messages_foreach (PackageData *pack, GetErrorsForEachData *data)
 
 	/* Create the path list */
 	data->path = g_list_prepend (data->path, pack);
+	data->handled = g_list_prepend (data->handled, pack);
 
-	g_list_foreach (pack->soft_depends, (GFunc)get_detailed_messages_foreach, data);
-	g_list_foreach (pack->hard_depends, (GFunc)get_detailed_messages_foreach, data);
-	g_list_foreach (pack->modifies, (GFunc)get_detailed_messages_foreach, data);
-	g_list_foreach (pack->breaks, (GFunc)get_detailed_messages_foreach, data);
+	if (pack->status != PACKAGE_CANCELLED) {
+		g_list_foreach (pack->depends, (GFunc)get_detailed_messages_foreach, data);
+		g_list_foreach (pack->modifies, (GFunc)get_detailed_messages_foreach, data);
+		g_list_foreach (pack->breaks, (GFunc)get_detailed_messages_breaks_foreach, data);
+	}
 
 	/* Pop the currect pack from the path */
 	data->path = g_list_remove (data->path, pack);
 }
 
 static void
-get_detailed_uninstall_messages_foreach (PackageData *pack, 
+get_detailed_uninstall_messages_foreach (GtkObject *foo,
 					 GetErrorsForEachData *data)
 {
 	char *message = NULL;
@@ -210,10 +296,21 @@ get_detailed_uninstall_messages_foreach (PackageData *pack,
 	GList **errors = &(data->errors);
 	PackageData *previous_pack = NULL;
 	PackageData *top_pack = NULL;
+	PackageData *pack = NULL;
+
+	if (IS_PACKAGEDATA (foo)) {
+		pack = PACKAGEDATA (foo);
+	} else if (IS_PACKAGEBREAKS (foo)) {
+		pack = packagebreaks_get_package (PACKAGEBREAKS (foo));
+	} else if (IS_PACKAGEDEPENDENCY (foo)) {
+		pack = PACKAGEDEPENDENCY (foo)->package;
+	} else {
+		g_assert_not_reached ();
+	}
 
 	if (data->path) {
-		previous_pack = (PackageData*)(data->path->data);
-		top_pack = (PackageData*)(g_list_last (data->path)->data);
+		previous_pack = PACKAGEDATA(data->path->data);
+		top_pack = PACKAGEDATA(g_list_last (data->path)->data);
 		if (top_pack == previous_pack) {
 			previous_pack = NULL;
 		}
@@ -224,6 +321,7 @@ get_detailed_uninstall_messages_foreach (PackageData *pack,
 
 	switch (pack->status) {
 	case PACKAGE_UNKNOWN_STATUS:
+	case PACKAGE_CANCELLED:
 		break;
 	case PACKAGE_SOURCE_NOT_SUPPORTED:
 		message = g_strdup_printf (_("%s is a source package, which is not yet supported"), 
@@ -257,6 +355,9 @@ get_detailed_uninstall_messages_foreach (PackageData *pack,
 		break;
 	case PACKAGE_RESOLVED:
 		break;
+	case PACKAGE_PACKSYS_FAILURE:
+		message = g_strdup_printf ("Cannot access the local package system");
+		break;
 	}
 
 	if (message != NULL) {
@@ -270,8 +371,7 @@ get_detailed_uninstall_messages_foreach (PackageData *pack,
 	/* Create the path list */
 	data->path = g_list_prepend (data->path, pack);
 
-	g_list_foreach (pack->soft_depends, (GFunc)get_detailed_uninstall_messages_foreach, data);
-	g_list_foreach (pack->hard_depends, (GFunc)get_detailed_uninstall_messages_foreach, data);
+	g_list_foreach (pack->depends, (GFunc)get_detailed_uninstall_messages_foreach, data);
 	g_list_foreach (pack->modifies, (GFunc)get_detailed_uninstall_messages_foreach, data);
 	g_list_foreach (pack->breaks, (GFunc)get_detailed_uninstall_messages_foreach, data);
 
@@ -314,8 +414,8 @@ compare_problem_case (EazelInstallProblemCase *a, EazelInstallProblemCase *b)
 			result = compare_problem_case (a->u.cannot_solve.problem,
 						       b->u.cannot_solve.problem);
 			break;
-		case EI_PROBLEM_CONTINUE_WITH_FORCE:
-			result = 0;
+		case EI_PROBLEM_CONTINUE_WITH_FLAG:
+			result = (a->u.continue_with_flag.flag == a->u.continue_with_flag.flag);
 			break;
 		case EI_PROBLEM_CASCADE_REMOVE: {
 			GList *a_iterator;
@@ -327,11 +427,11 @@ compare_problem_case (EazelInstallProblemCase *a, EazelInstallProblemCase *b)
 			/* Check that all packages in a occur in b */
 			for (a_iterator = a->u.cascade.packages; 
 			     a_iterator && !result; a_iterator = g_list_next (a_iterator)) {
-				PackageData *a_pack = (PackageData*)a_iterator->data;
+				PackageData *a_pack = PACKAGEDATA(a_iterator->data);
 				GList *b_iterator;
 				for (b_iterator = b->u.cascade.packages; b_iterator; 
 				     b_iterator = g_list_next (b_iterator)) {
-					PackageData *b_pack = (PackageData*)b_iterator->data;
+					PackageData *b_pack = PACKAGEDATA(b_iterator->data);
 					result = eazel_install_package_compare (a_pack, b_pack);
 				}
 			}
@@ -428,19 +528,24 @@ add_cannot_solve_case (EazelInstallProblem *problem,
 }
 
 static void
-add_continue_with_force_case (EazelInstallProblem *problem,
-			      EazelInstallProblemCase *org_pcase,
-			      gboolean file_conflict,
+add_continue_with_flag_case (EazelInstallProblem *problem,
+			     EazelInstallProblemCase *org_pcase,
+			     EazelInstallProblemContinueFlag flag,
+			     gboolean file_conflict,
 			      GList **output)
 {
-	EazelInstallProblemCase *pcase = eazel_install_problem_case_new (EI_PROBLEM_CONTINUE_WITH_FORCE);
-
-#ifdef EIP_DEBUG
-	g_message ("add_continue_with_force_case");
-#endif /* EIP_DEBUG */
+	EazelInstallProblemCase *pcase = eazel_install_problem_case_new (EI_PROBLEM_CONTINUE_WITH_FLAG);
 
 	pcase->u.cannot_solve.problem = org_pcase;
 	pcase->file_conflict = file_conflict;
+	pcase->u.continue_with_flag.flag = flag;
+
+#ifdef EIP_DEBUG
+	g_message ("add_continue_with_flag_case %p %d %d", 
+		   pcase, flag,
+		   pcase->u.continue_with_flag.flag);
+#endif /* EIP_DEBUG */
+
 
 	if (!add_case (problem, pcase, output)) {
 		eazel_install_problem_case_destroy (pcase);
@@ -450,61 +555,69 @@ add_continue_with_force_case (EazelInstallProblem *problem,
 
 static void
 add_force_install_both_case (EazelInstallProblem *problem,
-			     const PackageData *pack_1, 
-			     const PackageData *pack_2,
+			     PackageData *pack_1, 
+			     PackageData *pack_2,
 			     GList **output) 
 {
 	EazelInstallProblemCase *pcase = eazel_install_problem_case_new (EI_PROBLEM_FORCE_INSTALL_BOTH);
-	PackageData *copy_1 = packagedata_copy (pack_1, FALSE);
-	PackageData *copy_2 = packagedata_copy (pack_2, FALSE);
 
 #ifdef EIP_DEBUG
 	g_message ("add_force_install_both_case");
 #endif /* EIP_DEBUG */
-	pcase->u.force_install_both.pack_1 = copy_1;
-	pcase->u.force_install_both.pack_2 = copy_2;
+	gtk_object_ref (GTK_OBJECT (pack_1));
+	gtk_object_ref (GTK_OBJECT (pack_2));
+
+	pcase->u.force_install_both.pack_1 = pack_1;
+	pcase->u.force_install_both.pack_2 = pack_2;
 	
 	if (!add_case (problem, pcase, output)) {
-		add_continue_with_force_case (problem, pcase, FALSE, output);
+		add_continue_with_flag_case (problem, 
+					     pcase, 
+					     EazelInstallProblemContinueFlag_FORCE, 
+					     FALSE, 
+					     output);
 	}
 }
 
 static void
 add_force_remove_case (EazelInstallProblem *problem,
-		       const PackageData *pack,
+		       PackageData *pack,
 		       gboolean file_conflict,
 		       GList **output) 
 {
 	EazelInstallProblemCase *pcase = eazel_install_problem_case_new (EI_PROBLEM_FORCE_REMOVE);
-	PackageData *copy = packagedata_copy (pack, FALSE);
 
 #ifdef EIP_DEBUG
 	g_message ("add_force_remove_case");
 #endif /* EIP_DEBUG */
-
-	pcase->u.force_remove.pack = copy;
+	
+	gtk_object_ref (GTK_OBJECT (pack));
+	pcase->u.force_remove.pack = pack;
 	pcase->file_conflict = file_conflict;
 	
 	if (!add_case (problem, pcase, output)) {
-		add_continue_with_force_case (problem, pcase, file_conflict, output);
+		add_continue_with_flag_case (problem, 
+					     pcase, 
+					     EazelInstallProblemContinueFlag_FORCE, 
+					     file_conflict, output);
 	}
 }
 
 static void
 add_remove_case (EazelInstallProblem *problem,
-		 const PackageData *pack,
+		 PackageData *pack,
 		 gboolean file_conflict,
 		 GList **output)
 {
 
 	EazelInstallProblemCase *pcase = eazel_install_problem_case_new (EI_PROBLEM_REMOVE);
-	PackageData *copy = packagedata_copy (pack, FALSE);
 
 #ifdef EIP_DEBUG
 	g_message ("add_remove_case");
 #endif /* EIP_DEBUG */
-
-	pcase->u.remove.pack = copy;
+	
+	gtk_object_ref (GTK_OBJECT (pack));
+	pcase->u.remove.pack = pack;
 	pcase->file_conflict = file_conflict;
 	
 	if (!add_case (problem, pcase, output)) {
@@ -515,13 +628,12 @@ add_remove_case (EazelInstallProblem *problem,
 
 static void
 add_update_case (EazelInstallProblem *problem,
-		 const PackageData *pack,
+		 PackageData *pack,
 		 gboolean file_conflict,
 		 GList **output)
 {
 	EazelInstallProblemCase *pcase = eazel_install_problem_case_new (EI_PROBLEM_UPDATE);
 	PackageData *copy = packagedata_new ();
-
 #ifdef EIP_DEBUG
 	g_message ("add_update_case");
 #endif /* EIP_DEBUG */
@@ -556,13 +668,17 @@ add_cascade_remove (EazelInstallProblem *problem,
 	if (!add_case (problem, pcase, output)) {
 		GList *iterator;
 		for (iterator = *packs; iterator; iterator = g_list_next (iterator)) {
-			PackageData *pack = (PackageData*)(iterator->data);
+			PackageData *pack = PACKAGEDATA(iterator->data);
 			add_force_remove_case (problem, pack, file_conflict, output);
 		}
 		eazel_install_problem_case_destroy (pcase);
 	}
 }
 
+static void
+get_detailed_cases_breaks_foreach (PackageBreaks *breakage, GetErrorsForEachData *data)
+{
+}
 
 /* 
    FIXME bugzilla.eazel.com
@@ -570,10 +686,23 @@ add_cascade_remove (EazelInstallProblem *problem,
    - package status looks ok, check modification_status
 */
 static void
-get_detailed_cases_foreach (PackageData *pack, GetErrorsForEachData *data)
+get_detailed_cases_foreach (GtkObject *foo, 
+			    GetErrorsForEachData *data)
 {
 	/* GList **errors = &(data->errors); */
 	PackageData *previous_pack = NULL;
+	PackageData *pack = NULL;
+	gboolean no_problem_added = TRUE;
+
+	if (IS_PACKAGEDATA (foo)) {
+		pack = PACKAGEDATA (foo);
+	} else if (IS_PACKAGEBREAKS (foo)) {
+		pack = packagebreaks_get_package (PACKAGEBREAKS (foo));
+	} else if (IS_PACKAGEDEPENDENCY (foo)) {
+		pack = PACKAGEDEPENDENCY (foo)->package;
+	} else {
+		g_assert_not_reached ();
+	}
 
 #ifdef EIP_DEBUG
 	g_message ("get_detailed_cases_foreach (%p)", pack);
@@ -584,28 +713,27 @@ get_detailed_cases_foreach (PackageData *pack, GetErrorsForEachData *data)
 		   packagedata_modstatus_enum_to_str (pack->modify_status));
 #endif /* EIP_DEBUG */
 
+	if (g_list_find (data->handled, pack)) { return; }
+
 	if (data->path) {
-		previous_pack = (PackageData*)(data->path->data);
+		previous_pack = PACKAGEDATA (data->path->data);
 	}
 
 	switch (pack->status) {
 	case PACKAGE_UNKNOWN_STATUS:
 		break;
+	case PACKAGE_CANCELLED:
+		break;
 	case PACKAGE_SOURCE_NOT_SUPPORTED:
 		break;
 	case PACKAGE_FILE_CONFLICT:
-		g_message ("%s:%d", __FILE__, __LINE__);
-		if ((pack->name!= NULL) && previous_pack && (strcmp (pack->name, previous_pack->name) != 0)) {
-			add_update_case (data->problem, pack, TRUE, &(data->errors)); 
-		} else {
-			g_warning ("%s:%d : oops", __FILE__,__LINE__);
-		}
 		break;
 	case PACKAGE_DEPENDENCY_FAIL:
-		if (pack->soft_depends || pack->hard_depends) {
+		if (pack->depends) {
 		} else {
 			if (previous_pack && previous_pack->status == PACKAGE_BREAKS_DEPENDENCY) {
 				add_update_case (data->problem, pack, FALSE, &(data->errors)); 
+				no_problem_added = FALSE;
 			} else {
 				g_warning ("%s:%d : oops", __FILE__,__LINE__);
 			}
@@ -614,6 +742,10 @@ get_detailed_cases_foreach (PackageData *pack, GetErrorsForEachData *data)
 	case PACKAGE_BREAKS_DEPENDENCY:
 		break;
 	case PACKAGE_INVALID:
+		if (previous_pack) {
+			add_remove_case (data->problem, pack, FALSE, &(data->errors));
+			no_problem_added = FALSE;
+		}
 		break;
 	case PACKAGE_CANNOT_OPEN:
 		break;
@@ -624,21 +756,57 @@ get_detailed_cases_foreach (PackageData *pack, GetErrorsForEachData *data)
 	case PACKAGE_CIRCULAR_DEPENDENCY: 
 		if (previous_pack && previous_pack->status == PACKAGE_CIRCULAR_DEPENDENCY) {
 			add_force_install_both_case (data->problem, pack, previous_pack, &(data->errors));
+			no_problem_added = FALSE;
 		} else {
-			g_warning ("%s:%d : oops", __FILE__,__LINE__);
+			/*
+			add_cannot_solve_case (data->problem,
+					       NULL,
+					       &(data->errors));
+			*/
 		}
 		break;
 	case PACKAGE_RESOLVED:
 		break;
+	case PACKAGE_PACKSYS_FAILURE:
+		break;
+	}
+
+	if (no_problem_added) {
+		if (pack->status == PACKAGE_CANCELLED) {
+			switch (pack->modify_status) {
+			case PACKAGE_MOD_UNTOUCHED:
+				break;
+			case PACKAGE_MOD_DOWNGRADED:
+				add_continue_with_flag_case (data->problem,
+							     NULL,
+							     EazelInstallProblemContinueFlag_DOWNGRADE,
+							     FALSE,
+							     &(data->errors));
+				no_problem_added = FALSE;
+				break;
+			case PACKAGE_MOD_UPGRADED:
+				add_continue_with_flag_case (data->problem,
+							     NULL,
+							     EazelInstallProblemContinueFlag_UPGRADE,
+							     FALSE,
+							     &(data->errors));
+				no_problem_added = FALSE;
+				break;
+			case PACKAGE_MOD_INSTALLED:
+				break;
+			case PACKAGE_MOD_UNINSTALLED:
+				break;
+			}
+		}
 	}
 
 	/* Create the path list */
 	data->path = g_list_prepend (data->path, pack);
+	data->handled = g_list_prepend (data->handled, pack);
 
-	g_list_foreach (pack->soft_depends, (GFunc)get_detailed_cases_foreach, data);
-	g_list_foreach (pack->hard_depends, (GFunc)get_detailed_cases_foreach, data);
+	g_list_foreach (pack->depends, (GFunc)get_detailed_cases_foreach, data);
 	g_list_foreach (pack->modifies, (GFunc)get_detailed_cases_foreach, data);
-	g_list_foreach (pack->breaks, (GFunc)get_detailed_cases_foreach, data);
+	g_list_foreach (pack->breaks, (GFunc)get_detailed_cases_breaks_foreach, data);
 
 	/* Pop the currect pack from the path */
 	data->path = g_list_remove (data->path, pack);
@@ -650,9 +818,20 @@ get_detailed_cases_foreach (PackageData *pack, GetErrorsForEachData *data)
    - package status looks ok, check modification_status
 */
 static void
-get_detailed_uninstall_cases_foreach (PackageData *pack, GetErrorsForEachData *data)
+get_detailed_uninstall_cases_foreach (GtkObject *foo, GetErrorsForEachData *data)
 {
 	PackageData *previous_pack = NULL;
+	PackageData *pack = NULL;
+
+	if (IS_PACKAGEDATA (foo)) {
+		pack = PACKAGEDATA (foo);
+	} else if (IS_PACKAGEBREAKS (foo)) {
+		pack = packagebreaks_get_package (PACKAGEBREAKS (foo));
+	} else if (IS_PACKAGEDEPENDENCY (foo)) {
+		pack = PACKAGEDEPENDENCY (foo)->package;
+	} else {
+		g_assert_not_reached ();
+	}
 	
 #ifdef EIP_DEBUG
 	g_message ("get_detailed_uninstall_cases_foreach (data->path = %p)", data->path);
@@ -661,7 +840,7 @@ get_detailed_uninstall_cases_foreach (PackageData *pack, GetErrorsForEachData *d
 #endif /* EIP_DEBUG */
 
 	if (data->path) {
-		previous_pack = (PackageData*)(data->path->data);
+		previous_pack = PACKAGEDATA(data->path->data);
 	}
 
 	if (pack->toplevel) {
@@ -670,6 +849,7 @@ get_detailed_uninstall_cases_foreach (PackageData *pack, GetErrorsForEachData *d
 
 	switch (pack->status) {
 	case PACKAGE_UNKNOWN_STATUS:
+	case PACKAGE_CANCELLED:
 		break;
 	case PACKAGE_SOURCE_NOT_SUPPORTED:
 		break;
@@ -678,7 +858,8 @@ get_detailed_uninstall_cases_foreach (PackageData *pack, GetErrorsForEachData *d
 	case PACKAGE_DEPENDENCY_FAIL:
 	case PACKAGE_BREAKS_DEPENDENCY:
 		/* This is what a uninstall fail tree will normally have */
-		data->packs = g_list_prepend (data->packs, packagedata_copy (pack, FALSE));
+		gtk_object_ref (GTK_OBJECT (pack));
+		data->packs = g_list_prepend (data->packs, pack);
 		break;
 	case PACKAGE_INVALID:
 		break;
@@ -692,13 +873,14 @@ get_detailed_uninstall_cases_foreach (PackageData *pack, GetErrorsForEachData *d
 		break;
 	case PACKAGE_RESOLVED:
 		break;
+	case PACKAGE_PACKSYS_FAILURE:
+		break;
 	}
 
 	/* Create the path list */
 	data->path = g_list_prepend (data->path, pack);
 
-	g_list_foreach (pack->soft_depends, (GFunc)get_detailed_uninstall_cases_foreach, data);
-	g_list_foreach (pack->hard_depends, (GFunc)get_detailed_uninstall_cases_foreach, data);
+	g_list_foreach (pack->depends, (GFunc)get_detailed_uninstall_cases_foreach, data);
 	g_list_foreach (pack->modifies, (GFunc)get_detailed_uninstall_cases_foreach, data);
 	g_list_foreach (pack->breaks, (GFunc)get_detailed_uninstall_cases_foreach, data);
 
@@ -709,7 +891,7 @@ get_detailed_uninstall_cases_foreach (PackageData *pack, GetErrorsForEachData *d
 		data->path = NULL;
 	}
 
-	if (pack->toplevel) {
+	if (pack->toplevel && pack->status == PACKAGE_BREAKS_DEPENDENCY) {
                 /* This is just to make sure the toplevel is first */
 		data->packs = g_list_reverse (data->packs);
 		add_cascade_remove (data->problem, &(data->packs), FALSE, &(data->errors));
@@ -799,7 +981,7 @@ eazel_install_problem_case_to_string (EazelInstallProblemCase *pcase, gboolean n
 		}
 		iterator  = pcase->u.cascade.packages;
 		while (iterator) {
-			PackageData *pack = (PackageData*)iterator->data;
+			PackageData *pack = PACKAGEDATA(iterator->data);
 			iterator = g_list_next (iterator);
 			if (iterator) {
 				if (g_list_next (iterator)) {
@@ -817,14 +999,18 @@ eazel_install_problem_case_to_string (EazelInstallProblemCase *pcase, gboolean n
 		g_string_free (str, TRUE);
 	}
 	break;
-	case EI_PROBLEM_CONTINUE_WITH_FORCE: {
-		/* FIXME
-		   needs better string */
-		if (! name_only) {
-			if (pcase->file_conflict) {
-				message = g_strdup_printf ("Complete operation by ignoring file conflicts");
-			} else {
-				message = g_strdup_printf ("Complete operation by ignoring problems");
+	case EI_PROBLEM_CONTINUE_WITH_FLAG: {
+		if (!name_only) {
+			switch (pcase->u.continue_with_flag.flag) {
+			case EazelInstallProblemContinueFlag_FORCE:
+				message = g_strdup_printf (_("Continue with force"));
+				break;
+			case EazelInstallProblemContinueFlag_UPGRADE:
+				message = g_strdup_printf (_("Allow upgrades"));
+				break;
+			case EazelInstallProblemContinueFlag_DOWNGRADE:
+				message = g_strdup_printf (_("Allow downgrade"));
+				break;
 			}
 		}
 	}
@@ -881,7 +1067,7 @@ eazel_install_problem_case_foreach_destroy (EazelInstallProblemCase *pcase,
 	case EI_PROBLEM_CANNOT_SOLVE:
 		eazel_install_problem_case_foreach_destroy (pcase->u.cannot_solve.problem, NULL);
 		break;
-	case EI_PROBLEM_CONTINUE_WITH_FORCE:
+	case EI_PROBLEM_CONTINUE_WITH_FLAG:
 		break;
 	case EI_PROBLEM_CASCADE_REMOVE:
 		g_list_foreach (pcase->u.cascade.packages,
@@ -1048,12 +1234,11 @@ eazel_install_problem_step (EazelInstallProblem *problem)
    encountered problems in the given PackageData tree */
 void
 eazel_install_problem_tree_to_case (EazelInstallProblem *problem,
-				    const PackageData *pack,
+				    PackageData *pack,
 				    gboolean uninstall,
 				    GList **output)
 {
 	GetErrorsForEachData data;
-	PackageData *pack_copy;
 
 	P_SANITY (problem);
 
@@ -1066,16 +1251,18 @@ eazel_install_problem_tree_to_case (EazelInstallProblem *problem,
 	data.problem = problem;
 	data.errors = (*output);
 	data.path = NULL;
-	pack_copy = packagedata_copy (pack, TRUE);
+	data.handled = NULL;
 	problem->in_step_problem_mode = FALSE;
 
+	gtk_object_ref (GTK_OBJECT (pack));
+
 	if (uninstall) {
-		get_detailed_uninstall_cases_foreach (pack_copy, &data);
+		get_detailed_uninstall_cases_foreach (GTK_OBJECT (pack), &data);
 	} else {
-		get_detailed_cases_foreach (pack_copy, &data);
+		get_detailed_cases_foreach (GTK_OBJECT (pack), &data);
 	}
 
-	gtk_object_unref (GTK_OBJECT (pack_copy));
+	gtk_object_unref (GTK_OBJECT (pack));
 	(*output) = data.errors;
 }
 
@@ -1083,27 +1270,27 @@ eazel_install_problem_tree_to_case (EazelInstallProblem *problem,
    encountered problems in the given PackageData tree */
 GList* 
 eazel_install_problem_tree_to_string (EazelInstallProblem *problem,				      
-				      const PackageData *pack,
+				      PackageData *pack,
 				      gboolean uninstall)
 {
 	GList *result = NULL;
 	GetErrorsForEachData data;
-	PackageData *pack_copy;
 
 	P_SANITY_VAL (problem, result);
 
 	data.problem = problem;
 	data.errors = NULL;
 	data.path = NULL;
-	pack_copy = packagedata_copy (pack, TRUE);
+	data.handled = NULL;
 
+	gtk_object_ref (GTK_OBJECT (pack));
 	if (uninstall) {
-		get_detailed_uninstall_messages_foreach (pack_copy, &data);
+		get_detailed_uninstall_messages_foreach (GTK_OBJECT (pack), &data);
 	} else {
-		get_detailed_messages_foreach (pack_copy, &data);
+		get_detailed_messages_foreach (GTK_OBJECT (pack), &data);
 	}
 
-	gtk_object_unref (GTK_OBJECT (pack_copy));
+	gtk_object_unref (GTK_OBJECT (pack));
 	result = data.errors;
 
 	return result;
@@ -1213,32 +1400,31 @@ build_categories_from_problem_list (EazelInstallProblem *problem,
 
 		switch (pcase->problem) {
 		case EI_PROBLEM_UPDATE:
-			packages = g_list_prepend (packages, 
-						   packagedata_copy (pcase->u.update.pack, TRUE));
+			gtk_object_ref (GTK_OBJECT (pcase->u.update.pack));
+			packages = g_list_prepend (packages, pcase->u.update.pack);
 			break;
 		case EI_PROBLEM_FORCE_INSTALL_BOTH:
-			packages = g_list_prepend (packages, 
-						   packagedata_copy (pcase->u.force_install_both.pack_1, TRUE));
-			packages = g_list_prepend (packages, 
-						   packagedata_copy (pcase->u.force_install_both.pack_2, TRUE));
+			gtk_object_ref (GTK_OBJECT (pcase->u.force_install_both.pack_1));
+			gtk_object_ref (GTK_OBJECT (pcase->u.force_install_both.pack_2));
+			packages = g_list_prepend (packages, pcase->u.force_install_both.pack_1);
+			packages = g_list_prepend (packages, pcase->u.force_install_both.pack_2);
 			break;
 		case EI_PROBLEM_REMOVE:
-			packages = g_list_prepend (packages, 
-						   packagedata_copy (pcase->u.remove.pack, TRUE));
+			gtk_object_ref (GTK_OBJECT (pcase->u.remove.pack));
+			packages = g_list_prepend (packages, pcase->u.remove.pack);
 			break;
 		case EI_PROBLEM_FORCE_REMOVE:
-			packages = g_list_prepend (packages, 
-						   packagedata_copy (pcase->u.force_remove.pack, TRUE));
+			gtk_object_ref (GTK_OBJECT (pcase->u.force_remove.pack));
+			packages = g_list_prepend (packages, pcase->u.force_remove.pack);
 			break;
 		case EI_PROBLEM_CASCADE_REMOVE: {
 			GList *iterator;
-			g_message ("%s:%d cascade", __FILE__, __LINE__);
 			for (iterator = pcase->u.cascade.packages; iterator; iterator = g_list_next (iterator)) {
 				packages = g_list_prepend (packages, iterator->data);
 			}
 		}
 		break;
-		case EI_PROBLEM_CONTINUE_WITH_FORCE:
+		case EI_PROBLEM_CONTINUE_WITH_FLAG:
 		case EI_PROBLEM_INCONSISTENCY:
 		case EI_PROBLEM_CANNOT_SOLVE:
 			break;
@@ -1308,7 +1494,7 @@ eazel_install_problem_step_problem (EazelInstallProblem *problem,
 					 pcase->file_conflict,
 					 &result);
 			break;
-		case EI_PROBLEM_CONTINUE_WITH_FORCE:		
+		case EI_PROBLEM_CONTINUE_WITH_FLAG:		
 			g_message ("%s:%d, conflict = %s", __FILE__, __LINE__, pcase->file_conflict ? "TRUE":"FALSE");
 			add_cannot_solve_case (problem, 
 					       pcase, 
@@ -1316,10 +1502,11 @@ eazel_install_problem_step_problem (EazelInstallProblem *problem,
 			break;
 		case EI_PROBLEM_FORCE_INSTALL_BOTH:
 			g_message ("%s:%d, conflict = %s", __FILE__, __LINE__, pcase->file_conflict ? "TRUE":"FALSE");
-			add_continue_with_force_case (problem, 
-						      pcase, 
-						      pcase->file_conflict,
-						      &result);
+			add_continue_with_flag_case (problem, 
+						     pcase, 
+						     EazelInstallProblemContinueFlag_FORCE,
+						     pcase->file_conflict,
+						     &result);
 			break;
 		case EI_PROBLEM_CASCADE_REMOVE:
 			g_message ("%s:%d, conflict = %s", __FILE__, __LINE__, pcase->file_conflict ? "TRUE":"FALSE");
@@ -1337,10 +1524,11 @@ eazel_install_problem_step_problem (EazelInstallProblem *problem,
 			break;
 		case EI_PROBLEM_FORCE_REMOVE:
 			g_message ("%s:%d, conflict = %s", __FILE__, __LINE__, pcase->file_conflict ? "TRUE":"FALSE");
-			add_continue_with_force_case (problem, 
-						      pcase, 
-						      pcase->file_conflict,
-						      &result);
+			add_continue_with_flag_case (problem, 
+						     pcase, 
+						     EazelInstallProblemContinueFlag_FORCE,
+						     pcase->file_conflict,
+						     &result);
 			break;
 		case EI_PROBLEM_INCONSISTENCY:
 			g_message ("%s:%d, conflict = %s", __FILE__, __LINE__, pcase->file_conflict ? "TRUE":"FALSE");
@@ -1369,6 +1557,33 @@ eazel_install_problem_step_problem (EazelInstallProblem *problem,
    the most important ones, and execute them given 
    a EazelInstall service object */
 
+#ifndef EAZEL_INSTALL_NO_CORBA
+static void
+eazel_install_problem_done (EazelInstallCallback *service,
+			    gboolean result,
+			    gboolean *done)
+{
+	(*done) = TRUE;
+}
+
+static void
+eazel_install_problem_wait_for_completion (EazelInstallProblem *problem,
+					   EazelInstallCallback *service)
+{
+	guint handle;
+	gboolean done = FALSE;
+	handle = gtk_signal_connect (GTK_OBJECT (service), "done", 
+				     GTK_SIGNAL_FUNC (eazel_install_problem_done), 
+				     (gpointer)&done);
+
+	while (!done) {
+		g_main_iteration (TRUE);
+	}
+
+	gtk_signal_disconnect (GTK_OBJECT (service), handle);
+}
+#endif
+
 /*
   FIXME:
   I need to hookup to service's "done" signal, if successfull, erase
@@ -1402,10 +1617,13 @@ eazel_install_problem_handle_cases (EazelInstallProblem *problem,
 
 	P_SANITY (problem);
 
+	/* the service_ are used to store the current values */
 	service_force = FALSE; 
 	service_update = FALSE; 
 	service_downgrade = FALSE; 
 	service_uninstall = FALSE;
+
+	/* These are used to set new values */
 	force = FALSE; 
 	update = FALSE; 
 	downgrade = FALSE;
@@ -1437,14 +1655,18 @@ eazel_install_problem_handle_cases (EazelInstallProblem *problem,
 	   First get the old values to we can reset them */
 #ifdef EAZEL_INSTALL_NO_CORBA
 	service_force = eazel_install_get_force (service);
-	service_update = eazel_install_get_update (service);
+	service_update = eazel_install_get_upgrade (service);
 	service_downgrade = eazel_install_get_downgrade (service);
 #else /* EAZEL_INSTALL_NO_CORBA */
 	service_force = GNOME_Trilobite_Eazel_Install__get_force (corba_service, &ev);
-	service_update = GNOME_Trilobite_Eazel_Install__get_update (corba_service, &ev);
+	service_update = GNOME_Trilobite_Eazel_Install__get_upgrade (corba_service, &ev);
 	service_downgrade = GNOME_Trilobite_Eazel_Install__get_downgrade (corba_service, &ev);
 #endif /* EAZEL_INSTALL_NO_CORBA */
 
+	force = service_force;
+	update = service_update;
+	downgrade = service_downgrade;
+
 	/* now determine the new parameters */
 	switch (dominant_problem_type) {
 	case EI_PROBLEM_UPDATE:
@@ -1452,15 +1674,25 @@ eazel_install_problem_handle_cases (EazelInstallProblem *problem,
 		update = TRUE;
 		downgrade = FALSE;
 		break;
-	case EI_PROBLEM_CONTINUE_WITH_FORCE:		
-		/* FIXME
-		   this is a complete hack to make the installer
-		   work without force installing */
-#ifdef EAZEL_INSTALL_NO_CORBA		
-		eazel_install_set_ignore_file_conflicts (service, TRUE);
-#else /* EAZEL_INSTALL_NO_CORBA */
-		GNOME_Trilobite_Eazel_Install__set_ignore_file_conflicts (corba_service, TRUE, &ev);
-#endif /* EAZEL_INSTALL_NO_CORBA */
+	case EI_PROBLEM_CONTINUE_WITH_FLAG: {
+		GList *iterator;
+
+		for (iterator = dominant_problems; iterator; iterator = g_list_next (iterator)) {
+			EazelInstallProblemCase *pcase = (EazelInstallProblemCase*)iterator->data;
+			switch (pcase->u.continue_with_flag.flag) {
+			case EazelInstallProblemContinueFlag_FORCE:
+				force = TRUE;
+				break;
+			case EazelInstallProblemContinueFlag_UPGRADE:
+				update = TRUE;
+				break;
+			case EazelInstallProblemContinueFlag_DOWNGRADE:
+				downgrade = TRUE;
+				break;
+			}
+		}
+		break;
+	}
 	case EI_PROBLEM_FORCE_INSTALL_BOTH:
 		force = TRUE;
 		update = TRUE;
@@ -1488,12 +1720,12 @@ eazel_install_problem_handle_cases (EazelInstallProblem *problem,
 	/* set the new parameters */
 #ifdef EAZEL_INSTALL_NO_CORBA		
 	eazel_install_set_force (service, force);
-	eazel_install_set_update (service, update);
+	eazel_install_set_upgrade (service, update);
 	eazel_install_set_downgrade (service, downgrade);
 #else /* EAZEL_INSTALL_NO_CORBA */
-	GNOME_Trilobite_Eazel_Install__set_force (corba_service, service_force, &ev);
-	GNOME_Trilobite_Eazel_Install__set_update (corba_service, service_update, &ev);
-	GNOME_Trilobite_Eazel_Install__set_downgrade (corba_service, service_downgrade, &ev);
+	GNOME_Trilobite_Eazel_Install__set_force (corba_service, force, &ev);
+	GNOME_Trilobite_Eazel_Install__set_upgrade (corba_service, update, &ev);
+	GNOME_Trilobite_Eazel_Install__set_downgrade (corba_service, downgrade, &ev);
 #endif /* EAZEL_INSTALL_NO_CORBA */
 
 	/* do we add any of the usergiven cateogries ? */
@@ -1506,7 +1738,7 @@ eazel_install_problem_handle_cases (EazelInstallProblem *problem,
 			final_categories = g_list_copy (categories);
 		}
 		break;
-	case EI_PROBLEM_CONTINUE_WITH_FORCE:
+	case EI_PROBLEM_CONTINUE_WITH_FLAG:
 		if (service_uninstall) {
 			if (!categories) {
 				final_categories = categorydata_list_copy (*uninstall_categories);
@@ -1551,7 +1783,7 @@ eazel_install_problem_handle_cases (EazelInstallProblem *problem,
 
 	/* fire it off */
 	switch (dominant_problem_type) {
-	case EI_PROBLEM_CONTINUE_WITH_FORCE:
+	case EI_PROBLEM_CONTINUE_WITH_FLAG:
 		if (service_uninstall) {
 #ifdef EAZEL_INSTALL_NO_CORBA
 			eazel_install_uninstall_packages (service, final_categories, root);
@@ -1592,13 +1824,17 @@ eazel_install_problem_handle_cases (EazelInstallProblem *problem,
 		break;
 	}
 
+#ifndef EAZEL_INSTALL_NO_CORBA 
+	eazel_install_problem_wait_for_completion (problem, service);
+#endif
+
 #ifdef EAZEL_INSTALL_NO_CORBA
 	eazel_install_set_force (service, service_force);
-	eazel_install_set_update (service, service_update);
+	eazel_install_set_upgrade (service, service_update);
 	eazel_install_set_downgrade (service, service_downgrade);
 #else /* EAZEL_INSTALL_NO_CORBA */
 	GNOME_Trilobite_Eazel_Install__set_force (corba_service, service_force, &ev);
-	GNOME_Trilobite_Eazel_Install__set_update (corba_service, service_update, &ev);
+	GNOME_Trilobite_Eazel_Install__set_upgrade (corba_service, service_update, &ev);
 	GNOME_Trilobite_Eazel_Install__set_downgrade (corba_service, service_downgrade, &ev);
 #endif /* EAZEL_INSTALL_NO_CORBA */
 
