@@ -21,6 +21,7 @@
  * Authors: J Shane Culpepper <pepper@eazel.com>
  *          Joe Shaw <joe@helixcode.com>
  *          Eskil Heyn Olsen <eskil@eazel.com>
+ *	    Robey Pointer <robey@eazel.com>
  */
 
 /* eazel-install - services command line install/update/uninstall
@@ -71,13 +72,21 @@ parse_package (xmlNode* package, gboolean set_toplevel) {
 	}
 
 	tmp = trilobite_xml_get_string (package, "BYTESIZE");
-	if (tmp) {
+	if (tmp != NULL) {
 		rv->bytesize = atoi (tmp);
 		g_free (tmp);
 	} else {
 		rv->bytesize = 0;
 	}
 
+	tmp = trilobite_xml_get_string (package, "FILESIZE");
+	if (tmp != NULL) {
+		rv->filesize = atoi (tmp);
+		g_free (tmp);
+	} else {
+		rv->filesize = 0;
+	}
+
 	tmp = trilobite_xml_get_string (package, "PROVIDES");
 	if (tmp) {
 		char **files;
@@ -105,28 +114,18 @@ parse_package (xmlNode* package, gboolean set_toplevel) {
 	}
 	
 	/* Dependency Lists */
-	rv->soft_depends = NULL;
-	rv->hard_depends = NULL;
 	rv->breaks = NULL;
 	rv->modifies = NULL;
 
 	dep = package->xmlChildrenNode;
 	while (dep) {
-		if (g_strcasecmp (dep->name, "SOFT_DEPEND") == 0) {
+		if (g_strcasecmp (dep->name, "DEPENDENCY") == 0) {
+			PackageDependency *dependency = g_new0 (PackageDependency, 1);
 			PackageData* depend;
 
 			depend = parse_package (dep, FALSE);
-			packagedata_add_pack_to_soft_depends (rv, depend);
-		} else if (g_strcasecmp (dep->name, "HARD_DEPEND") == 0) {
-			PackageData* depend;
-
-			depend = parse_package (dep, FALSE);
-			packagedata_add_pack_to_hard_depends (rv, depend);
-		} else if (g_strcasecmp (dep->name, "BREAKS") == 0) {
-			PackageData* depend;
-
-			depend = parse_package (dep, FALSE);
-			packagedata_add_pack_to_breaks (rv, depend);
+			dependency->package = depend;
+			packagedata_add_pack_to_depends (rv, dependency);
 		} else if (g_strcasecmp (dep->name, "MODIFIES") == 0) {
 			PackageData* depend;
 
@@ -533,18 +532,16 @@ parse_pkg_template (const char* pkg_template_file, char **result) {
    If given a droot and a title, uses that so create a subnode
    (primarily used for the recursiveness of PackageData objects.
    If not, creates a node with the name "PACKAGE" */
-xmlNodePtr
-eazel_install_packagedata_to_xml (const PackageData *pack, 
-				  char *title, 
-				  xmlNodePtr droot, 
-				  gboolean include_provides)
+static xmlNodePtr
+eazel_install_packagedata_to_xml_int (const PackageData *pack, 
+				      char *title, 
+				      xmlNodePtr droot, 
+				      gboolean include_provides,
+				      GList **path)
 {
 	xmlNodePtr root, node;
 	char *tmp;
 	GList *iterator;
-	/* This is a horrible crackpatch for PR3 used to check for recursive
-	   depends... */
-	static GList *path = NULL;
 
 	if (droot) {
 		g_assert (title != NULL);
@@ -552,6 +549,7 @@ eazel_install_packagedata_to_xml (const PackageData *pack,
 	} else {
 		root = xmlNewNode (NULL, "PACKAGE");
 	}
+
 	node = xmlNewChild (root, NULL, "NAME", pack->name);
 	node = xmlNewChild (root, NULL, "VERSION", pack->version);
 	node = xmlNewChild (root, NULL, "MINOR", pack->minor);
@@ -575,10 +573,14 @@ eazel_install_packagedata_to_xml (const PackageData *pack,
 		g_free (tmp);
 	}
 
-	tmp = g_strdup_printf ("%d", pack->bytesize);
+	tmp = g_strdup_printf ("%u", pack->bytesize);
 	node = xmlNewChild (root, NULL, "BYTESIZE", tmp);
 	g_free (tmp);
 
+	tmp = g_strdup_printf ("%u", pack->filesize);
+	node = xmlNewChild (root, NULL, "FILESIZE", tmp);
+	g_free (tmp);
+
 	if (include_provides && pack->provides) {
 		tmp = g_strdup ((char*)(pack->provides->data));
 		for (iterator = g_list_next (pack->provides); iterator; iterator = g_list_next (iterator)) {
@@ -592,46 +594,40 @@ eazel_install_packagedata_to_xml (const PackageData *pack,
 	}
 	
 	for (iterator = pack->depends; iterator; iterator = iterator->next) {
-		if (g_list_find (path, ((PackageDependency*)iterator->data)->package) == NULL) {
-			path = g_list_prepend (path, ((PackageDependency*)iterator->data)->package);
-			eazel_install_packagedata_to_xml (((PackageDependency*)iterator->data)->package, 
-							  "SOFT_DEPEND", 
-							  root, 
-							  include_provides);
-			path = g_list_remove (path, ((PackageDependency*)iterator->data)->package);
+		if (g_list_find (*path, PACKAGEDEPENDENCY (iterator->data)->package) == NULL) {
+			(*path) = g_list_prepend (*path, PACKAGEDEPENDENCY (iterator->data)->package);
+			eazel_install_packagedata_to_xml_int (PACKAGEDEPENDENCY (iterator->data)->package, 
+							      "DEPENDENCY", 
+							      root, 
+							      include_provides,
+							      path);
+			(*path) = g_list_remove (*path, PACKAGEDEPENDENCY (iterator->data)->package);
 		}
 	}
-	for (iterator = pack->soft_depends; iterator; iterator = iterator->next) {
-		eazel_install_packagedata_to_xml ((PackageData*)iterator->data, 
-						  "SOFT_DEPEND", 
-						  root, 
-						  include_provides);
-	}
-	for (iterator = pack->hard_depends; iterator; iterator = iterator->next) {
-		node = xmlNewChild (root, NULL, "HARD_DEPEND", NULL);
-		eazel_install_packagedata_to_xml ((PackageData*)iterator->data, 
-						  "HARD_DEPEND", 
-						  root,
-						  include_provides);
-	}
-	for (iterator = pack->breaks; iterator; iterator = iterator->next) {
-		eazel_install_packagedata_to_xml ((PackageData*)iterator->data, 
-						  "BREAKS", 
-						  root,
-						  include_provides);
-	}
 	for (iterator = pack->modifies; iterator; iterator = iterator->next) {
-		eazel_install_packagedata_to_xml ((PackageData*)iterator->data, 
-						  "MODIFIES", 
-						  root,
-						  include_provides);
+		GList *subpath = NULL;
+		eazel_install_packagedata_to_xml_int (PACKAGEDATA (iterator->data), 
+						      "MODIFIES", 
+						      root,
+						      include_provides,
+						      &subpath);
 	}
 
 	return root;
 }
 
 xmlNodePtr
-eazel_install_packagelist_to_xml (GList *packages, gboolean include_provides) {
+eazel_install_packagedata_to_xml (const PackageData *pack, 
+				  gboolean include_provides)
+{
+	GList *path = NULL;
+
+	return eazel_install_packagedata_to_xml_int (pack, NULL, NULL, include_provides, &path);
+}
+
+xmlNodePtr
+eazel_install_packagelist_to_xml (GList *packages, gboolean include_provides)
+{
 	xmlNodePtr node;
 	GList *iterator;
 
@@ -639,13 +635,13 @@ eazel_install_packagelist_to_xml (GList *packages, gboolean include_provides) {
 	node = xmlNewNode (NULL, "PACKAGES");
 	for (iterator = packages; iterator; iterator = iterator->next) {
 		xmlAddChild (node, 
-			     eazel_install_packagedata_to_xml ((PackageData*)iterator->data, 
-							       NULL, NULL, include_provides)
+			     eazel_install_packagedata_to_xml (PACKAGEDATA (iterator->data), 
+							       include_provides)
 			);
 	}
 
 	return node;	
-};
+}
 
 xmlNodePtr
 eazel_install_categorydata_to_xml (const CategoryData *category)
@@ -681,7 +677,16 @@ osd_parse_provides (PackageData *pack, xmlNodePtr node, GList **feature_list)
 				dep = packagedependency_new ();
 				dep->version = trilobite_xml_get_string (child, "version");
 				tmp = trilobite_xml_get_string (child, "sense");
-				dep->sense = eazel_softcat_convert_sense_flags (atoi (tmp));
+				if (atoi (tmp) > 0) {
+					/* old-style numeric "12" sense */
+					dep->sense = eazel_softcat_convert_sense_flags (atoi (tmp));
+				} else {
+					/* new-style symbolic ">=" sense */
+					dep->sense = eazel_softcat_string_to_sense_flags (tmp);
+				}
+				if (dep->sense == 0) {
+					g_warning ("unreadable sense %s from softcat :(", tmp);
+				}
 				g_free (tmp);
 				got_package = TRUE;
 			}
@@ -744,14 +749,21 @@ static void
 osd_parse_file_list (PackageData *pack, xmlNodePtr node)
 {
 	xmlNodePtr child;
-	char *tmp;
+	char *tmp, *filename, *dir;
 
 	child = node->xmlChildrenNode;
 	while (child) {
 		if (g_strcasecmp (child->name, "FILE") == 0) {
 			tmp = xmlNodeGetContent (child);
-			pack->provides = g_list_prepend (pack->provides, g_strdup (tmp));
+			dir = trilobite_xml_get_string (child, "DIRECTORY");
+			if (dir != NULL) {
+				filename = g_strdup_printf ("%s/", tmp);
+				g_free (dir);
+			} else {
+				filename = g_strdup (tmp);
+			}
 			xmlFree (tmp);
+			pack->provides = g_list_prepend (pack->provides, filename);
 		} else {
 			/* bad : thing in file list that isn't a file */
 			trilobite_debug ("XML file list contains %s (not FILE)", child->name);
@@ -781,33 +793,41 @@ osd_parse_feature_list (PackageData *pack, xmlNodePtr node)
 	pack->features = g_list_reverse (pack->features);
 }
 
-static void
+static gboolean
 osd_parse_implementation (PackageData *pack,
 			  xmlNodePtr node)
 {
 	xmlNodePtr child;
+	char *tmp;
 
 	child = node->xmlChildrenNode;
 	while (child) {
 		if (g_strcasecmp (child->name, "PROCESSOR")==0) {
 			pack->archtype = trilobite_xml_get_string (child, "VALUE");
 		} else if (g_strcasecmp (child->name, "OS")==0) {
-			char *dtmp = trilobite_xml_get_string (child, "VALUE");
-			if (dtmp) {
-				pack->distribution.name = trilobite_get_distribution_enum (dtmp,
+			tmp = trilobite_xml_get_string (child, "VALUE");
+			if (tmp) {
+				pack->distribution.name = trilobite_get_distribution_enum (tmp,
 											   TRUE);
-				g_free (dtmp);
+				g_free (tmp);
 			} 
 		} else if (g_strcasecmp (child->name, "CODEBASE")==0) {			
 			char *tmp;
 			pack->remote_url = trilobite_xml_get_string (child, "HREF");
 			tmp = trilobite_xml_get_string (child, "SIZE");
-			if (tmp) {
-				pack->bytesize = atoi (tmp);
+			if (tmp != NULL) {
+				pack->bytesize = strtoul (tmp, NULL, 10);
 				g_free (tmp);
 			} else {
 				pack->bytesize = 0;
 			}
+			tmp = trilobite_xml_get_string (child, "RPM-SIZE");
+			if (tmp != NULL) {
+				pack->filesize = strtoul (tmp, NULL, 10);
+				g_free (tmp);
+			} else {
+				pack->filesize = 0;
+			}
 		} else if (g_strcasecmp (child->name, "FILES") == 0) {
 			/* oh boy... exhaustive file list */
 			osd_parse_file_list (pack, child);
@@ -815,12 +835,24 @@ osd_parse_implementation (PackageData *pack,
 			/* list of "features" (usually shared libs) in the toplevel package */
 			osd_parse_feature_list (pack, child);
 		} else if (g_strcasecmp (child->name, "DEPENDENCY")==0) {
-			osd_parse_dependency (pack, child);
+			tmp = trilobite_xml_get_string (child, "ACTION");
+			if (tmp != NULL) {
+				if (strcasecmp (tmp, "Install") == 0) {
+					osd_parse_dependency (pack, child);
+				} else if (strcasecmp (tmp, "Unresolved") == 0) {
+					g_warning ("Unresolved dependencies!  Get Adam on the bat-phone!");
+					return FALSE;
+				} else {
+					g_warning ("Unknown ACTION: '%s'", tmp);
+				}
+			}
 		} else {
 			/* trilobite_debug ("unparsed tag \"%s\" in IMPLEMENTATION", child->name); */
 		}
 		child = child->next;
 	}
+
+	return TRUE;
 }
 
 static PackageData*
@@ -848,7 +880,10 @@ osd_parse_softpkg (xmlNodePtr softpkg)
 			}
 			xmlFree (tmp);
 		} else if (g_strcasecmp (child->name, "IMPLEMENTATION")==0) {
-			osd_parse_implementation (result, child);
+			if (osd_parse_implementation (result, child) == FALSE) {
+				gtk_object_unref (GTK_OBJECT (result));
+				return NULL;
+			}
 		} else if (g_strcasecmp (child->name, "EAZEL_ID") == 0) {
 			result->eazel_id = trilobite_xml_get_string (child, "VALUE");
 		} else {
@@ -860,13 +895,14 @@ osd_parse_softpkg (xmlNodePtr softpkg)
 	return result;
 }
 
-static GList*
-osd_parse_shared (xmlDocPtr doc)
+static GList *
+osd_parse_shared (xmlDocPtr doc, char **db_revision)
 {
 	GList *result;
 	xmlNodePtr base, child;
 
 	result = NULL;
+	*db_revision = NULL;
 
 	base = doc->root;
 	if (base == NULL) {
@@ -889,6 +925,19 @@ osd_parse_shared (xmlDocPtr doc)
 			} else {
 				trilobite_debug ("SOFTPKG parse failed");
 			}
+		} else if (g_strcasecmp (child->name, "DB_CONTROL") == 0) {
+			if (*db_revision != NULL) {
+				g_warning ("weird: softcat has multiple DB versions simultaneously");
+				g_free (*db_revision);
+			}
+			*db_revision = trilobite_xml_get_string (child, "VALUE");
+			if ((*db_revision != NULL) && (strlen (*db_revision) == 0)) {
+				/* empty revision of "" should just be dropped */
+				g_free (*db_revision);
+				*db_revision = NULL;
+			} else {
+				trilobite_debug ("(softcat db revision %s)", *db_revision);
+			}
 		} else {
 			trilobite_debug ("child is not a SOFTPKG, but a \"%s\"", child->name);
 		}
@@ -900,7 +949,7 @@ osd_parse_shared (xmlDocPtr doc)
 
 /* returns FALSE if the XML was all horked */
 gboolean
-eazel_install_packagelist_parse (GList **list, const char *mem, int size)
+eazel_install_packagelist_parse (GList **list, const char *mem, int size, char **db_revision)
 {
 	xmlDocPtr doc;
 	char *ptr, *docptr, *end;
@@ -934,7 +983,7 @@ eazel_install_packagelist_parse (GList **list, const char *mem, int size)
 		return FALSE;
 	}
 
-	*list = osd_parse_shared (doc);	
+	*list = osd_parse_shared (doc, db_revision);
 
 	xmlFreeDoc (doc);
 	g_free (docptr);
