@@ -51,6 +51,19 @@ eazel_softcat_finalize (GtkObject *object)
 
 	softcat = EAZEL_SOFTCAT (object);
 
+	g_free (softcat->private->server);
+	softcat->private->server = NULL;
+	g_free (softcat->private->server_str);
+	softcat->private->server_str = NULL;
+	g_free (softcat->private->cgi_path);
+	softcat->private->cgi_path = NULL;
+	g_free (softcat->private->username);
+	softcat->private->username = NULL;
+	g_free (softcat->private->db_revision);
+	softcat->private->db_revision = NULL;
+
+	g_free (softcat->private);
+
 	if (GTK_OBJECT_CLASS (eazel_softcat_parent_class)->finalize) {
 		GTK_OBJECT_CLASS (eazel_softcat_parent_class)->finalize (object);
 	}
@@ -106,13 +119,15 @@ eazel_softcat_class_initialize (EazelSoftCatClass *klass)
 }
 
 static void
-eazel_softcat_initialize (EazelSoftCat *softcat) {
+eazel_softcat_initialize (EazelSoftCat *softcat)
+{
 	g_assert (softcat != NULL);
 	g_assert (EAZEL_IS_SOFTCAT (softcat));
 
 	softcat->private = g_new0 (EazelSoftCatPrivate, 1);
 	softcat->private->retries = 3;
 	softcat->private->delay = 100;
+	softcat->private->db_revision = NULL;
 }
 
 GtkType
@@ -277,6 +292,13 @@ eazel_softcat_set_retry (EazelSoftCat *softcat, unsigned int retries, unsigned i
 	softcat->private->delay = delay_us;
 }
 
+void 
+eazel_softcat_reset_server_update_flag (EazelSoftCat *softcat)
+{
+	g_free (softcat->private->db_revision);
+	softcat->private->db_revision = NULL;
+}
+
 const char *
 eazel_softcat_error_string (EazelSoftCatError err)
 {
@@ -287,6 +309,10 @@ eazel_softcat_error_string (EazelSoftCatError err)
 		return "internal error";
 	case EAZEL_SOFTCAT_ERROR_SERVER_UNREACHABLE:
 		return "softcat server is unreachable";
+	case EAZEL_SOFTCAT_ERROR_MULTIPLE_RESPONSES:
+		return "softcat server returned multiple responses to a single-package query";
+	case EAZEL_SOFTCAT_ERROR_SERVER_UPDATED:
+		return "softcat has been updated since last request";
 	case EAZEL_SOFTCAT_ERROR_NO_SUCH_PACKAGE:
 		return "no such package";
 	}
@@ -298,8 +324,9 @@ eazel_softcat_error_string (EazelSoftCatError err)
   actual real implementation stuff
 *****************************************/
 
-/* can be OR'd together for "greater than or equal" etc -- this happens often */
-/* --- private to me.  everyone else should use the real sense flags in eazel-softcat.h */
+/* can be OR'd together for "greater than or equal" etc -- this happens often. */
+/* --- private to me.  everyone else should use the real sense flags in eazel-softcat.h. */
+/* these are the numbers that the softcat server uses. */
 typedef enum {
 	SOFTCAT_SENSE_FLAG_LESS = 2,
 	SOFTCAT_SENSE_FLAG_GREATER = 4,
@@ -361,6 +388,31 @@ eazel_softcat_sense_flags_to_string (EazelSoftCatSense flags)
 	return out;
 }
 
+EazelSoftCatSense
+eazel_softcat_string_to_sense_flags (const char *str)
+{
+	EazelSoftCatSense out = 0;
+	const char *p;
+
+	for (p = str; *p != '\0'; p++) {
+		switch (*p) {
+		case '<':
+			out |= EAZEL_SOFTCAT_SENSE_LT;
+			break;
+		case '>':
+			out |= EAZEL_SOFTCAT_SENSE_GT;
+			break;
+		case '=':
+			out |= EAZEL_SOFTCAT_SENSE_EQ;
+			break;
+		default:
+			/* ignore */
+			break;
+		}
+	}
+	return out;
+}
+
 #ifdef EAZEL_INSTALL_SLIM
 /* wow, i had no idea all these chars were evil.  they must be stopped! */
 static char _bad[] = {
@@ -446,6 +498,15 @@ get_search_url_for_package (EazelSoftCat *softcat, const PackageData *package, i
 	char *dist_name;
 	char *url_str;
 
+	g_assert (package != NULL);
+
+	/* bail out early if there's not enough info to go with */
+	if ((package->eazel_id == NULL) && (package->suite_id == NULL) && (package->name == NULL) &&
+	    (package->provides == NULL)) {
+		trilobite_debug ("softcat: no search url for completely-empty package");
+		return NULL;
+	}
+
 	verify_softcat_fields (softcat);
 	dist = trilobite_get_distribution ();
 
@@ -470,6 +531,25 @@ get_search_url_for_package (EazelSoftCat *softcat, const PackageData *package, i
 		add_to_url (url, "?rpm_id=", package->eazel_id);
 		add_to_url (url, "&arch=", arch);
 		g_free (arch);
+	} else if (package->suite_id != NULL) {
+		/* find by suite-id! */
+		/* this devolves into several different cases.  softcat cares
+		 * about the differences between them, but we don't.
+		 */
+		if (package->suite_id[0] == 'P') {
+			add_to_url (url, "?product_id=", package->suite_id+2);
+		} else if (package->suite_id[0] == 'S') {
+			add_to_url (url, "?suite_id=", package->suite_id+2);
+		} else if (package->suite_id[0] == 'N') {
+			add_to_url (url, "?product_name=", package->suite_id+2);
+		} else if (package->suite_id[0] == 'X') {
+			add_to_url (url, "?suite_name=", package->suite_id+2);
+		} else {
+			g_assert_not_reached ();
+		}
+		arch = trilobite_get_distribution_arch ();
+		add_to_url (url, "&arch=", arch);
+		g_free (arch);
 	} else if (package->name == NULL) {
 		/* find by provides list! */
 		g_assert ((package->provides != NULL) && (g_list_length (package->provides) > 0));
@@ -507,34 +587,23 @@ get_search_url_for_package (EazelSoftCat *softcat, const PackageData *package, i
 }
 
 
-static gboolean
-is_filename_probably_a_directory (const char *filename, const GList *provides)
-{
-	const GList *iter;
-	gboolean is_dir = FALSE;
-
-	for (iter = g_list_first ((GList *)provides); iter != NULL; iter = g_list_next (iter)) {
-		const char *filename2 = (const char *)(iter->data);
-		/* substring match is not sufficient (think of "libfoo.so.0" & "libfoo.so.0.0") */
-		if ((strlen (filename2) > strlen (filename)) &&
-		    (strncmp (filename, filename2, strlen (filename)) == 0) &&
-		    (filename2[strlen (filename)] == '/')) {
-			is_dir = TRUE;
-			break;
-		}
-	}
-	return is_dir;
-}
+/* directories will end with '/' */
+/* TEMPORARY FIXME: to work around a very odd bug in softcat, throw away duplicate filenames */
 
 static void
 remove_directories_from_provides_list (PackageData *pack)
 {
 	GList *iter, *next_iter;
+	GList *newlist;
+	char *filename;
 
+	newlist = NULL;
 	for (iter = g_list_first (pack->provides); iter != NULL; ) {
-		if (is_filename_probably_a_directory ((char *)(iter->data), pack->provides)) {
-			next_iter = iter->prev;
+		filename = (char *)(iter->data);
 
+		if ((filename != NULL) && (filename[0] != '\0') &&
+		    (filename[strlen (filename)-1] == '/')) {
+			next_iter = iter->prev;
 			g_free (iter->data);
 			pack->provides = g_list_remove (pack->provides, iter->data);
 			iter = next_iter;
@@ -542,15 +611,21 @@ remove_directories_from_provides_list (PackageData *pack)
 				iter = g_list_first (pack->provides);
 			}
 		} else {
+			if (g_list_find_custom (newlist, filename, (GCompareFunc)strcmp) == NULL) {
+				newlist = g_list_prepend (newlist, g_strdup (filename));
+			}
 			iter = g_list_next (iter);
 		}
 	}
+
+	/* replace old pack->provides with newlist */
+	g_list_foreach (pack->provides, (GFunc)g_free, NULL);
+	g_list_free (pack->provides);
+	pack->provides = newlist;
 }
 
-/* Given a partially filled packagedata object, 
-   check softcat, and fill it with the desired info */
 EazelSoftCatError
-eazel_softcat_get_info (EazelSoftCat *softcat, PackageData *package, int sense_flags, int fill_flags)
+eazel_softcat_query (EazelSoftCat *softcat, PackageData *package, int sense_flags, int fill_flags, GList **result)
 {
 	char *search_url;
 	char *body = NULL;
@@ -558,8 +633,10 @@ eazel_softcat_get_info (EazelSoftCat *softcat, PackageData *package, int sense_f
 	int tries_left;
 	gboolean got_happy;
 	GList *packages;
-	PackageData *full_package;
+	char *db_revision;
+	int err;
 
+	db_revision = NULL;
 	search_url = get_search_url_for_package (softcat, package, sense_flags);
 	if (search_url == NULL) {
 		trilobite_debug ("no search url :(");
@@ -574,7 +651,7 @@ eazel_softcat_get_info (EazelSoftCat *softcat, PackageData *package, int sense_f
 	     tries_left--) {
 		got_happy = trilobite_fetch_uri (search_url, &body, &length);
 		if (got_happy) {
-			got_happy = eazel_install_packagelist_parse (&packages, body, length);
+			got_happy = eazel_install_packagelist_parse (&packages, body, length, &db_revision);
 			if (! got_happy) {
 				/* boo.  bogus xml.  long live softcat! */
 				trilobite_debug ("bogus xml.");
@@ -591,6 +668,8 @@ eazel_softcat_get_info (EazelSoftCat *softcat, PackageData *package, int sense_f
 	if (! got_happy) {
 		if (package->eazel_id != NULL) {
 			g_warning ("couldn't fetch info about package id %s", package->eazel_id);
+		} else if (package->suite_id != NULL) {
+			g_warning ("couldn't fetch info about suite id %s", package->suite_id);
 		} else if (package->name != NULL) {
 			g_warning ("couldn't fetch info about package '%s'", package->name);
 		} else if ((package->provides != NULL) && (package->provides->data != NULL)) {
@@ -603,46 +682,132 @@ eazel_softcat_get_info (EazelSoftCat *softcat, PackageData *package, int sense_f
 		return EAZEL_SOFTCAT_ERROR_SERVER_UNREACHABLE;
 	}
 
+	if ((db_revision != NULL) && (softcat->private->db_revision == NULL)) {
+		softcat->private->db_revision = db_revision;
+		db_revision = NULL;
+	} else if (db_revision != NULL) {
+		if (strcmp (softcat->private->db_revision, db_revision) != 0) {
+			g_warning ("SoftCat has been updated since last request!");
+			err = EAZEL_SOFTCAT_ERROR_SERVER_UPDATED;
+			goto out;
+		}
+	}
+
 	if (g_list_length (packages) == 0) {
 		trilobite_debug ("no matches for that package.");
-		g_free (body);
-		g_free (search_url);
-		return EAZEL_SOFTCAT_ERROR_NO_SUCH_PACKAGE;
+		err = EAZEL_SOFTCAT_ERROR_NO_SUCH_PACKAGE;
+		goto out;
+	}
+
+	trilobite_debug ("package info ok.");
+	*result = packages;
+	err = EAZEL_SOFTCAT_SUCCESS;
+
+out:
+	g_free (body);
+	g_free (search_url);
+	g_free (db_revision);
+
+	return err;
+}
+
+/* Given a partially filled packagedata object, 
+   check softcat, and fill it with the desired info */
+EazelSoftCatError
+eazel_softcat_get_info (EazelSoftCat *softcat, PackageData *package, int sense_flags, int fill_flags)
+{
+	GList *packages;
+	PackageData *full_package;
+	EazelSoftCatError err;
+
+	err = eazel_softcat_query (softcat, package, sense_flags, fill_flags, &packages);
+	if (err != EAZEL_SOFTCAT_SUCCESS) {
+		return err;
 	}
 
 	if (g_list_length (packages) > 1) {
-		trilobite_debug ("more than one match!  using the first one.");
+		if (package->suite_id) {
+			/* More than one package returned and we queried on a suite Id.
+			   Make deps and put into "package", remember to strip dirs in 
+			   provides if needed */
+			GList *iterator;
+
+			trilobite_debug ("softcat query returned suite with %d elements", 
+					 g_list_length (packages));
+			for (iterator = packages; iterator; iterator = g_list_next (iterator)) {
+				PackageData *pack = PACKAGEDATA (iterator->data);
+				PackageDependency *dep = g_new0 (PackageDependency, 1);
+
+				if (fill_flags & PACKAGE_FILL_NO_DIRS_IN_PROVIDES) {
+					remove_directories_from_provides_list (pack);
+				}
+				gtk_object_ref (GTK_OBJECT (pack));
+				pack->fillflag = fill_flags;
+
+				dep->package = pack;
+				dep->version = g_strdup (pack->version);
+
+				if (dep->version) {
+					/* FIXME: should a suite be EQ or GE ? If GE, any newer version
+					   that's already installed will be ok, if EQ, the suites depends
+					   on an exact version */
+					dep->sense = EAZEL_SOFTCAT_SENSE_GE;
+				} else {
+					dep->sense = EAZEL_SOFTCAT_SENSE_ANY;
+				}
+
+				packagedata_add_pack_to_depends (package, dep);
+			}
+		} else {
+			g_warning ("softcat query returned %d results!", g_list_length (packages));
+			err = EAZEL_SOFTCAT_ERROR_MULTIPLE_RESPONSES;
+			g_list_foreach (packages, (GFunc)gtk_object_unref, NULL);
+			g_list_free (packages);
+			return err;
+		}
+	} else {
+		full_package = PACKAGEDATA (packages->data);
+		packagedata_fill_in_missing (package, full_package, fill_flags);
+		if (fill_flags & PACKAGE_FILL_NO_DIRS_IN_PROVIDES) {
+			remove_directories_from_provides_list (package);
+		}
 	}
-
-	full_package = (PackageData *) packages->data;
-	packagedata_fill_in_missing (package, full_package, fill_flags);
-	remove_directories_from_provides_list (package);
-
-	g_list_foreach (packages, (GFunc)packagedata_destroy, GINT_TO_POINTER (TRUE));
+	g_list_foreach (packages, (GFunc)gtk_object_unref, NULL);
 	g_list_free (packages);
-	g_free (body);
-	g_free (search_url);
-	trilobite_debug ("package info ok.");
-
-	return EAZEL_SOFTCAT_SUCCESS;
+	return err;
 }
 
 /* Check if there's a newer version in SoftCat.
  * Returns TRUE and fills in 'newpack' if there is, returns FALSE otherwise.
  */
 gboolean
-eazel_softcat_available_update (EazelSoftCat *softcat, PackageData *oldpack, PackageData **newpack, int fill_flags)
+eazel_softcat_available_update (EazelSoftCat *softcat, 
+				PackageData *oldpack, 
+				PackageData **newpack, 
+				int fill_flags)
 {
-	*newpack = packagedata_new ();
-	(*newpack)->name = g_strdup (oldpack->name);
-	(*newpack)->version = g_strdup (oldpack->version);
-	(*newpack)->distribution = oldpack->distribution;
-	(*newpack)->archtype = g_strdup (oldpack->archtype);
+	PackageData *tmp_pack;
+	gboolean result = TRUE;
 
-	if (eazel_softcat_get_info (softcat, *newpack, EAZEL_SOFTCAT_SENSE_GT, fill_flags) != EAZEL_SOFTCAT_SUCCESS) {
-		packagedata_destroy (*newpack, TRUE);
-		*newpack = NULL;
-		return FALSE;
+	tmp_pack = packagedata_new ();
+	tmp_pack->name = g_strdup (oldpack->name);
+	tmp_pack->version = g_strdup (oldpack->version);
+	tmp_pack->distribution = oldpack->distribution;
+	tmp_pack->archtype = g_strdup (oldpack->archtype);
+
+	if (eazel_softcat_get_info (softcat, tmp_pack, EAZEL_SOFTCAT_SENSE_GT, fill_flags) != EAZEL_SOFTCAT_SUCCESS) {
+		result = FALSE;
 	}
-	return TRUE;
+
+	if (newpack!=NULL && result==TRUE) {
+		(*newpack) = tmp_pack;
+	} else {
+		gtk_object_unref (GTK_OBJECT (tmp_pack));
+		/* Null in case it's givin */
+		if (newpack != NULL) {
+			(*newpack) = NULL;
+		}
+	}
+
+	return result;
 }
