@@ -79,7 +79,7 @@ header_destroy (GtkObject *object)
 	g_free (header->details);
 	
 	/* Chain destroy */
-	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
+	NAUTILUS_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
 /* EazelServicesHeader public methods */
