@@ -46,21 +46,23 @@
 #include <libnautilus-extensions/nautilus-string.h>
 #include <libnautilus-extensions/nautilus-font-factory.h>
 #include <libnautilus-extensions/nautilus-stock-dialogs.h>
+#include <libnautilus-extensions/nautilus-xml-extensions.h>
 
 #include <gnome.h>
+#include <gtk/gtkeventbox.h>
 #include <libgnomevfs/gnome-vfs.h>
-#include <libgnorba/gnorba.h>
-#include <limits.h>
-
 #include <ghttp.h>
 
+#include <limits.h>
+
 #define MAX_SERVICE_ITEMS 32
 #define TEXT_VIEW_CHUNK_SIZE 65536
 #define MAX_FILE_SIZE	1024*1024
+#define MAX_SELECTION_SIZE 256
 
-struct _NautilusTextViewDetails {
+struct NautilusTextViewDetails {
 	NautilusFile *file;
-	NautilusView *nautilus_view;
+        GtkWidget *event_box;
 	GnomeVFSAsyncHandle *file_handle;	
 	char *buffer;
 	
@@ -116,9 +118,13 @@ static void zoomable_zoom_out_callback				(BonoboZoomable       *zoomable,
 static void zoomable_zoom_to_fit_callback			(BonoboZoomable       *zoomable,
 								 NautilusTextView      *directory_view);
 
+static void nautilus_text_view_load_uri                         (NautilusTextView *view,
+                                                                 const char *uri);
+
+
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusTextView,
                                    nautilus_text_view,
-                                   GTK_TYPE_EVENT_BOX)
+                                   NAUTILUS_TYPE_VIEW)
 
 static float text_view_preferred_zoom_levels[] = { .25, .50, .75, 1.0, 1.5, 2.0, 4.0 };
 static int   text_view_preferred_font_sizes[] = { 9, 10, 12, 14, 18, 24, 36 };
@@ -130,10 +136,8 @@ static void
 nautilus_text_view_initialize_class (NautilusTextViewClass *klass)
 {
 	GtkObjectClass *object_class;
-	GtkWidgetClass *widget_class;
 	
 	object_class = GTK_OBJECT_CLASS (klass);
-	widget_class = GTK_WIDGET_CLASS (klass);
 	
 	object_class->destroy = nautilus_text_view_destroy;
 }
@@ -147,7 +151,9 @@ nautilus_text_view_initialize (NautilusTextView *text_view)
 	
 	text_view->details = g_new0 (NautilusTextViewDetails, 1);
 
-	text_view->details->nautilus_view = nautilus_view_new (GTK_WIDGET (text_view));
+        text_view->details->event_box = gtk_event_box_new ();
+
+        nautilus_view_construct (NAUTILUS_VIEW (text_view), text_view->details->event_box);
 
 	/* set up the zoomable interface */
 	text_view->details->zoomable = bonobo_zoomable_new ();
@@ -167,23 +173,24 @@ nautilus_text_view_initialize (NautilusTextView *text_view)
 					     text_view_preferred_zoom_levels, NULL,
 					     NAUTILUS_N_ELEMENTS (text_view_preferred_zoom_levels));
 	
-	bonobo_object_add_interface (BONOBO_OBJECT (text_view->details->nautilus_view),
+	bonobo_object_add_interface (BONOBO_OBJECT (text_view),
 				     BONOBO_OBJECT (text_view->details->zoomable));
  
     	
-	gtk_signal_connect (GTK_OBJECT (text_view->details->nautilus_view), 
+	gtk_signal_connect (GTK_OBJECT (text_view), 
 			    "load_location",
 			    text_view_load_location_callback, 
 			    text_view);
 			    	
+        /* FIXME: eventually, get this from preferences */	
 	/* set up the default font */
-	text_view->details->font_name = g_strdup ("helvetica"); /* eventually, get this from preferences */	
+	text_view->details->font_name = g_strdup ("helvetica"); 
 
 	/* allocate a vbox to contain the text widget */
 	
 	text_view->details->container = gtk_vbox_new (FALSE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (text_view->details->container), 0);
-	gtk_container_add (GTK_CONTAINER (text_view), GTK_WIDGET (text_view->details->container));
+	gtk_container_add (GTK_CONTAINER (text_view->details->event_box), GTK_WIDGET (text_view->details->container));
 	
 	/* allocate the text object */
 	text_view->details->text_display = gtk_text_new (NULL, NULL);
@@ -211,13 +218,13 @@ nautilus_text_view_initialize (NautilusTextView *text_view)
 
 	/* get notified when we are activated so we can merge in our menu items */
         gtk_signal_connect (GTK_OBJECT (nautilus_view_get_bonobo_control
-					(text_view->details->nautilus_view)),
+					(NAUTILUS_VIEW (text_view))),
                             "activate",
                             merge_bonobo_menu_items,
                             text_view);
 		 	
 	/* finally, we can show everything */	
-	gtk_widget_show_all (GTK_WIDGET (text_view));
+	gtk_widget_show_all (GTK_WIDGET (text_view->details->event_box));
 }
 
 static void
@@ -247,15 +254,15 @@ nautilus_text_view_destroy (GtkObject *object)
 	 
 	g_free (text_view->details);
 
-	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
+	NAUTILUS_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
-
-/* Component embedding support */
-NautilusView *
-nautilus_text_view_get_nautilus_view (NautilusTextView *text_view)
+/* here's a callback for the async close, which does nothing */
+static void
+file_close_callback (GnomeVFSAsyncHandle *handle,
+			  GnomeVFSResult result,
+			  gpointer callback_data)
 {
-	return text_view->details->nautilus_view;
 }
 
 /* this routine is called when we're finished reading to deallocate the buffer and
@@ -268,6 +275,12 @@ done_file_read (NautilusTextView *text_view, gboolean succeeded)
 		g_free (text_view->details->buffer);
 		text_view->details->buffer = NULL;		
 	}
+	
+	if (text_view->details->file_handle != NULL) {
+		gnome_vfs_async_close (text_view->details->file_handle,
+				       file_close_callback,
+				       NULL);
+	}
 }
  
 /* this callback handles the next chunk of data from the file by copying it into the
@@ -314,7 +327,7 @@ file_read_callback (GnomeVFSAsyncHandle *vfs_handle,
 		} else {
 			char *filename = nautilus_file_get_name(text_view->details->file);
 			char *message = g_strdup_printf (_("Sorry, but %s is too large for Nautilus to load all of it."), filename);
-			nautilus_error_dialog (message, _("File too large"), NULL);
+			nautilus_show_error_dialog (message, _("File too large"), NULL);
 		
 			g_free (filename);
 			g_free (message);
@@ -383,8 +396,9 @@ nautilus_text_view_update (NautilusTextView *text_view)
 }
 
 
-void
-nautilus_text_view_load_uri (NautilusTextView *text_view, const char *uri)
+static void
+nautilus_text_view_load_uri (NautilusTextView *text_view, 
+                             const char *uri)
 {
  
 	detach_file (text_view);
@@ -398,9 +412,9 @@ text_view_load_location_callback (NautilusView *view,
                                    const char *location,
                                    NautilusTextView *text_view)
 {
-	nautilus_view_report_load_underway (text_view->details->nautilus_view);
+	nautilus_view_report_load_underway (NAUTILUS_VIEW (text_view));
 	nautilus_text_view_load_uri (text_view, location);
-	nautilus_view_report_load_complete (text_view->details->nautilus_view);
+	nautilus_view_report_load_complete (NAUTILUS_VIEW (text_view));
 }
 
 /* update the font and redraw */
@@ -453,7 +467,6 @@ handle_service_menu_item (BonoboUIComponent *ui, gpointer user_data, const char
 			g_free (mapped_text);
 			
 			/* load the resultant page through gnome-vfs */
-
 			if (nautilus_read_entire_file (uri, &text_size, &text_ptr) == GNOME_VFS_OK) {
  				gtk_editable_delete_text (GTK_EDITABLE (parameters->text_view->details->text_display), 0, -1);   
 				gtk_text_insert (GTK_TEXT (parameters->text_view->details->text_display),
@@ -468,14 +481,18 @@ handle_service_menu_item (BonoboUIComponent *ui, gpointer user_data, const char
 		g_free (selected_text);
 	} else {
 		selected_text = get_selected_text (GTK_EDITABLE (parameters->text_view->details->text_display));
-	
 		if (selected_text != NULL) {
+			/* truncate the selection to a reasonable size */
+			if (strlen (selected_text) > MAX_SELECTION_SIZE) {
+				selected_text [MAX_SELECTION_SIZE] = '\0';
+			}
+			
 			/* formulate the url */
 			mapped_text = gnome_vfs_escape_string (selected_text);
 			uri = g_strdup_printf (parameters->service_template, mapped_text);
 			
 			/* goto the url */	
-			nautilus_view_open_location_in_this_window (parameters->text_view->details->nautilus_view, uri);
+			nautilus_view_open_location_in_this_window (NAUTILUS_VIEW (parameters->text_view), uri);
 
 			g_free (uri);
 			g_free (selected_text);
@@ -559,9 +576,9 @@ add_one_service (NautilusTextView *text_view, BonoboControl *control, const char
 		service_node = xmlDocGetRootElement (service_definition);
 	
 		/* extract the label and template */
-		label = xmlGetProp (service_node, "label");
+		label = nautilus_xml_get_property_translated (service_node, "label");	
+		tooltip = nautilus_xml_get_property_translated (service_node, "tooltip");	
 		template = xmlGetProp (service_node, "template");
-		tooltip = xmlGetProp (service_node, "tooltip");
 		source_mode = xmlGetProp (service_node, "source");
 		
 		if (label != NULL && template != NULL) {
@@ -616,12 +633,18 @@ add_one_service (NautilusTextView *text_view, BonoboControl *control, const char
 	}
 }
 
-/* iterate through the passed in services directory */
-static void
-add_services_to_menu (NautilusTextView *text_view, BonoboControl *control, const char *services_directory, int* index)
+/* iterate through the passed in services directory to add services to the menu.
+ * Don't add a service with the same name as one that has already been added.
+ */
+static GList *
+add_services_to_menu (NautilusTextView *text_view,
+			BonoboControl *control,
+			const char *services_directory,
+			GList *added_services,
+			int* index)
 {
 	GnomeVFSResult result;
-	GnomeVFSDirectoryList *file_list;	
+	GList *file_list, *element;	
 	GnomeVFSFileInfo *current_file_info;
 	char *services_uri, *service_xml_path;
 	
@@ -630,28 +653,31 @@ add_services_to_menu (NautilusTextView *text_view, BonoboControl *control, const
 	
 	if (result != GNOME_VFS_OK) {
 		g_free (services_uri);
-		return;
+		return added_services;
 	}
 
 	/* iterate through the directory */
-	current_file_info = gnome_vfs_directory_list_first (file_list);
-	while (current_file_info != NULL) {
+	for (element = file_list; element != NULL; element = element->next) {
+		current_file_info = element->data;
+
 		if (*index >= MAX_SERVICE_ITEMS) {
 			break;
 		}
 		
 		if (nautilus_istr_has_suffix (current_file_info->name, ".xml")) {		
-			service_xml_path = nautilus_make_path (services_directory, current_file_info->name);
-			add_one_service (text_view, control, service_xml_path, index);
-			g_free (service_xml_path);
+			if (g_list_find_custom (added_services, current_file_info->name,
+						(GCompareFunc) strcmp) == NULL) {
+				service_xml_path = nautilus_make_path (services_directory, current_file_info->name);
+				add_one_service (text_view, control, service_xml_path, index);
+				added_services = g_list_prepend (added_services, g_strdup (current_file_info->name));
+				g_free (service_xml_path);
+			}
 		}
-			
-		current_file_info = gnome_vfs_directory_list_next (file_list);
 	}
 	
 	g_free (services_uri);
-	gnome_vfs_directory_list_destroy (file_list);	
-
+	gnome_vfs_file_info_list_free (file_list);	
+	return added_services;
 }
 
 /* build the services menu from the shared and private services/text directories */
@@ -659,25 +685,33 @@ static void
 nautilus_text_view_build_service_menu (NautilusTextView *text_view, BonoboControl *control)
 {
 	char *services_directory, *user_directory, *nautilus_datadir;
+	GList *added_services;
 	int index;
-	
 	index = 0;
 	
 	/* first, add the services from the global directory */
 	nautilus_datadir = nautilus_make_path (DATADIR, "nautilus");
 	services_directory = nautilus_make_path (nautilus_datadir, "services/text");
-	add_services_to_menu (text_view, control, services_directory, &index);
+	added_services = add_services_to_menu (text_view, control, services_directory, NULL, &index);
 	g_free (nautilus_datadir);
 	g_free (services_directory);
 	
 	/* now add services from the user-specific directory, if any */	
 	user_directory = nautilus_get_user_directory ();
 	services_directory = nautilus_make_path (user_directory, "services/text");
-	add_services_to_menu (text_view, control, services_directory, &index);
+	added_services = add_services_to_menu (text_view, control, services_directory, added_services, &index);
 
+	/* add any services that are present in the updates folder */
+	g_free (services_directory);
+	g_free (user_directory);
+	user_directory = nautilus_get_user_directory ();
+	services_directory = nautilus_make_path (user_directory, "updates/services/text");
+	added_services = add_services_to_menu (text_view, control, services_directory, added_services, &index);
+	
 	text_view->details->service_item_count = index;	
 	g_free (services_directory);
 	g_free (user_directory);
+	nautilus_g_list_free_deep (added_services);
 }
 
 /* handle updating the service menu items according to the selection state of the text display */
@@ -693,7 +727,7 @@ update_service_menu_items (GtkWidget *widget, GdkEventButton *event, gpointer *u
 	int index;
 	
 	text_view = NAUTILUS_TEXT_VIEW (user_data);
-	control = nautilus_view_get_bonobo_control (text_view->details->nautilus_view);	
+	control = nautilus_view_get_bonobo_control (NAUTILUS_VIEW (text_view));	
 	ui = bonobo_control_get_ui_component (control);
 	
 	/* determine if there is a selection */
@@ -730,7 +764,7 @@ merge_bonobo_menu_items (BonoboControl *control, gboolean state, gpointer user_d
 	text_view = NAUTILUS_TEXT_VIEW (user_data);
 
 	if (state) {
-		nautilus_view_set_up_ui (text_view->details->nautilus_view,
+		nautilus_view_set_up_ui (NAUTILUS_VIEW (text_view),
 				         DATADIR,
 					 "nautilus-text-view-ui.xml",
 					 "nautilus-text-view");
