@@ -135,7 +135,7 @@ get_bonobo_properties (BonoboPropertyBag *bag,
 
 		case LOCATION:
 		{
-			char *location = nautilus_theme_get_theme_data ("throbber", "URL");
+			char *location = nautilus_theme_get_theme_data ("throbber", "url");
 			if (location != NULL) {
 				BONOBO_ARG_SET_STRING (arg, location);
 				g_free (location);
@@ -200,14 +200,9 @@ nautilus_throbber_destroy (GtkObject *object)
 		bonobo_object_unref (BONOBO_OBJECT (throbber->details->property_bag));
 	}
 	
-	if (throbber->details->control) {
-		bonobo_object_unref (BONOBO_OBJECT (throbber->details->control));
-	}
-
-
 	g_free (throbber->details);
 
-	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
+	NAUTILUS_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
 BonoboObject*
@@ -279,7 +274,7 @@ nautilus_throbber_initialize (NautilusThrobber *throbber)
 	gtk_container_add (GTK_CONTAINER (box), widget);
 
 	/* set up the delay from the theme */
-	delay_str = nautilus_theme_get_theme_data ("throbber", "DELAY");
+	delay_str = nautilus_theme_get_theme_data ("throbber", "delay");
 	
 	if (delay_str) {
 		throbber->details->delay = atoi (delay_str);
@@ -292,8 +287,10 @@ nautilus_throbber_initialize (NautilusThrobber *throbber)
 	throbber->details->control = (BonoboObject*) bonobo_control_new (box);
 	
 	/* attach a property bag with the configure property */
-	throbber->details->property_bag = bonobo_property_bag_new (get_bonobo_properties, set_bonobo_properties, throbber);
-	bonobo_control_set_properties (BONOBO_CONTROL(throbber->details->control), throbber->details->property_bag);
+	throbber->details->property_bag = bonobo_property_bag_new (get_bonobo_properties, 
+								   set_bonobo_properties, throbber);
+	bonobo_control_set_properties (BONOBO_CONTROL (throbber->details->control), 
+				       throbber->details->property_bag);
 	
 	bonobo_property_bag_add (throbber->details->property_bag, "throbbing", THROBBING, BONOBO_ARG_BOOLEAN, NULL,
 				 "Throbber active", 0);
@@ -412,7 +409,7 @@ nautilus_throbber_map (GtkWidget *widget)
 	
 	throbber = NAUTILUS_THROBBER (widget);
 	
-	NAUTILUS_CALL_PARENT_CLASS (GTK_WIDGET_CLASS, map, (widget));
+	NAUTILUS_CALL_PARENT (GTK_WIDGET_CLASS, map, (widget));
 	throbber->details->ready = TRUE;
 }
 
@@ -548,12 +545,12 @@ nautilus_throbber_load_images (NautilusThrobber *throbber)
 	
 	nautilus_throbber_unload_images (throbber);
 
-	image_theme = nautilus_theme_get_theme_data ("throbber", "IMAGE_THEME");
+	image_theme = nautilus_theme_get_theme_data ("throbber", "image_theme");
 	throbber->details->quiescent_pixbuf = load_themed_image ("throbber/rest.png", image_theme, throbber->details->small_mode);
 
 	/* images are of the form throbber/001.png, 002.png, etc, so load them into a list */
 
-	frames = nautilus_theme_get_theme_data ("throbber", "FRAME_COUNT");
+	frames = nautilus_theme_get_theme_data ("throbber", "frame_count");
 	if (frames != NULL) {
 		throbber->details->max_frame = atoi (frames);
 		g_free (frames);
@@ -586,7 +583,7 @@ nautilus_throbber_button_press_event (GtkWidget *widget, GdkEventButton *event)
 	BonoboArg *location_arg;
 	
 	throbber = NAUTILUS_THROBBER (widget);
-	location = nautilus_theme_get_theme_data ("throbber", "URL");
+	location = nautilus_theme_get_theme_data ("throbber", "url");
 	if (location != NULL) {
 		location_arg = bonobo_arg_new (BONOBO_ARG_STRING);
 		BONOBO_ARG_SET_STRING (location_arg, location);			
@@ -620,7 +617,7 @@ nautilus_throbber_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 	int throbber_width, throbber_height;
 	NautilusThrobber *throbber = NAUTILUS_THROBBER (widget);
 
-	NAUTILUS_CALL_PARENT_CLASS (GTK_WIDGET_CLASS, size_allocate, (widget, allocation));
+	NAUTILUS_CALL_PARENT (GTK_WIDGET_CLASS, size_allocate, (widget, allocation));
 
 	get_throbber_dimensions (throbber, &throbber_width, &throbber_height);
 	
