@@ -37,6 +37,7 @@
 #include <libnautilus-extensions/nautilus-gtk-macros.h>
 #include <libnautilus-extensions/nautilus-glib-extensions.h>
 #include <libnautilus-extensions/nautilus-file.h>
+#include <libnautilus-extensions/nautilus-file-attributes.h>
 #include <libnautilus-extensions/nautilus-file-utilities.h>
 #include <libnautilus-extensions/nautilus-string.h>
 #include <libnautilus-extensions/nautilus-icon-factory.h>
@@ -46,7 +47,6 @@
 
 #include <stdio.h>
 
-
 #define DISPLAY_TIMEOUT_INTERVAL_MSECS 500
 
 
@@ -57,9 +57,6 @@ static void         notify_done_loading    (NautilusTreeView *view,
 static void         notify_node_seen       (NautilusTreeView *view, 
 					    NautilusTreeNode *node);
 
-static NautilusCTreeNode *nautilus_tree_view_find_parent_node (NautilusTreeView *view, 
-							       NautilusFile     *file);
-
 static gboolean           ctree_is_node_expanded              (NautilusCTree     *ctree,
 							       NautilusCTreeNode *node);
 static NautilusCTreeNode *file_to_view_node                   (NautilusTreeView  *view,
@@ -93,18 +90,28 @@ static void tree_select_row_callback            (NautilusCTree         *tree,
 static void size_allocate_callback              (NautilusCTree         *tree,
 						 GtkAllocation         *allocation,
 						 gpointer               data);
-
-
 static void nautilus_tree_view_load_uri         (NautilusTreeView      *view,
 						 const char            *uri);
-static void nautilus_tree_view_update_all_icons (NautilusTreeView *view);
+static void nautilus_tree_view_update_all_icons (NautilusTreeView      *view);
 
+static void got_activation_uri_callback         (NautilusFile          *file,
+						 gpointer               callback_data);
+static void cancel_possible_activation          (NautilusTreeView      *view);
 
+static void nautilus_tree_view_update_model_node (NautilusTreeView     *view,
+						  NautilusTreeNode     *node);
 
 static void nautilus_tree_view_initialize_class (NautilusTreeViewClass *klass);
 static void nautilus_tree_view_initialize       (NautilusTreeView      *view);
 static void nautilus_tree_view_destroy          (GtkObject             *object);
 
+static void register_unparented_node		(NautilusTreeView      *view,
+						 NautilusTreeNode      *node);
+static void forget_unparented_node		(NautilusTreeView      *view,
+						 NautilusTreeNode      *node);
+static void insert_unparented_nodes		(NautilusTreeView      *view,
+						 NautilusTreeNode      *node);
+
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusTreeView, nautilus_tree_view, GTK_TYPE_SCROLLED_WINDOW)
      
@@ -121,6 +128,42 @@ nautilus_tree_view_initialize_class (NautilusTreeViewClass *klass)
 }
 
 
+static void
+unlink_view_node_from_uri (NautilusTreeView *view,
+			   NautilusCTreeNode *view_node)
+{
+	gpointer orig_key, value;
+
+	if (g_hash_table_lookup_extended (view->details->view_node_to_uri_map,
+					  view_node, &orig_key, &value)) {
+		g_hash_table_remove (view->details->view_node_to_uri_map, view_node);
+		g_free (value);
+	}
+}
+
+/* URI will be g_free'd eventually */
+static void
+link_view_node_with_uri (NautilusTreeView *view,
+			 NautilusCTreeNode *view_node,
+			 const char *uri)
+{
+	unlink_view_node_from_uri (view, view_node);
+	g_hash_table_insert (view->details->view_node_to_uri_map,
+			     view_node, (gpointer) uri);
+}
+
+/* Returned string is only valid until next link or unlink of VIEW-NODE */
+static const char *
+map_view_node_to_uri (NautilusTreeView *view,
+		      NautilusCTreeNode *view_node)
+{
+	gpointer value = g_hash_table_lookup (view->details->view_node_to_uri_map,
+					      view_node);
+	g_assert (value != NULL);
+	return value;
+}
+
+
 static gboolean
 nautilus_tree_view_should_skip_file (NautilusTreeView *view,
 				     NautilusFile *file)
@@ -132,9 +175,54 @@ nautilus_tree_view_should_skip_file (NautilusTreeView *view,
 		  nautilus_file_is_directory (file)));
 }
 
+/* This is different to the should_skip function above, in that it
+ * also searches all parent files of URI. It will return true iff
+ * URI may be shown in the tree view display. Note that URI may
+ * not exist when this is called.
+ */
+static gboolean
+nautilus_tree_view_would_include_uri (NautilusTreeView *view,
+				      const char *uri)
+{
+	char *copy, *component;
+
+	/* The tree view currently only ever shows `file:' URIs */
+
+	if (!nautilus_str_has_prefix (uri, "file:")) {
+		return FALSE;
+	}
+
+	if (!view->details->show_hidden_files
+	    || !view->details->show_backup_files) {
+		copy = g_strdup (uri);
+		while (1) {
+			component = strrchr (copy, '/');
+			if (component != NULL) {
+				if ((!view->details->show_hidden_files
+				     && nautilus_file_name_matches_hidden_pattern (component + 1))
+				    || (!view->details->show_backup_files
+					&& nautilus_file_name_matches_backup_pattern (component + 1))) {
+					/* Don't show this file */
+					g_free (copy);
+					return FALSE;
+				} else {
+					/* Chop the bottom-most component from the uri */
+					*component = 0;
+				}
+			} else {
+				break;
+			}
+		}
+		g_free (copy);
+	}
+
+	return TRUE;
+}
+
 static void
 nautilus_tree_view_insert_model_node (NautilusTreeView *view, NautilusTreeNode *node)
 {
+	NautilusTreeNode *parent_node;
 	NautilusCTreeNode *parent_view_node;
  	NautilusCTreeNode *view_node;
 	NautilusFile *file;
@@ -145,6 +233,9 @@ nautilus_tree_view_insert_model_node (NautilusTreeView *view, NautilusTreeNode *
 	GdkBitmap *open_mask;
 	char *uri;
 
+	g_return_if_fail (NAUTILUS_IS_TREE_VIEW (view));
+	g_return_if_fail (NAUTILUS_IS_TREE_NODE (node));
+
 	file = nautilus_tree_node_get_file (node);
 
 	if (nautilus_tree_view_should_skip_file (view, file)) {
@@ -156,81 +247,97 @@ nautilus_tree_view_insert_model_node (NautilusTreeView *view, NautilusTreeNode *
 	printf ("Inserting URI into tree: %s\n", nautilus_file_get_uri (file));
 #endif
 
-	parent_view_node = nautilus_tree_view_find_parent_node (view, file);
-
+	parent_node = nautilus_tree_node_get_parent (node);
+	parent_view_node = (parent_node == NULL ? NULL
+			    : nautilus_tree_view_model_node_to_view_node (view, parent_node));
 
 #ifdef DEBUG_TREE
 	printf ("parent_view_node 0x%x (%s)\n", (unsigned) parent_view_node, 
-		nautilus_file_get_uri (nautilus_tree_view_node_to_file (view, node)));
+		nautilus_file_get_uri (nautilus_tree_view_node_to_file (view, parent_view_node)));
 #endif
 
 
-	text[0] = nautilus_file_get_name (file);
-	text[1] = NULL;
+	if (parent_view_node == NULL && !nautilus_tree_node_is_toplevel (node)) {
+		register_unparented_node (view, node);
+	} else {
+		text[0] = nautilus_file_get_name (file);
+		text[1] = NULL;
 
-	if (nautilus_tree_view_model_node_to_view_node (view, node) == NULL) {
-		nautilus_icon_factory_get_pixmap_and_mask_for_file (file,
-								    NULL,
-								    NAUTILUS_ICON_SIZE_FOR_MENUS,
-								    &closed_pixmap,
-								    &closed_mask);
+		if (nautilus_tree_view_model_node_to_view_node (view, node) == NULL) {
+			nautilus_icon_factory_get_pixmap_and_mask_for_file (file,
+									    NULL,
+									    NAUTILUS_ICON_SIZE_FOR_MENUS,
+									    &closed_pixmap,
+									    &closed_mask);
 
-		nautilus_icon_factory_get_pixmap_and_mask_for_file (file,
-								    "accept",
-								    NAUTILUS_ICON_SIZE_FOR_MENUS,
-								    &open_pixmap,
-								    &open_mask);
+			nautilus_icon_factory_get_pixmap_and_mask_for_file (file,
+									    "accept",
+									    NAUTILUS_ICON_SIZE_FOR_MENUS,
+									    &open_pixmap,
+									    &open_mask);
 
 
-		view_node = nautilus_ctree_insert_node (NAUTILUS_CTREE (view->details->tree),
-							parent_view_node, 
-							NULL,
-							text,
-							TREE_SPACING,
-							closed_pixmap, closed_mask, open_pixmap, open_mask,
-							! nautilus_file_is_directory (file),
-							FALSE);
-		gdk_pixmap_unref (closed_pixmap);
-		gdk_pixmap_unref (open_pixmap);
-		if (closed_mask != NULL) {
-			gdk_bitmap_unref (closed_mask);
-		}
-		if (open_mask != NULL) {
-			gdk_bitmap_unref (open_mask);
-		}
+			view->details->inserting_node = TRUE;
+			GTK_CLIST_UNSET_FLAG (GTK_CLIST (view->details->tree),
+					      CLIST_AUTO_SORT);
+			view_node = nautilus_ctree_insert_node (NAUTILUS_CTREE (view->details->tree),
+								parent_view_node, 
+								NULL,
+								text,
+								TREE_SPACING,
+								closed_pixmap, closed_mask, open_pixmap, open_mask,
+								! nautilus_file_is_directory (file),
+								FALSE);
+			GTK_CLIST_SET_FLAG (GTK_CLIST (view->details->tree),
+					    CLIST_AUTO_SORT);
+			view->details->inserting_node = FALSE;
 
-		nautilus_ctree_node_set_row_data (NAUTILUS_CTREE (view->details->tree),
-						  view_node,
-						  node);
+			gdk_pixmap_unref (closed_pixmap);
+			gdk_pixmap_unref (open_pixmap);
+			if (closed_mask != NULL) {
+				gdk_bitmap_unref (closed_mask);
+			}
+			if (open_mask != NULL) {
+				gdk_bitmap_unref (open_mask);
+			}
 
-		g_assert (g_hash_table_lookup (view->details->file_to_node_map, file) == NULL);
+			nautilus_ctree_node_set_row_data (NAUTILUS_CTREE (view->details->tree),
+							  view_node,
+							  node);
 
-		nautilus_file_ref (file);
-		g_hash_table_insert (view->details->file_to_node_map, file, view_node); 
+			g_assert (g_hash_table_lookup (view->details->file_to_node_map, file) == NULL);
+
+			nautilus_file_ref (file);
+			g_hash_table_insert (view->details->file_to_node_map, file, view_node); 
 		
-		if (nautilus_file_is_directory (nautilus_tree_node_get_file (node))) {
 			uri = nautilus_file_get_uri (file);
-			
-			if (nautilus_tree_expansion_state_is_node_expanded (view->details->expansion_state, uri)) {
-				if (!ctree_is_node_expanded (NAUTILUS_CTREE (view->details->tree),
-							     view_node)) {
-					nautilus_ctree_expand (NAUTILUS_CTREE (view->details->tree),
-							       view_node);
-				} 
-			} else {
-				if (ctree_is_node_expanded (NAUTILUS_CTREE (view->details->tree),
-							    view_node)) {
-					nautilus_ctree_collapse (NAUTILUS_CTREE (view->details->tree),
-								 view_node);
+			link_view_node_with_uri (view, view_node, uri);
+
+			if (nautilus_file_is_directory (nautilus_tree_node_get_file (node))) {
+				if (nautilus_tree_expansion_state_is_node_expanded (view->details->expansion_state, uri)) {
+					if (!ctree_is_node_expanded (NAUTILUS_CTREE (view->details->tree),
+								     view_node)) {
+						nautilus_ctree_expand (NAUTILUS_CTREE (view->details->tree),
+								       view_node);
+					} 
+				} else {
+					if (ctree_is_node_expanded (NAUTILUS_CTREE (view->details->tree),
+								    view_node)) {
+						nautilus_ctree_collapse (NAUTILUS_CTREE (view->details->tree),
+									 view_node);
+					}
 				}
 			}
 
-			g_free (uri);
+			nautilus_ctree_sort_single_node (NAUTILUS_CTREE (view->details->tree), view_node);
+
+			insert_unparented_nodes (view, node);
+		} else {
+			nautilus_tree_view_update_model_node (view, node);
 		}
+
+		g_free (text[0]);
 	}
-
-	g_free (text[0]);
-
 	notify_node_seen (view, node);
 }
 
@@ -239,17 +346,22 @@ forget_view_node (NautilusTreeView *view,
 		  NautilusCTreeNode *view_node)
 {
 	NautilusFile *file;
-	char *uri;
+	NautilusTreeNode *node;
 
 	file = nautilus_tree_view_node_to_file (view, view_node);
+	node = nautilus_tree_view_node_to_model_node (view, view_node);
 
-	uri = nautilus_file_get_uri (file);
-	nautilus_tree_expansion_state_remove_node
-		(view->details->expansion_state, uri);
-	g_free (uri);
+	g_return_if_fail (node != NULL);
+
+	forget_unparented_node (view, node);
+
+	nautilus_ctree_node_set_row_data (NAUTILUS_CTREE (view->details->tree),
+					  view_node, NULL);
 
 	g_hash_table_remove (view->details->file_to_node_map, file);
 	nautilus_file_unref (file);
+
+	unlink_view_node_from_uri (view, view_node);
 }
 
 static void
@@ -272,6 +384,10 @@ nautilus_tree_view_remove_model_node (NautilusTreeView *view, NautilusTreeNode *
 {
 	NautilusCTreeNode *view_node;
 	NautilusFile *file;
+	const char *uri;
+
+	g_return_if_fail (NAUTILUS_IS_TREE_VIEW (view));
+	g_return_if_fail (NAUTILUS_IS_TREE_NODE (node));
 
 	file = nautilus_tree_node_get_file (node);
 
@@ -282,6 +398,17 @@ nautilus_tree_view_remove_model_node (NautilusTreeView *view, NautilusTreeNode *
 
  	view_node = nautilus_tree_view_model_node_to_view_node (view, node);
  	if (view_node != NULL) {
+		/* The URI associated with FILE may have been renamed by now,
+		 * so using nautilus_file_get_uri () is no good (since it would
+		 * give the new name, not the old name). Hence the extra hash
+		 * table mapping view nodes to URIs..
+		 *
+		 * Note that it would be better to remove the expansion
+		 * state of the children, but that breaks renaming..
+		 */
+		 uri = map_view_node_to_uri (view, view_node);
+		 nautilus_tree_expansion_state_remove_node (view->details->expansion_state, uri);
+
 		forget_view_node_and_children (view, view_node);
  		nautilus_ctree_remove_node (NAUTILUS_CTREE (view->details->tree),
  					    view_node);
@@ -327,6 +454,9 @@ nautilus_tree_view_update_model_node (NautilusTreeView *view, NautilusTreeNode *
 	GdkPixmap *open_pixmap;
 	GdkBitmap *open_mask;
 	
+	g_return_if_fail (NAUTILUS_IS_TREE_VIEW (view));
+	g_return_if_fail (NAUTILUS_IS_TREE_NODE (node));
+
 	file = nautilus_tree_node_get_file (node);
 
 	if (nautilus_tree_view_should_skip_file (view, file)) {
@@ -343,6 +473,8 @@ nautilus_tree_view_update_model_node (NautilusTreeView *view, NautilusTreeNode *
 	if (view_node != NULL) {
 		name = nautilus_file_get_name (file);
 	
+		link_view_node_with_uri (view, view_node, nautilus_file_get_uri (file));
+
 		nautilus_icon_factory_get_pixmap_and_mask_for_file (file,
 								    NULL,
 								    NAUTILUS_ICON_SIZE_FOR_MENUS,
@@ -395,11 +527,69 @@ nautilus_tree_view_update_model_node (NautilusTreeView *view, NautilusTreeNode *
 
 			g_free (uri);
 		}
+
+		insert_unparented_nodes (view, node);
 	} else {
 		nautilus_tree_view_insert_model_node (view, node);
 	}
 }
 
+static void
+register_unparented_node (NautilusTreeView *view, NautilusTreeNode *node)
+{
+	g_return_if_fail (NAUTILUS_IS_TREE_VIEW (view));
+	g_return_if_fail (NAUTILUS_IS_TREE_NODE (node));
+	g_return_if_fail (!nautilus_tree_node_is_toplevel (node));
+
+	if (g_list_find (view->details->unparented_tree_nodes, node) == NULL) {
+		view->details->unparented_tree_nodes = g_list_prepend (view->details->unparented_tree_nodes, node);
+	}
+}
+
+static void
+forget_unparented_node (NautilusTreeView *view, NautilusTreeNode *node)
+{
+	g_return_if_fail (NAUTILUS_IS_TREE_VIEW (view));
+	g_return_if_fail (NAUTILUS_IS_TREE_NODE (node));
+
+	view->details->unparented_tree_nodes = g_list_remove (view->details->unparented_tree_nodes, node);
+}
+
+static void
+insert_unparented_nodes (NautilusTreeView *view, NautilusTreeNode *node)
+{
+	NautilusFile *file, *sub_file;
+	NautilusDirectory *directory;
+	GList *p, *to_add;
+	NautilusTreeNode *sub_node;
+
+	g_return_if_fail (NAUTILUS_IS_TREE_VIEW (view));
+	g_return_if_fail (NAUTILUS_IS_TREE_NODE (node));
+
+	file = nautilus_tree_node_get_file (node);
+
+	if (nautilus_file_is_directory (file)) {
+		directory = nautilus_tree_node_get_directory (node);
+		if (directory != NULL) {
+			to_add = NULL;
+			for (p = view->details->unparented_tree_nodes; p != NULL; p = p->next) {
+				sub_node = p->data;
+				sub_file = nautilus_tree_node_get_file (sub_node);
+				if (nautilus_directory_contains_file (directory, sub_file)) {
+					to_add = g_list_prepend (to_add, sub_node);
+				}
+			}
+			for (p = to_add; p != NULL; p = p->next) {
+				sub_node = p->data;
+				view->details->unparented_tree_nodes = g_list_remove (view->details->unparented_tree_nodes, sub_node);
+				nautilus_tree_view_insert_model_node (view, sub_node);
+				
+			}
+			g_list_free (to_add);
+		}
+	}
+}
+
 
 static void
 notify_done_loading (NautilusTreeView *view,
@@ -561,6 +751,29 @@ nautilus_tree_view_model_node_removed_callback (NautilusTreeModel *model,
 					   node);
 }
 
+static void
+nautilus_tree_view_model_node_renamed_callback (NautilusTreeModel *model,
+						const char	  *old_uri,
+						const char 	  *new_uri,
+						gpointer           callback_data)
+{
+	NautilusTreeView *view;
+
+	/* Propagate the expansion state of the old name to the new name */
+
+	view = NAUTILUS_TREE_VIEW (callback_data);
+
+	if (nautilus_tree_view_would_include_uri (view, new_uri)) {
+		if (nautilus_tree_expansion_state_is_node_expanded (view->details->expansion_state, old_uri)) {
+			nautilus_tree_expansion_state_expand_node (view->details->expansion_state, new_uri);
+		} else {
+			nautilus_tree_expansion_state_collapse_node (view->details->expansion_state, new_uri);
+		}
+	}
+
+	nautilus_tree_expansion_state_remove_node (view->details->expansion_state, old_uri);
+}
+
 static void
 nautilus_tree_view_model_done_loading_callback (NautilusTreeModel *model,
 						NautilusTreeNode  *node,
@@ -593,6 +806,10 @@ nautilus_tree_view_load_from_filesystem (NautilusTreeView *view)
 			    "node_removed",
 			    nautilus_tree_view_model_node_removed_callback,
 			    view);
+	gtk_signal_connect (GTK_OBJECT (view->details->model),
+			    "node_being_renamed",
+			    nautilus_tree_view_model_node_renamed_callback,
+			    view);
 	gtk_signal_connect (GTK_OBJECT (view->details->model),
 			    "done_loading_children",
 			    nautilus_tree_view_model_done_loading_callback,
@@ -627,6 +844,38 @@ filtering_changed_callback (gpointer callback_data)
 }
 
 
+static gint
+ctree_compare_rows (GtkCList      *clist,
+		    gconstpointer  ptr1,
+		    gconstpointer  ptr2)
+{
+	NautilusTreeView *view;
+	NautilusFile *file1, *file2;
+	gint result;
+
+	/* Default to returning -1 in unknown cases. This magically
+	 * ensures that inserted nodes are initially placed at the
+	 * head of the list, instead of wasting time comparing
+	 * against all files for no reason (makes expanding /dev/
+	 * actually bearable)
+	 */
+	result = -1;
+
+	view = gtk_object_get_data (GTK_OBJECT (clist), "tree_view");
+	g_assert (view != NULL);
+
+	if (!view->details->inserting_node) {
+		file1 = nautilus_tree_view_node_to_file (view, nautilus_ctree_find_node_ptr (NAUTILUS_CTREE (view->details->tree), (NautilusCTreeRow *) ptr1));
+		file2 = nautilus_tree_view_node_to_file (view, nautilus_ctree_find_node_ptr (NAUTILUS_CTREE (view->details->tree), (NautilusCTreeRow *) ptr2));
+
+		if (file1 != NULL && file2 != NULL) {
+			result = nautilus_file_compare_for_sort (file1, file2, NAUTILUS_FILE_SORT_BY_NAME);
+		}
+	}
+
+	return result;
+}
+
 static void
 nautilus_tree_view_initialize (NautilusTreeView *view)
 {
@@ -656,10 +905,16 @@ nautilus_tree_view_initialize (NautilusTreeView *view)
         gtk_clist_set_selection_mode (GTK_CLIST (view->details->tree), GTK_SELECTION_SINGLE);
 	gtk_clist_set_auto_sort (GTK_CLIST (view->details->tree), TRUE);
 	gtk_clist_set_sort_type (GTK_CLIST (view->details->tree), GTK_SORT_ASCENDING);
+	gtk_clist_set_compare_func (GTK_CLIST (view->details->tree),
+				    ctree_compare_rows);
 	gtk_clist_set_column_auto_resize (GTK_CLIST (view->details->tree), 0, TRUE);
 	gtk_clist_columns_autosize (GTK_CLIST (view->details->tree));
-
 	gtk_clist_set_reorderable (GTK_CLIST (view->details->tree), FALSE);
+	gtk_clist_set_row_height (GTK_CLIST (view->details->tree),
+				  MAX (NAUTILUS_ICON_SIZE_FOR_MENUS,
+				       view->details->tree->style->font->ascent
+				       + view->details->tree->style->font->descent));
+        nautilus_ctree_set_indent (NAUTILUS_CTREE (view->details->tree), 12);
 
 	gtk_signal_connect (GTK_OBJECT (view->details->tree),
 			    "tree_expand",
@@ -715,6 +970,7 @@ nautilus_tree_view_initialize (NautilusTreeView *view)
 			    view);
 
 	view->details->file_to_node_map = g_hash_table_new (NULL, NULL);
+	view->details->view_node_to_uri_map = g_hash_table_new (NULL, NULL);
 	
 	nautilus_tree_view_load_from_filesystem (view);
 
@@ -738,8 +994,7 @@ disconnect_model_handlers (NautilusTreeView *view)
 {
 	NautilusTreeNode *node;
 
-	node = nautilus_tree_model_get_node (view->details->model,
-					     "file:///");
+	node = nautilus_tree_model_get_node (view->details->model, "file:///");
 
 	if (node != NULL) {
 		nautilus_tree_model_monitor_remove (view->details->model, view);
@@ -754,6 +1009,12 @@ free_file_to_node_map_entry (gpointer key, gpointer value, gpointer callback_dat
 	nautilus_file_unref (NAUTILUS_FILE (key));
 }
 
+static void
+free_view_node_to_uri_map_entry (gpointer key, gpointer value, gpointer callback_data)
+{
+	g_free (value);			/* the URI */
+}
+
 static void
 nautilus_tree_view_destroy (GtkObject *object)
 {
@@ -761,6 +1022,8 @@ nautilus_tree_view_destroy (GtkObject *object)
 	
 	view = NAUTILUS_TREE_VIEW (object);
 
+	cancel_possible_activation (view);
+
 	if (view->details->pending_idle_id != 0) {
 		gtk_idle_remove (view->details->pending_idle_id);
 	}
@@ -782,6 +1045,11 @@ nautilus_tree_view_destroy (GtkObject *object)
 			      NULL);
 	g_hash_table_destroy (view->details->file_to_node_map);
 	
+	g_hash_table_foreach (view->details->view_node_to_uri_map,
+			      free_view_node_to_uri_map_entry,
+			      NULL);
+	g_hash_table_destroy (view->details->view_node_to_uri_map);
+
 	nautilus_tree_view_free_dnd (view);
 
 	disconnect_model_handlers (view);
@@ -790,12 +1058,14 @@ nautilus_tree_view_destroy (GtkObject *object)
 	nautilus_tree_expansion_state_save (view->details->expansion_state);
 	gtk_object_unref (GTK_OBJECT (view->details->expansion_state));
 
+	g_list_free (view->details->unparented_tree_nodes);
+
 	g_free (view->details->current_main_view_uri);
 	g_free (view->details->selected_uri);
 
 	g_free (view->details);
 	
-	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
+	NAUTILUS_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
 /**
@@ -944,6 +1214,7 @@ expand_uri_sequence_and_select_end (NautilusTreeView *view)
 	NautilusCTreeNode *view_node;
 	NautilusCTreeNode *last_valid_view_node;
 	NautilusFile *file;
+	NautilusTreeNode *node;
 
 	view_node = NULL;
 	last_valid_view_node = NULL;
@@ -1008,10 +1279,11 @@ expand_uri_sequence_and_select_end (NautilusTreeView *view)
 								  last_valid_view_node),
 			   TRUE);
 
-	call_when_uri_loaded_or_parent_done_loading (view, uri, 
-						     nautilus_tree_model_get_node (view->details->model,
-										   (char *) p->prev->data),
-						     expand_uri_sequence_and_select_end);
+	node = nautilus_tree_model_get_node (view->details->model, (char *) p->prev->data);
+	if (node != NULL) {
+		call_when_uri_loaded_or_parent_done_loading (view, uri, node,
+							     expand_uri_sequence_and_select_end);
+	}
 	
 	p->prev->next = NULL;
 	p->prev = NULL;
@@ -1038,6 +1310,10 @@ nautilus_tree_view_load_uri (NautilusTreeView *view,
 	g_free (view->details->current_main_view_uri);
 	view->details->current_main_view_uri = g_strdup (canonical_uri);
 
+	/* FIXME 6801 it seems likely that either nautilus_uris_match 
+	 * or nautilus_uris_match_ignore_fragments
+	 * should be used here
+	 */
 	if (nautilus_strcmp (canonical_uri, view->details->selected_uri) == 0) {
 		g_free (canonical_uri);
 		return;
@@ -1118,7 +1394,12 @@ static void
 reload_whole_tree (NautilusTreeView *view,
 		   gboolean          force_reload)
 {
-	reload_model_node_recursive (view, nautilus_tree_model_get_node (view->details->model, "file:///"), force_reload);
+	NautilusTreeNode *node;
+
+	node = nautilus_tree_model_get_node (view->details->model, "file:///");
+	if (node != NULL) {
+		reload_model_node_recursive (view, node, force_reload);
+	}
 }
 
 
@@ -1129,6 +1410,7 @@ expand_node_for_file (NautilusTreeView *view,
 {
 	char *uri;
 	gboolean ever_expanded;
+	NautilusTreeNode *node;
 
 	uri = nautilus_file_get_uri (file);
 	ever_expanded = nautilus_tree_expansion_state_was_ever_expanded (view->details->expansion_state, 
@@ -1137,10 +1419,10 @@ expand_node_for_file (NautilusTreeView *view,
 						   uri);
 	g_free (uri);
 
-	reload_model_node_recursive (view, 
-				     nautilus_tree_model_get_node_from_file (view->details->model, 
-									     file),
-				     ever_expanded);
+	node = nautilus_tree_model_get_node_from_file (view->details->model, file);
+	if (node != NULL) {
+		reload_model_node_recursive (view, node, ever_expanded);
+	}
 }
 
 static void
@@ -1187,27 +1469,72 @@ ctree_show_node (NautilusCTree *tree,
 }
 
 
+static void
+got_activation_uri_callback (NautilusFile *file,
+			     gpointer callback_data)
+{
+	char *uri;
+	NautilusTreeView *view;
+
+	view = NAUTILUS_TREE_VIEW (callback_data);
+
+	if (file == view->details->activation_uri_wait_file) {
+		uri = nautilus_file_get_activation_uri (file);
+		
+		if (uri != NULL &&
+		    !nautilus_uris_match_ignore_fragments (view->details->current_main_view_uri, uri) &&
+		    strncmp (uri, "command:", strlen ("command:")) != 0) {
+			nautilus_view_open_location_in_this_window 
+				(NAUTILUS_VIEW (view->details->nautilus_view), uri);
+			g_free (view->details->selected_uri);
+			view->details->selected_uri = g_strdup (uri);
+		}
+		
+		ctree_show_node (NAUTILUS_CTREE (view->details->tree), 
+				 file_to_view_node (view, file));
+		
+		g_free (uri);
+
+		nautilus_file_unref (view->details->activation_uri_wait_file);
+		view->details->activation_uri_wait_file = NULL;
+	}
+}
+
+static void
+cancel_possible_activation (NautilusTreeView *view)
+{
+	if (view->details->activation_uri_wait_file != NULL) {
+		nautilus_file_cancel_call_when_ready 
+			(view->details->activation_uri_wait_file,
+			 got_activation_uri_callback,
+			 view);
+		nautilus_file_unref (view->details->activation_uri_wait_file);
+	}
+
+	view->details->activation_uri_wait_file = NULL;
+}
+
 static void
 tree_select_row_callback (NautilusCTree              *tree,
 			  NautilusCTreeNode          *node,
 			  gint                        column,
 			  NautilusTreeView           *view)
 {
-	char *uri;
-	
-	uri = nautilus_file_get_uri (nautilus_tree_view_node_to_file (view, node));
-	
-	if (uri != NULL &&
-	    nautilus_strcmp (view->details->current_main_view_uri, uri) != 0) {
-		nautilus_view_open_location_in_this_window (NAUTILUS_VIEW (view->details->nautilus_view), uri);
-		
-		g_free (view->details->selected_uri);
-		view->details->selected_uri = g_strdup (uri);
-	}
+	GList *attributes;
 
-	ctree_show_node (tree, node);
+	cancel_possible_activation (view);
 
-	g_free (uri);
+	view->details->activation_uri_wait_file = nautilus_tree_view_node_to_file (view,
+										   node);
+	nautilus_file_ref (view->details->activation_uri_wait_file);
+
+	attributes = g_list_prepend (NULL, NAUTILUS_FILE_ATTRIBUTE_ACTIVATION_URI);
+	
+	nautilus_file_call_when_ready (view->details->activation_uri_wait_file,
+				       attributes,
+				       got_activation_uri_callback,
+				       view);
+ 
 }
 
 static NautilusCTreeNode *
@@ -1249,28 +1576,3 @@ nautilus_tree_view_update_all_icons (NautilusTreeView *view)
 					 nautilus_tree_view_model_node_changed_callback,
 					 view);
 }
-
-
-static NautilusCTreeNode *
-nautilus_tree_view_find_parent_node (NautilusTreeView *view, 
-				     NautilusFile     *file)
-{
-	NautilusTreeNode *node;
-
-	node = nautilus_tree_model_get_node_from_file (view->details->model, 
-						       file);
-
-	if (node == NULL) {
-		g_print ("You've run into an intermittent tree view bug.\n");
-		g_print ("Running Nautilus again will probably not hit this bug.\n");
-		g_print ("The tree view didn't have a node for %s\n", nautilus_file_get_uri (file));
-		g_print ("The tree view had the following nodes:\n\n");
-		nautilus_tree_model_dump_files (view->details->model);
-
-		/* NOW die. */
-		g_assert_not_reached ();
-	}
-
-	return nautilus_tree_view_model_node_to_view_node (view, nautilus_tree_node_get_parent (node));
-}
-
