@@ -194,7 +194,7 @@ nautilus_font_picker_destroy (GtkObject* object)
 	g_free (font_picker->detail);
 
 	/* Chain */
-	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
+	NAUTILUS_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
 static void
