@@ -32,6 +32,7 @@
 #include "nautilus-preferences-group.h"
 #include "nautilus-preferences-item.h"
 #include "nautilus-string.h"
+#include "nautilus-medusa-support.h"
 #include "nautilus-view-identifier.h"
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
@@ -50,23 +51,25 @@ static const char USE_PROXY_KEY[] = "/system/gnome-vfs/use-http-proxy";
 static const char SYSTEM_GNOME_VFS_PATH[] = "/system/gnome-vfs";
 
 /* Forward declarations */
-static char *     global_preferences_get_sidebar_panel_key               (const char    *panel_iid);
-static gboolean   global_preferences_is_sidebar_panel_enabled_cover      (gpointer       data,
-									  gpointer       callback_data);
+static char *     global_preferences_get_sidebar_panel_key               (const char             *panel_iid);
+static gboolean   global_preferences_is_sidebar_panel_enabled_cover      (gpointer                data,
+									  gpointer                callback_data);
 static GList *    global_preferences_get_sidebar_panel_view_identifiers  (void);
-static gboolean   global_preferences_close_dialog_callback               (GtkWidget     *dialog,
-									  gpointer       user_data);
+static gboolean   global_preferences_close_dialog_callback               (GtkWidget              *dialog,
+									  gpointer                user_data);
 static void       global_preferences_install_sidebar_panel_defaults      (void);
 static void       global_preferences_install_sidebar_panel_descriptions  (void);
 static void       global_preferences_install_defaults                    (void);
 static void       global_preferences_install_visibility                  (void);
-static void       global_preferences_install_speed_tradeoff_descriptions (const char    *name,
-									  const char    *description);
+static void       global_preferences_install_speed_tradeoff_descriptions (const char             *name,
+									  const char             *description);
 static void       global_preferences_install_home_location_defaults      (void);
+static void       global_preferences_install_medusa_defaults             (void);
 static void       global_preferences_install_descriptions                (void);
-static int        compare_view_identifiers                               (gconstpointer  a,
-									  gconstpointer  b);
+static int        compare_view_identifiers                               (gconstpointer           a,
+									  gconstpointer           b);
 static GtkWidget *global_preferences_create_dialog                       (void);
+static GtkWidget *global_preferences_create_search_pane                  (NautilusPreferencesBox *preference_box);
 
 static GtkWidget *global_prefs_dialog = NULL;
 
@@ -83,21 +86,17 @@ global_preferences_install_descriptions (void)
 	g_return_if_fail (preferences_registered == FALSE);
 	preferences_registered = TRUE;
 
-	/* Theme */
 	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_THEME,
 					      _("current theme"));
 
-	/* Window create new */
 	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_WINDOW_ALWAYS_NEW,
-					      _("Open each item in a separate window"));
+					      _("Open each file or folder in a separate window"));
 	
-	/* Trash confirm */
 	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_CONFIRM_TRASH,
-					      _("Ask before deleting items from the trash"));
+					      _("Ask before emptying the Trash"));
 	
-	/* Click activation type */
 	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_CLICK_POLICY,
-					      _("Click policy"));
+					      _("Click Behavior"));
 	
 	nautilus_preferences_enumeration_insert (NAUTILUS_PREFERENCES_CLICK_POLICY,
 						 _("single"),
@@ -109,31 +108,54 @@ global_preferences_install_descriptions (void)
 						 _("Activate items with a double click"),
 						 NAUTILUS_CLICK_POLICY_DOUBLE);
 	
-	/*
-	 * Speed tradeoffs
+	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_EXECUTABLE_TEXT_ACTIVATION,
+					      _("Executable Text Files"));
+	
+	nautilus_preferences_enumeration_insert (NAUTILUS_PREFERENCES_EXECUTABLE_TEXT_ACTIVATION,
+						 _("launch"),
+						 _("Execute files when they are clicked"),
+						 NAUTILUS_EXECUTABLE_TEXT_LAUNCH);
+	
+	nautilus_preferences_enumeration_insert (NAUTILUS_PREFERENCES_EXECUTABLE_TEXT_ACTIVATION,
+						 _("display"),
+						 _("Display files when they are clicked"),
+						 NAUTILUS_EXECUTABLE_TEXT_DISPLAY);
+	
+	nautilus_preferences_enumeration_insert (NAUTILUS_PREFERENCES_EXECUTABLE_TEXT_ACTIVATION,
+						 _("ask"),
+						 _("Ask each time"),
+						 NAUTILUS_EXECUTABLE_TEXT_ASK);
+
+	/* Note that I've changed all these descriptions to match the displayed text.
+	 * The original intention was to use them differently, but currently they aren't
+	 * used at all. By matching the displayed text, at least localizers won't have to
+	 * localize two different strings. If we ever make them appear separately, we will
+	 * have to reword them (as we would have had to anyway).
 	 */
 	global_preferences_install_speed_tradeoff_descriptions (NAUTILUS_PREFERENCES_SHOW_TEXT_IN_ICONS,
-								_("Display text in icons"));
+								_("Show Text in Icons"));
+	
+	global_preferences_install_speed_tradeoff_descriptions (NAUTILUS_PREFERENCES_SHOW_DIRECTORY_ITEM_COUNTS,
+								_("Show Count of Items in Folders"));
 	
 	global_preferences_install_speed_tradeoff_descriptions (NAUTILUS_PREFERENCES_SHOW_IMAGE_FILE_THUMBNAILS,
-								_("Show thumbnails for image files"));
+								_("Show Thumbnails for Image Files"));
 
 	global_preferences_install_speed_tradeoff_descriptions (NAUTILUS_PREFERENCES_USE_PUBLIC_METADATA,
-								_("Read and write metadata in each folder"));
+								_("Make Folder Appearance Details Public"));
 	
 	global_preferences_install_speed_tradeoff_descriptions (NAUTILUS_PREFERENCES_PREVIEW_SOUND,
-								_("Play a sound file when the mouse is over it"));
+								_("Preview Sound Files"));
 	
-	/* Appearance options */
 	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_SMOOTH_GRAPHICS_MODE,
 					      _("Use smoother (but slower) graphics"));
 
 	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_DIRECTORY_VIEW_FONT_FAMILY,
-					      _("Font family used to display file names"));
+					      _("Use this font family to display file names:"));
 
 	
-	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_START_WITH_TOOL_BAR,
-					      _("Display tool bar in new windows"));
+	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_START_WITH_TOOLBAR,
+					      _("Display toolbar in new windows"));
 
 	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_START_WITH_LOCATION_BAR,
 					      _("Display location bar in new windows"));
@@ -148,59 +170,53 @@ global_preferences_install_descriptions (void)
 					      _("Use Nautilus to draw the desktop"));
 								  
 	/* search tradeoffs */
-	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_SEARCH_METHOD,
-					      _("Always do slow, complete search"));
+	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_USE_FAST_SEARCH,
+					      _("Enable fast search (indexes your hard drive)"));
 
-	/* search bar type */
 	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_SEARCH_BAR_TYPE,
 					      _("search type to do by default"));
 	
 	nautilus_preferences_enumeration_insert (NAUTILUS_PREFERENCES_SEARCH_BAR_TYPE,
 						 _("search by text"),
-						 _("Search for files by text only"),
+						 _("Search for files by file name only"),
 						 NAUTILUS_SIMPLE_SEARCH_BAR);
 	
 	nautilus_preferences_enumeration_insert (NAUTILUS_PREFERENCES_SEARCH_BAR_TYPE,
 						 _("search by text and properties"),
-						 _("Search for files by text and by their properties"),
+						 _("Search for files by file name and file properties"),
 						 NAUTILUS_COMPLEX_SEARCH_BAR);
 	
-	/* web search uri  */
 	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_SEARCH_WEB_URI,
-					      _("Search Web Location"));
+					      _("Search Engine Location"));
 
 	
 	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_SHOW_HIDDEN_FILES,
-					      _("Show hidden files (starting with \".\")"));
+					      _("Show hidden files (file names start with \".\")"));
 	
 	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_SHOW_BACKUP_FILES,
-					      _("Show backup files (ending with \"~\")"));
+					      _("Show backup files (file names end with \"~\")"));
 	
 	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_SHOW_SPECIAL_FLAGS,
 					      _("Show special flags in Properties window"));
 	
 	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES,
-					      _("Show only folders in tree sidebar panel"));
+					      _("Show only folders (no files) in the tree"));
 
 	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_CAN_ADD_CONTENT,
 					      _("Can add Content"));
 
-	/* built-in bookmarks */
 	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_HIDE_BUILT_IN_BOOKMARKS,
-					      _("Don't include the built-in bookmarks"));
+					      _("Don't include the built-in bookmarks in the Bookmarks menu"));
 	
 
-	/* Home location */
 	nautilus_preferences_set_description (NAUTILUS_PREFERENCES_HOME_URI,
-					      _("Home Location"));
+					      _("Location:"));
 
-	/* Sidebar panel descriptions */
 	global_preferences_install_sidebar_panel_descriptions ();
 	
-	/* Proxy descriptions */
 	nautilus_preferences_set_description (USE_PROXY_KEY, _("Use HTTP Proxy"));
-	nautilus_preferences_set_description (PROXY_HOST_KEY, _("HTTP Proxy"));
-	nautilus_preferences_set_description (PROXY_PORT_KEY, _("HTTP Proxy Port"));
+	nautilus_preferences_set_description (PROXY_HOST_KEY, _("Location:"));
+	nautilus_preferences_set_description (PROXY_PORT_KEY, _("Port:"));
 }
 
 /**
@@ -233,6 +249,18 @@ global_preferences_install_defaults (void)
 	nautilus_preferences_default_set_integer (NAUTILUS_PREFERENCES_SHOW_TEXT_IN_ICONS,
 						  NAUTILUS_USER_LEVEL_NOVICE,
 						  NAUTILUS_SPEED_TRADEOFF_LOCAL_ONLY);
+
+	/* Don't show remote directory item counts for Beginner users because computing them
+	 * can be annoyingly slow, especially for FTP. If we make this fast enough for FTP in
+	 * particular, we should change this default to ALWAYS.
+	 */
+	nautilus_preferences_default_set_integer (NAUTILUS_PREFERENCES_SHOW_DIRECTORY_ITEM_COUNTS,
+						  NAUTILUS_USER_LEVEL_NOVICE,
+						  NAUTILUS_SPEED_TRADEOFF_LOCAL_ONLY);
+	
+	nautilus_preferences_default_set_integer (NAUTILUS_PREFERENCES_SHOW_DIRECTORY_ITEM_COUNTS,
+						  NAUTILUS_USER_LEVEL_INTERMEDIATE,
+						  NAUTILUS_SPEED_TRADEOFF_ALWAYS);
 	
 	nautilus_preferences_default_set_string (NAUTILUS_PREFERENCES_DIRECTORY_VIEW_FONT_FAMILY,
 						 NAUTILUS_USER_LEVEL_NOVICE,
@@ -242,6 +270,10 @@ global_preferences_install_defaults (void)
 						  NAUTILUS_USER_LEVEL_NOVICE,
 						  NAUTILUS_CLICK_POLICY_DOUBLE);
 	
+	nautilus_preferences_default_set_integer (NAUTILUS_PREFERENCES_EXECUTABLE_TEXT_ACTIVATION,
+						  NAUTILUS_USER_LEVEL_NOVICE,
+						  NAUTILUS_EXECUTABLE_TEXT_ASK);
+	
 	nautilus_preferences_default_set_string (NAUTILUS_PREFERENCES_THEME,
 						 NAUTILUS_USER_LEVEL_NOVICE,
 						 "default");
@@ -266,16 +298,12 @@ global_preferences_install_defaults (void)
 						  NAUTILUS_USER_LEVEL_NOVICE,
 						  FALSE);
 	nautilus_preferences_default_set_boolean (NAUTILUS_PREFERENCES_SHOW_SPECIAL_FLAGS,
-						  NAUTILUS_USER_LEVEL_HACKER,
+						  NAUTILUS_USER_LEVEL_ADVANCED,
 						  TRUE);
 	
 	nautilus_preferences_default_set_boolean (NAUTILUS_PREFERENCES_SHOW_DESKTOP,
 						  NAUTILUS_USER_LEVEL_NOVICE,
 						  TRUE);
-	nautilus_preferences_default_set_boolean (NAUTILUS_PREFERENCES_SEARCH_METHOD,
-						  NAUTILUS_USER_LEVEL_NOVICE,
-						  TRUE);
-
 	nautilus_preferences_default_set_boolean (NAUTILUS_PREFERENCES_CAN_ADD_CONTENT,
 						  NAUTILUS_USER_LEVEL_NOVICE,
 						  FALSE);
@@ -312,7 +340,7 @@ global_preferences_install_defaults (void)
 						 NAUTILUS_USER_LEVEL_NOVICE,
 						 "http://www.eazel.com/websearch");
 
-	nautilus_preferences_default_set_boolean (NAUTILUS_PREFERENCES_START_WITH_TOOL_BAR,
+	nautilus_preferences_default_set_boolean (NAUTILUS_PREFERENCES_START_WITH_TOOLBAR,
 						  NAUTILUS_USER_LEVEL_NOVICE,
 						  TRUE);
 	nautilus_preferences_default_set_boolean (NAUTILUS_PREFERENCES_START_WITH_LOCATION_BAR,
@@ -345,6 +373,9 @@ global_preferences_install_defaults (void)
 
 	/* Home location */
 	global_preferences_install_home_location_defaults ();
+
+	/* Medusa */
+	global_preferences_install_medusa_defaults ();
 }
 
 /**
@@ -372,7 +403,7 @@ global_preferences_install_visibility (void)
 						     NAUTILUS_USER_LEVEL_INTERMEDIATE);
 
 	nautilus_preferences_set_visible_user_level (NAUTILUS_PREFERENCES_CONFIRM_TRASH,
-						     NAUTILUS_USER_LEVEL_HACKER);
+						     NAUTILUS_USER_LEVEL_ADVANCED);
 
 	nautilus_preferences_set_visible_user_level (NAUTILUS_PREFERENCES_SHOW_HIDDEN_FILES,
 						     NAUTILUS_USER_LEVEL_INTERMEDIATE);
@@ -384,12 +415,15 @@ global_preferences_install_visibility (void)
 						     NAUTILUS_USER_LEVEL_INTERMEDIATE);
 
 	nautilus_preferences_set_visible_user_level (NAUTILUS_PREFERENCES_SHOW_SPECIAL_FLAGS,
-						     NAUTILUS_USER_LEVEL_HACKER);
+						     NAUTILUS_USER_LEVEL_ADVANCED);
+
+	nautilus_preferences_set_visible_user_level (NAUTILUS_PREFERENCES_EXECUTABLE_TEXT_ACTIVATION,
+						     NAUTILUS_USER_LEVEL_ADVANCED);
 
 	nautilus_preferences_set_visible_user_level (NAUTILUS_PREFERENCES_SMOOTH_GRAPHICS_MODE,
 						     NAUTILUS_USER_LEVEL_INTERMEDIATE);
 
-	nautilus_preferences_set_visible_user_level (NAUTILUS_PREFERENCES_START_WITH_TOOL_BAR,
+	nautilus_preferences_set_visible_user_level (NAUTILUS_PREFERENCES_START_WITH_TOOLBAR,
 						     NAUTILUS_USER_LEVEL_INTERMEDIATE);
 
 	nautilus_preferences_set_visible_user_level (NAUTILUS_PREFERENCES_START_WITH_LOCATION_BAR,
@@ -407,6 +441,9 @@ global_preferences_install_visibility (void)
 	nautilus_preferences_set_visible_user_level (NAUTILUS_PREFERENCES_SHOW_TEXT_IN_ICONS,
 						     NAUTILUS_USER_LEVEL_INTERMEDIATE);
 
+	nautilus_preferences_set_visible_user_level (NAUTILUS_PREFERENCES_SHOW_DIRECTORY_ITEM_COUNTS,
+						     NAUTILUS_USER_LEVEL_INTERMEDIATE);
+
 	nautilus_preferences_set_visible_user_level (NAUTILUS_PREFERENCES_SHOW_IMAGE_FILE_THUMBNAILS,
 						     NAUTILUS_USER_LEVEL_INTERMEDIATE);
 
@@ -414,12 +451,12 @@ global_preferences_install_visibility (void)
 						     NAUTILUS_USER_LEVEL_INTERMEDIATE);
 
 	nautilus_preferences_set_visible_user_level (NAUTILUS_PREFERENCES_USE_PUBLIC_METADATA,
-						     NAUTILUS_USER_LEVEL_HACKER);
+						     NAUTILUS_USER_LEVEL_ADVANCED);
 
 	nautilus_preferences_set_visible_user_level (NAUTILUS_PREFERENCES_SEARCH_BAR_TYPE,
 						     NAUTILUS_USER_LEVEL_INTERMEDIATE);
 
-	nautilus_preferences_set_visible_user_level (NAUTILUS_PREFERENCES_SEARCH_METHOD,
+	nautilus_preferences_set_visible_user_level (NAUTILUS_PREFERENCES_USE_FAST_SEARCH,
 						     NAUTILUS_USER_LEVEL_INTERMEDIATE);
 
 	nautilus_preferences_set_visible_user_level (NAUTILUS_PREFERENCES_SEARCH_WEB_URI,
@@ -456,9 +493,9 @@ global_preferences_create_dialog (void)
 	GtkWidget		*prefs_dialog;
 	NautilusPreferencesBox	*preference_box;
 	GtkWidget		*directory_views_pane;
+	GtkWidget		*windows_and_desktop_pane;
 	GtkWidget		*sidebar_panels_pane;
 	GtkWidget		*appearance_pane;
-	GtkWidget		*file_indexing_pane;
 	GtkWidget		*tradeoffs_pane;
 	GtkWidget		*navigation_pane;
 
@@ -501,117 +538,106 @@ global_preferences_create_dialog (void)
 							 NAUTILUS_PREFERENCES_DIRECTORY_VIEW_FONT_FAMILY,
 							 NAUTILUS_PREFERENCE_ITEM_FONT_FAMILY);
 
-	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (appearance_pane), _("Views"));
-	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (appearance_pane),
-							 2,
-							 NAUTILUS_PREFERENCES_START_WITH_TOOL_BAR,
-							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
-	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (appearance_pane),
-							 2,
-							 NAUTILUS_PREFERENCES_START_WITH_LOCATION_BAR,
-							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
-	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (appearance_pane),
-							 2,
-							 NAUTILUS_PREFERENCES_START_WITH_STATUS_BAR,
-							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
-	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (appearance_pane),
-							 2,
-							 NAUTILUS_PREFERENCES_START_WITH_SIDEBAR,
-							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
-	
-	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (appearance_pane), _("Desktop"));
-	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (appearance_pane),
-							 3,
+
+	/*
+	 * Windows & Desktop pane
+	 */
+	windows_and_desktop_pane = nautilus_preferences_box_add_pane (preference_box,
+								      _("Windows & Desktop"),
+								      _("Windows & Desktop Settings"));
+
+	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (windows_and_desktop_pane), _("Desktop"));
+	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (windows_and_desktop_pane),
+							 0,
 							 NAUTILUS_PREFERENCES_SHOW_DESKTOP,
 							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
 
+	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (windows_and_desktop_pane), _("Opening New Windows"));
+	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (windows_and_desktop_pane),
+							 1,
+							 NAUTILUS_PREFERENCES_WINDOW_ALWAYS_NEW,
+							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
+	
+	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (windows_and_desktop_pane),
+							 1,
+							 NAUTILUS_PREFERENCES_START_WITH_TOOLBAR,
+							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
+	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (windows_and_desktop_pane),
+							 1,
+							 NAUTILUS_PREFERENCES_START_WITH_LOCATION_BAR,
+							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
+	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (windows_and_desktop_pane),
+							 1,
+							 NAUTILUS_PREFERENCES_START_WITH_STATUS_BAR,
+							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
+	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (windows_and_desktop_pane),
+							 1,
+							 NAUTILUS_PREFERENCES_START_WITH_SIDEBAR,
+							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
+	
+	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (windows_and_desktop_pane), _("Trash Behavior"));
+	
+	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (windows_and_desktop_pane),
+							 2,
+							 NAUTILUS_PREFERENCES_CONFIRM_TRASH,
+							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
+	
 	
 	/*
 	 * Folder Views pane
 	 */
 	directory_views_pane = nautilus_preferences_box_add_pane (preference_box,
-								 _("Folder Views"),
-								 _("Folder Views Settings"));
-	
-	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane), _("Window Behavior"));
-	
-	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane),
-							 0,
-							 NAUTILUS_PREFERENCES_WINDOW_ALWAYS_NEW,
-							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
+								 _("Icon & List Views"),
+								 _("Icon & List Views Settings"));
 	
 	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane), _("Click Behavior"));
 	
 	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane),
-							 1,
+							 0,
 							 NAUTILUS_PREFERENCES_CLICK_POLICY,
 							 NAUTILUS_PREFERENCE_ITEM_ENUM);
 
-	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane), _("Trash Behavior"));
+	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane), _("Executable Text Files"));
+	
+	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane),
+							 1,
+							 NAUTILUS_PREFERENCES_EXECUTABLE_TEXT_ACTIVATION,
+							 NAUTILUS_PREFERENCE_ITEM_ENUM);
+
+	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane), _("Show/Hide Options"));
 	
 	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane),
 							 2,
-							 NAUTILUS_PREFERENCES_CONFIRM_TRASH,
-							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
-	
-	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane), _("Display"));
-	
-	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane),
-							 3,
-							 NAUTILUS_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES,
-							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
-
-	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane),
-							 3,
 							 NAUTILUS_PREFERENCES_SHOW_HIDDEN_FILES,
 							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
 
 	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane),
-							 3,
+							 2,
 							 NAUTILUS_PREFERENCES_SHOW_BACKUP_FILES,
 							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
 
 	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (directory_views_pane),
-							 3,
+							 2,
 							 NAUTILUS_PREFERENCES_SHOW_SPECIAL_FLAGS,
 							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
 
-	/*
-	 * Search Settings 
-	 */
-	file_indexing_pane = nautilus_preferences_box_add_pane (preference_box,
-								_("Search"),
-								_("Search Settings"));
-	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (file_indexing_pane),
-					     _("Search Complexity Options"));
-	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (file_indexing_pane),
-							 0,
-							 NAUTILUS_PREFERENCES_SEARCH_BAR_TYPE,
-							 NAUTILUS_PREFERENCE_ITEM_ENUM);
-	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (file_indexing_pane),
-					     _("Search Tradeoffs"));
-	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (file_indexing_pane),
-							 1,
-							 NAUTILUS_PREFERENCES_SEARCH_METHOD,
-							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
-	
-	
-	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (file_indexing_pane),
-					     _("Search Locations"));
-	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (file_indexing_pane),
-							 2,
-							 NAUTILUS_PREFERENCES_SEARCH_WEB_URI,
-							 NAUTILUS_PREFERENCE_ITEM_EDITABLE_STRING);
-					
 	/*
 	 * Sidebar panels pane
 	 */
 	sidebar_panels_pane = nautilus_preferences_box_add_pane (preference_box,
 								 _("Sidebar Panels"),
-								 _("Sidebar Panels Description"));
+								 _("Sidebar Panels Settings"));
 	
 	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (sidebar_panels_pane), 
-					     _("Choose which panels should appear in the sidebar"));
+					     _("Tabs"));
+
+	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (sidebar_panels_pane), 
+					     _("Tree"));
+
+	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (sidebar_panels_pane),
+							 1,
+							 NAUTILUS_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES,
+							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
 
 	{
 		char *preference_key;
@@ -643,6 +669,11 @@ global_preferences_create_dialog (void)
 		nautilus_view_identifier_list_free (view_identifiers);
 	}
 
+	/*
+	 * Search Settings 
+	 */
+	global_preferences_create_search_pane (preference_box);
+
 	/*
 	 * Navigation
 	 */
@@ -650,14 +681,14 @@ global_preferences_create_dialog (void)
 							    _("Navigation"),
 							    _("Navigation Settings"));
 
-	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (navigation_pane), _("Home Location"));
+	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (navigation_pane), _("Home"));
 	
 	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (navigation_pane),
 							 0,
 							 NAUTILUS_PREFERENCES_HOME_URI,
 							 NAUTILUS_PREFERENCE_ITEM_EDITABLE_STRING);
 
-	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (navigation_pane), _("Proxy Settings"));
+	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (navigation_pane), _("HTTP Proxy Settings"));
 	
 	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (navigation_pane),
 							 1,
@@ -697,17 +728,24 @@ global_preferences_create_dialog (void)
 							 NAUTILUS_PREFERENCES_SHOW_TEXT_IN_ICONS,
 							 NAUTILUS_PREFERENCE_ITEM_SHORT_ENUM);
 
-	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (tradeoffs_pane), _("Show Thumbnails for Image Files"));
+	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (tradeoffs_pane), _("Show Count of Items in Folders"));
 	
 	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (tradeoffs_pane),
 							 1,
-							 NAUTILUS_PREFERENCES_SHOW_IMAGE_FILE_THUMBNAILS,
+							 NAUTILUS_PREFERENCES_SHOW_DIRECTORY_ITEM_COUNTS,
 							 NAUTILUS_PREFERENCE_ITEM_SHORT_ENUM);
 
-	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (tradeoffs_pane), _("Previewing Sound Files"));
+	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (tradeoffs_pane), _("Show Thumbnails for Image Files"));
 	
 	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (tradeoffs_pane),
 							 2,
+							 NAUTILUS_PREFERENCES_SHOW_IMAGE_FILE_THUMBNAILS,
+							 NAUTILUS_PREFERENCE_ITEM_SHORT_ENUM);
+
+	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (tradeoffs_pane), _("Preview Sound Files"));
+	
+	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (tradeoffs_pane),
+							 3,
 							 NAUTILUS_PREFERENCES_PREVIEW_SOUND,
 							 NAUTILUS_PREFERENCE_ITEM_SHORT_ENUM);
 
@@ -716,7 +754,7 @@ global_preferences_create_dialog (void)
 	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (tradeoffs_pane), _("Make Folder Appearance Details Public"));
 	
 	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (tradeoffs_pane),
-							 3,
+							 4,
 							 NAUTILUS_PREFERENCES_USE_PUBLIC_METADATA,
 							 NAUTILUS_PREFERENCE_ITEM_SHORT_ENUM);
 
@@ -727,6 +765,64 @@ global_preferences_create_dialog (void)
 	return prefs_dialog;
 }
 
+/* Update the sensitivity of the search pane when the medusa blocked state changes */
+static void
+global_preferences_medusa_blocked_changed_callback (gpointer callback_data)
+{
+	gboolean medusa_blocked;
+	
+	g_return_if_fail (GTK_IS_WIDGET (callback_data));
+
+	medusa_blocked = nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_MEDUSA_BLOCKED);
+
+	gtk_widget_set_sensitive (GTK_WIDGET (callback_data), !medusa_blocked);
+}
+
+static GtkWidget *
+global_preferences_create_search_pane (NautilusPreferencesBox *preference_box)
+{
+	GtkWidget *search_pane;
+	GtkWidget *fast_search_group;
+	
+	g_return_val_if_fail (NAUTILUS_IS_PREFERENCES_BOX (preference_box), NULL);
+
+ 	/*
+ 	 * Search Settings 
+ 	 */
+	search_pane = nautilus_preferences_box_add_pane (preference_box,
+								_("Search"),
+								_("Search Settings"));
+	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (search_pane),
+					     _("Search Complexity Options"));
+	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (search_pane),
+							 0,
+							 NAUTILUS_PREFERENCES_SEARCH_BAR_TYPE,
+							 NAUTILUS_PREFERENCE_ITEM_ENUM);
+	fast_search_group = nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (search_pane),
+								 _("Fast Search"));
+	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (search_pane),
+							 1,
+							 NAUTILUS_PREFERENCES_USE_FAST_SEARCH,
+							 NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
+	nautilus_preferences_pane_add_group (NAUTILUS_PREFERENCES_PANE (search_pane),
+					     _("Search Engines"));
+	nautilus_preferences_pane_add_item_to_nth_group (NAUTILUS_PREFERENCES_PANE (search_pane),
+							 2,
+							 NAUTILUS_PREFERENCES_SEARCH_WEB_URI,
+							 NAUTILUS_PREFERENCE_ITEM_EDITABLE_STRING);
+
+	/* Setup callbacks so that we can update the sensitivity of
+	 * the search pane when the medusa blocked state changes
+	 */
+	nautilus_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_MEDUSA_BLOCKED,
+						       global_preferences_medusa_blocked_changed_callback,
+						       fast_search_group,
+						       GTK_OBJECT (fast_search_group));
+	global_preferences_medusa_blocked_changed_callback (fast_search_group);
+
+	return search_pane;
+}
+
 /* Make a query to find out what sidebar panels are available. */
 static GList *
 global_preferences_get_sidebar_panel_view_identifiers (void)
@@ -821,14 +917,14 @@ static struct
 	const char *name;
 	gboolean novice_default;
 	gboolean intermediate_default;
-	gboolean hacker_default;
+	gboolean advanced_default;
 	int visible_user_level;
 } known_sidebar_panels[] =
 {
-	{ "OAFIID:nautilus_notes_view:7f04c3cb-df79-4b9a-a577-38b19ccd4185",       TRUE,  TRUE,  TRUE,  NAUTILUS_USER_LEVEL_NOVICE},
-	{ "OAFIID:hyperbola_navigation_tree:57542ce0-71ff-442d-a764-462c92514234", TRUE,  TRUE,  TRUE,  NAUTILUS_USER_LEVEL_NOVICE },
-	{ "OAFIID:nautilus_history_view:a7a85bdd-2ecf-4bc1-be7c-ed328a29aacb",     TRUE,  TRUE,  TRUE,  NAUTILUS_USER_LEVEL_NOVICE },
-	{ "OAFIID:nautilus_tree_view:2d826a6e-1669-4a45-94b8-23d65d22802d",        FALSE, FALSE, FALSE, NAUTILUS_USER_LEVEL_NOVICE },
+	{ "OAFIID:nautilus_notes_view:7f04c3cb-df79-4b9a-a577-38b19ccd4185",       TRUE,  TRUE,  TRUE,  NAUTILUS_USER_LEVEL_INTERMEDIATE},
+	{ "OAFIID:hyperbola_navigation_tree:57542ce0-71ff-442d-a764-462c92514234", TRUE,  TRUE,  TRUE,  NAUTILUS_USER_LEVEL_INTERMEDIATE },
+	{ "OAFIID:nautilus_history_view:a7a85bdd-2ecf-4bc1-be7c-ed328a29aacb",     TRUE,  TRUE,  TRUE,  NAUTILUS_USER_LEVEL_INTERMEDIATE },
+	{ "OAFIID:nautilus_tree_view:2d826a6e-1669-4a45-94b8-23d65d22802d",        FALSE, TRUE,  TRUE,  NAUTILUS_USER_LEVEL_INTERMEDIATE },
 };
 
 static void
@@ -847,8 +943,8 @@ global_preferences_install_sidebar_panel_defaults (void)
 							  NAUTILUS_USER_LEVEL_INTERMEDIATE,
 							  known_sidebar_panels[i].intermediate_default);
 		nautilus_preferences_default_set_boolean (key,
-							  NAUTILUS_USER_LEVEL_HACKER,
-							  known_sidebar_panels[i].hacker_default);
+							  NAUTILUS_USER_LEVEL_ADVANCED,
+							  known_sidebar_panels[i].advanced_default);
 
 		nautilus_preferences_set_visible_user_level (key,
 							     known_sidebar_panels[i].visible_user_level);
@@ -862,22 +958,24 @@ global_preferences_install_sidebar_panel_descriptions (void)
 {
  	GList *view_identifiers;
  	GList *iterator;
+ 	NautilusViewIdentifier *identifier;
+ 	char *key, *label;
 	
 	/* Install the descriptions for the available sidebar panels */
  	view_identifiers = global_preferences_get_sidebar_panel_view_identifiers ();
 
  	for (iterator = view_identifiers; iterator != NULL; iterator = iterator->next) {
- 		NautilusViewIdentifier *identifier;
- 		char *key;
 
  		identifier = iterator->data;
  		g_return_if_fail (identifier != NULL);
 		
  		key = global_preferences_get_sidebar_panel_key (identifier->iid);
  		g_return_if_fail (key != NULL);
-		
- 		nautilus_preferences_set_description (key, identifier->name);
+
+		label = g_strdup_printf (_("Display %s tab in sidebar"), identifier->name);
+ 		nautilus_preferences_set_description (key, label);
  		g_free (key);
+ 		g_free (label);
  	}
 
  	nautilus_view_identifier_list_free (view_identifiers);
@@ -959,6 +1057,54 @@ global_preferences_install_home_location_defaults (void)
 	g_free (default_intermediate_home_uri);
 }
 
+static void
+global_preferences_use_fast_search_changed_callback (gpointer callback_data)
+{
+	gboolean use_fast_search;
+
+	use_fast_search = nautilus_preferences_get_boolean (NAUTILUS_PREFERENCES_USE_FAST_SEARCH);
+
+	nautilus_medusa_enable_services (use_fast_search);
+}
+
+static void 
+global_preferences_medusa_state_changed_callback (gpointer callback_data)
+{
+	nautilus_preferences_set_boolean (NAUTILUS_PREFERENCES_USE_FAST_SEARCH,
+					  nautilus_medusa_services_have_been_enabled_by_user ());
+
+	nautilus_preferences_set_boolean (NAUTILUS_PREFERENCES_MEDUSA_BLOCKED,
+					  nautilus_medusa_blocked ());
+}
+
+static void
+global_preferences_install_medusa_defaults (void)
+{
+	gboolean medusa_blocked;
+	gboolean use_fast_search;
+
+	medusa_blocked = nautilus_medusa_blocked ();
+	use_fast_search = nautilus_medusa_services_have_been_enabled_by_user ();
+
+	/* This one controls the sensitivity */
+	nautilus_preferences_default_set_boolean (NAUTILUS_PREFERENCES_MEDUSA_BLOCKED,
+						  NAUTILUS_USER_LEVEL_NOVICE,
+						  medusa_blocked);
+
+	/* This is the one that appears in the preferences dialog */
+	nautilus_preferences_default_set_boolean (NAUTILUS_PREFERENCES_USE_FAST_SEARCH,
+						  NAUTILUS_USER_LEVEL_NOVICE,
+						  use_fast_search);
+
+
+	nautilus_preferences_add_callback (NAUTILUS_PREFERENCES_USE_FAST_SEARCH,
+					   global_preferences_use_fast_search_changed_callback,
+					   NULL);
+
+	nautilus_medusa_add_system_state_changed_callback (global_preferences_medusa_state_changed_callback,
+							   NULL);
+}
+
 static gboolean
 global_preferences_close_dialog_callback (GtkWidget   *dialog,
 					  gpointer    user_data)
