@@ -33,11 +33,15 @@
 
 #include <gtk/gtkcheckbutton.h>
 #include <gtk/gtksignal.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtkmain.h>
 
 #include "nautilus-radio-button-group.h"
 #include "nautilus-string-picker.h"
 #include "nautilus-text-caption.h"
 
+#include "nautilus-global-preferences.h"
+
 /* Arguments */
 enum
 {
@@ -50,12 +54,15 @@ enum
 static const guint PREFERENCES_ITEM_TITLE_SPACING = 4;
 static const guint PREFERENCES_ITEM_FRAME_BORDER_WIDTH = 6;
 static const NautilusPreferencesItemType PREFERENCES_ITEM_UNDEFINED_ITEM = -1U;
+static gboolean text_idle_handler = FALSE;
+static gboolean integer_idle_handler = FALSE;
 
 struct _NautilusPreferencesItemDetails
 {
 	gchar				*preference_name;
 	NautilusPreferencesItemType	item_type;
 	GtkWidget			*child;
+	guint				change_signal_ID;
 };
 
 /* NautilusPreferencesItemClass methods */
@@ -73,34 +80,36 @@ static void preferences_item_get_arg                   (GtkObject
 							guint                         arg_id);
 
 /* Private stuff */
-static void preferences_item_construct                 (NautilusPreferencesItem      *item,
-							const gchar                  *preference_name,
-							NautilusPreferencesItemType   item_type);
-static void preferences_item_create_enum               (NautilusPreferencesItem      *item,
-							const char     *preference_name);
-static void preferences_item_create_short_enum         (NautilusPreferencesItem      *item,
-							const char     *preference_name);
-static void preferences_item_create_boolean            (NautilusPreferencesItem      *item,
-							const char     *preference_name);
-static void preferences_item_create_editable_string    (NautilusPreferencesItem      *item,
-							const char     *preference_name);
-static void preferences_item_create_integer    (NautilusPreferencesItem      *item,
-							const char     *preference_name);
+static void preferences_item_construct                        (NautilusPreferencesItem      *item,
+							       const gchar                  *preference_name,
+							       NautilusPreferencesItemType   item_type);
+static void preferences_item_create_enum                      (NautilusPreferencesItem      *item,
+							       const char                   *preference_name);
+static void preferences_item_create_short_enum                (NautilusPreferencesItem      *item,
+							       const char                   *preference_name);
+static void preferences_item_create_boolean                   (NautilusPreferencesItem      *item,
+							       const char                   *preference_name);
+static void preferences_item_create_editable_string           (NautilusPreferencesItem      *item,
+							       const char                   *preference_name);
+static void preferences_item_create_integer                   (NautilusPreferencesItem      *item,
+							       const char                   *preference_name);
 static void preferences_item_create_font_family               (NautilusPreferencesItem      *item,
-							const char     *preference_name);
-static void preferences_item_create_theme	       (NautilusPreferencesItem      *item,
-							const char     *preference_name);
-static void enum_radio_group_changed_callback          (GtkWidget                    *button_group,
-							GtkWidget                    *button,
-							gpointer                      user_data);
-static void boolean_button_toggled_callback            (GtkWidget                    *button_group,
-							gpointer                      user_data);
-static void text_item_changed_callback                 (GtkWidget                    *string_picker,
-							gpointer                      user_data);
-static void editable_string_changed_callback           (GtkWidget                    *caption,
-							gpointer                      user_data);
-static void integer_changed_callback           (GtkWidget                    *caption,
-							gpointer                      user_data);
+							       const char                   *preference_name);
+static void preferences_item_create_theme	              (NautilusPreferencesItem      *item,
+							       const char                   *preference_name);
+static void preferences_item_update_text_settings_at_idle     (NautilusPreferencesItem      *preferences_item);
+static void preferences_item_update_integer_settings_at_idle  (NautilusPreferencesItem      *preferences_item);
+static void enum_radio_group_changed_callback                 (GtkWidget                    *button_group,
+							       GtkWidget                    *button,
+							       gpointer                      user_data);
+static void boolean_button_toggled_callback                   (GtkWidget                    *button_group,
+							       gpointer                      user_data);
+static void text_item_changed_callback                        (GtkWidget                    *string_picker,
+							       gpointer                      user_data);
+static void editable_string_changed_callback                  (GtkWidget                    *caption,
+							       gpointer                      user_data);
+static void integer_changed_callback                          (GtkWidget                    *caption,
+							       gpointer                      user_data);
 
 NAUTILUS_DEFINE_CLASS_BOILERPLATE (NautilusPreferencesItem, nautilus_preferences_item, GTK_TYPE_VBOX)
 
@@ -257,6 +266,7 @@ preferences_item_construct (NautilusPreferencesItem	*item,
 	g_assert (item->details->preference_name == NULL);
 	
 	item->details->preference_name = g_strdup (preference_name);
+	item->details->item_type = item_type;
 
 	/* Create the child widget according to the item type */
 	switch (item_type)
@@ -289,6 +299,9 @@ preferences_item_construct (NautilusPreferencesItem	*item,
 	}
 
 	g_assert (item->details->child != NULL);
+	g_assert (item->details->change_signal_ID != 0);
+
+	nautilus_preferences_item_update_displayed_value (item);
 
 	gtk_box_pack_start (GTK_BOX (item),
 			    item->details->child,
@@ -299,12 +312,31 @@ preferences_item_construct (NautilusPreferencesItem	*item,
 	gtk_widget_show (item->details->child);
 }
 
+static void
+preferences_item_update_enum (const NautilusPreferencesItem *item)
+{
+	gint value;
+	char *preference_name;
+	guint i;
+
+	g_assert (NAUTILUS_IS_PREFERENCES_ITEM (item));
+	g_assert (item->details->item_type == NAUTILUS_PREFERENCE_ITEM_ENUM);
+
+	preference_name = item->details->preference_name;
+ 	value = nautilus_preferences_get_integer (preference_name);
+
+	for (i = 0; i < nautilus_preferences_enumeration_get_num_entries (preference_name); i++) {
+		if (value == nautilus_preferences_enumeration_get_nth_value (preference_name, i)) {
+			nautilus_radio_button_group_set_active_index (NAUTILUS_RADIO_BUTTON_GROUP (item->details->child), i);
+		}
+	}
+}
+
 static void
 preferences_item_create_enum (NautilusPreferencesItem	*item,
 			      const char 	*preference_name)
 {
 	guint	i;
-	gint	value;
 
 	g_assert (item != NULL);
 	g_assert (preference_name != NULL);
@@ -313,8 +345,6 @@ preferences_item_create_enum (NautilusPreferencesItem	*item,
 
 	item->details->child = nautilus_radio_button_group_new (FALSE);
 		
- 	value = nautilus_preferences_get_integer (item->details->preference_name);
-	
 	for (i = 0; i < nautilus_preferences_enumeration_get_num_entries (preference_name); i++) {
 		char *description;
 		
@@ -326,17 +356,33 @@ preferences_item_create_enum (NautilusPreferencesItem	*item,
 						    description);
 
 		g_free (description);
-		
+	}
+	
+	item->details->change_signal_ID = 
+		gtk_signal_connect (GTK_OBJECT (item->details->child),
+			    	    "changed",
+			    	    GTK_SIGNAL_FUNC (enum_radio_group_changed_callback),
+			    	    (gpointer) item);
+}
+
+static void
+preferences_item_update_short_enum (const NautilusPreferencesItem *item)
+{
+	gint value;
+	char *preference_name;
+	guint i;
+
+	g_assert (NAUTILUS_IS_PREFERENCES_ITEM (item));
+	g_assert (item->details->item_type == NAUTILUS_PREFERENCE_ITEM_SHORT_ENUM);
+
+	preference_name = item->details->preference_name;
+ 	value = nautilus_preferences_get_integer (preference_name);
+
+	for (i = 0; i < nautilus_preferences_enumeration_get_num_entries (preference_name); i++) {
 		if (value == nautilus_preferences_enumeration_get_nth_value (preference_name, i)) {
-			
 			nautilus_radio_button_group_set_active_index (NAUTILUS_RADIO_BUTTON_GROUP (item->details->child), i);
 		}
 	}
-	
-	gtk_signal_connect (GTK_OBJECT (item->details->child),
-			    "changed",
-			    GTK_SIGNAL_FUNC (enum_radio_group_changed_callback),
-			    (gpointer) item);
 }
 
 /* This is just like preferences_item_create_enum except the choices
@@ -348,7 +394,6 @@ preferences_item_create_short_enum (NautilusPreferencesItem	*item,
 			      	    const char 	*preference_name)
 {
 	guint	i;
-	gint	value;
 
 	g_assert (item != NULL);
 	g_assert (preference_name != NULL);
@@ -357,8 +402,6 @@ preferences_item_create_short_enum (NautilusPreferencesItem	*item,
 
 	item->details->child = nautilus_radio_button_group_new (TRUE);
 		
- 	value = nautilus_preferences_get_integer (item->details->preference_name);
-	
 	for (i = 0; i < nautilus_preferences_enumeration_get_num_entries (preference_name); i++) {
 		char *description;
 		
@@ -370,27 +413,34 @@ preferences_item_create_short_enum (NautilusPreferencesItem	*item,
 						    description);
 
 		g_free (description);
-		
-		if (value == nautilus_preferences_enumeration_get_nth_value (preference_name, i)) {
-			
-			nautilus_radio_button_group_set_active_index (NAUTILUS_RADIO_BUTTON_GROUP (item->details->child), i);
-		}
 	}
 	
-	gtk_signal_connect (GTK_OBJECT (item->details->child),
-			    "changed",
-			    GTK_SIGNAL_FUNC (enum_radio_group_changed_callback),
-			    (gpointer) item);
+	item->details->change_signal_ID = 
+		gtk_signal_connect (GTK_OBJECT (item->details->child),
+			    	    "changed",
+			    	    GTK_SIGNAL_FUNC (enum_radio_group_changed_callback),
+			    	    (gpointer) item);
+}
+
+static void
+preferences_item_update_boolean (const NautilusPreferencesItem *item)
+{
+	gboolean value;
+
+	g_assert (NAUTILUS_IS_PREFERENCES_ITEM (item));
+	g_assert (item->details->item_type == NAUTILUS_PREFERENCE_ITEM_BOOLEAN);
+
+	value = nautilus_preferences_get_boolean (item->details->preference_name);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (item->details->child), value);
 }
 
 static void
 preferences_item_create_boolean (NautilusPreferencesItem	*item,
 				 const char 	*preference_name)
 {
-	gboolean	value;
 	char		*description;
 
-	g_assert (item != NULL);
+	g_assert (NAUTILUS_IS_PREFERENCES_ITEM (item));
 	g_assert (preference_name != NULL);
 
 	g_assert (item->details->preference_name != NULL);
@@ -399,57 +449,85 @@ preferences_item_create_boolean (NautilusPreferencesItem	*item,
 	g_assert (description != NULL);
 
 	item->details->child = gtk_check_button_new_with_label (description);
+	gtk_label_set_justify (GTK_LABEL (GTK_BIN (item->details->child)->child), GTK_JUSTIFY_LEFT);
 
 	g_free (description);
-
-	value = nautilus_preferences_get_boolean (item->details->preference_name);
-
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (item->details->child), value);
 				      
-	gtk_signal_connect (GTK_OBJECT (item->details->child),
-			    "toggled",
-			    GTK_SIGNAL_FUNC (boolean_button_toggled_callback),
-			    (gpointer) item);
+	item->details->change_signal_ID = 
+		gtk_signal_connect (GTK_OBJECT (item->details->child),
+			    	    "toggled",
+			    	    GTK_SIGNAL_FUNC (boolean_button_toggled_callback),
+			    	    (gpointer) item);
 }
 
 static void
-preferences_item_create_editable_string (NautilusPreferencesItem	*item,
-					 const char 	*preference_name)
+preferences_item_update_editable_string (const NautilusPreferencesItem *item)
 {
 	char	*current_value;
-	char	*description;
-	
-	g_assert (item != NULL);
-	g_assert (preference_name != NULL);
 
-	g_assert (item->details->preference_name != NULL);
-	description = nautilus_preferences_get_description (preference_name);
-	
-	g_assert (description != NULL);
-	
-	item->details->child = nautilus_text_caption_new ();
-
-	nautilus_caption_set_title_label (NAUTILUS_CAPTION (item->details->child), description);
-
-	g_free (description);
+	g_assert (NAUTILUS_IS_PREFERENCES_ITEM (item));
+	g_assert (item->details->item_type == NAUTILUS_PREFERENCE_ITEM_EDITABLE_STRING);
 
 	current_value = nautilus_preferences_get (item->details->preference_name);
 
 	g_assert (current_value != NULL);
 	nautilus_text_caption_set_text (NAUTILUS_TEXT_CAPTION (item->details->child), current_value);
 	g_free (current_value);
+}
+
+static void
+preferences_item_create_editable_string (NautilusPreferencesItem	*item,
+					 const char 	*preference_name)
+{
+	char	*description;
 	
- 	gtk_signal_connect (GTK_OBJECT (item->details->child),
- 			    "changed",
- 			    GTK_SIGNAL_FUNC (editable_string_changed_callback),
- 			    (gpointer) item);
+	g_assert (item != NULL);
+	g_assert (preference_name != NULL);
+
+	g_assert (item->details->preference_name != NULL);
+	description = nautilus_preferences_get_description (preference_name);
+	
+	g_assert (description != NULL);
+	
+	item->details->child = nautilus_text_caption_new ();
+
+	/* FIXME This is a special case for the home uri preference,
+	   in the future this should be generalized. */
+	if (g_strcasecmp (preference_name, NAUTILUS_PREFERENCES_HOME_URI) == 0)
+	{
+		nautilus_text_caption_set_expand_tilde (NAUTILUS_TEXT_CAPTION (item->details->child), TRUE);
+	}
+
+	nautilus_caption_set_title_label (NAUTILUS_CAPTION (item->details->child), description);
+
+	g_free (description);
+	
+	item->details->change_signal_ID = 
+		gtk_signal_connect (GTK_OBJECT (item->details->child),
+	 			    "changed",
+	 			    GTK_SIGNAL_FUNC (editable_string_changed_callback),
+			    	    (gpointer) item);
+}
+
+static void
+preferences_item_update_integer (const NautilusPreferencesItem *item)
+{
+	char *current_value;
+
+	g_assert (NAUTILUS_IS_PREFERENCES_ITEM (item));
+	g_assert (item->details->item_type == NAUTILUS_PREFERENCE_ITEM_INTEGER);
+
+	current_value = g_strdup_printf ("%d", nautilus_preferences_get_integer (item->details->preference_name));
+
+	g_assert (current_value != NULL);
+	nautilus_text_caption_set_text (NAUTILUS_TEXT_CAPTION (item->details->child), current_value);
+	g_free (current_value);
 }
 
 static void
 preferences_item_create_integer (NautilusPreferencesItem	*item,
 				 const char 	*preference_name)
 {
-	char *current_value;
 	char	*description;
 	
 	g_assert (item != NULL);
@@ -466,16 +544,28 @@ preferences_item_create_integer (NautilusPreferencesItem	*item,
 
 	g_free (description);
 
-	current_value = g_strdup_printf ("%d", nautilus_preferences_get_integer (item->details->preference_name));
+	item->details->change_signal_ID = 
+		gtk_signal_connect (GTK_OBJECT (item->details->child),
+	 			    "changed",
+	 			    GTK_SIGNAL_FUNC (integer_changed_callback),
+			    	    (gpointer) item);
+}
+
+static void
+preferences_item_update_font_family (const NautilusPreferencesItem *item)
+{
+	char *current_value;
+
+	g_assert (NAUTILUS_IS_PREFERENCES_ITEM (item));
+	g_assert (item->details->item_type == NAUTILUS_PREFERENCE_ITEM_FONT_FAMILY);
+
+	current_value = nautilus_preferences_get (item->details->preference_name);
 
 	g_assert (current_value != NULL);
-	nautilus_text_caption_set_text (NAUTILUS_TEXT_CAPTION (item->details->child), current_value);
+
+	nautilus_string_picker_set_selected_string (NAUTILUS_STRING_PICKER (item->details->child), current_value);
+
 	g_free (current_value);
-	
- 	gtk_signal_connect (GTK_OBJECT (item->details->child),
- 			    "changed",
- 			    GTK_SIGNAL_FUNC (integer_changed_callback),
- 			    (gpointer) item);
 }
 
 static void
@@ -483,7 +573,6 @@ preferences_item_create_font_family (NautilusPreferencesItem	*item,
 				     const char 	*preference_name)
 {
 	char			*description;
-	char			*current_value;
 	NautilusStringList	*font_list;
 
 	g_assert (item != NULL);
@@ -509,22 +598,13 @@ preferences_item_create_font_family (NautilusPreferencesItem	*item,
 	nautilus_string_list_insert (font_list, "lucida");
 
 	nautilus_string_picker_set_string_list (NAUTILUS_STRING_PICKER (item->details->child), font_list);
-
-	current_value = nautilus_preferences_get (item->details->preference_name);
-
-	g_assert (current_value != NULL);
-	g_assert (nautilus_string_list_contains (font_list, current_value));
-
-	nautilus_string_picker_set_selected_string (NAUTILUS_STRING_PICKER (item->details->child), current_value);
-
-	g_free (current_value);
-
 	nautilus_string_list_free (font_list);
-	
- 	gtk_signal_connect (GTK_OBJECT (item->details->child),
- 			    "changed",
- 			    GTK_SIGNAL_FUNC (text_item_changed_callback),
- 			    (gpointer) item);
+
+	item->details->change_signal_ID = 
+		gtk_signal_connect (GTK_OBJECT (item->details->child),
+	 			    "changed",
+	 			    GTK_SIGNAL_FUNC (text_item_changed_callback),
+			    	    (gpointer) item);
 }
 
 /* utility to determine if an image file exists in the candidate directory */
@@ -540,32 +620,30 @@ static const char *icon_file_name_suffixes[] =
 static gboolean
 has_image_file(const char *directory_uri, const char *dir_name, const char *required_file)
 {
-	char *temp_str, *base_uri;
+	char *temp_str, *base_uri_string;
 	guint index;
-	GnomeVFSResult result;
-	GnomeVFSFileInfo *file_info;
+	gboolean result;
+	GnomeVFSURI *base_uri, *uri;
 	
-	file_info = gnome_vfs_file_info_new ();
-	
-	temp_str = nautilus_make_path(directory_uri, dir_name);
-	base_uri = nautilus_make_path(temp_str, required_file);
-	g_free(temp_str);
+	temp_str = nautilus_make_path (directory_uri, dir_name);
+	base_uri_string = nautilus_make_path (temp_str, required_file);
+	base_uri = gnome_vfs_uri_new (base_uri_string);
+	g_free (temp_str);
+	g_free (base_uri_string);
 	
+	result = FALSE;
 	for (index = 0; index < NAUTILUS_N_ELEMENTS (icon_file_name_suffixes); index++) {
-		temp_str = g_strconcat (base_uri, icon_file_name_suffixes[index], NULL);
-		gnome_vfs_file_info_init (file_info);
-		result = gnome_vfs_get_file_info (temp_str, file_info, 0);
-		g_free(temp_str);
-		if (result == GNOME_VFS_OK) {
-			g_free(base_uri);
-			gnome_vfs_file_info_unref (file_info);
-			return TRUE;	
-		}
+		uri = gnome_vfs_uri_append_string (base_uri, icon_file_name_suffixes[index]);
+		result = gnome_vfs_uri_exists (uri);
+		gnome_vfs_uri_unref (uri);
+		if (result) {
+			break;
+		}		
 	}
 	
-	gnome_vfs_file_info_unref (file_info);
-	g_free(base_uri);
-	return FALSE;
+	gnome_vfs_uri_unref (base_uri);
+	g_free (base_uri);
+	return result;
 }
 
 /* add available icon themes to the theme list by iterating through the
@@ -576,7 +654,7 @@ add_icon_themes(NautilusStringList *theme_list, char *required_file)
 	char *directory_uri;
 	GnomeVFSResult result;
 	GnomeVFSFileInfo *current_file_info;
-	GnomeVFSDirectoryList *list;
+	GList *list, *element;
 	char *pixmap_directory;
 		
 	pixmap_directory = nautilus_get_pixmap_directory ();
@@ -593,17 +671,34 @@ add_icon_themes(NautilusStringList *theme_list, char *required_file)
 	}
 
 	/* interate through the directory for each file */
-	current_file_info = gnome_vfs_directory_list_first(list);
-	while (current_file_info != NULL) {
+	for (element = list; element != NULL; element = element->next) {
+		current_file_info = element->data;
 		if ((current_file_info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) &&
-			(current_file_info->name[0] != '.'))
+			(current_file_info->name[0] != '.')) {
 			if (has_image_file(directory_uri, current_file_info->name, required_file))
-				nautilus_string_list_insert (theme_list, current_file_info->name);	
-		current_file_info = gnome_vfs_directory_list_next(list);
+				nautilus_string_list_insert (theme_list, current_file_info->name);
+		}
 	}
 	
 	g_free(directory_uri);
-	gnome_vfs_directory_list_destroy(list);
+	gnome_vfs_file_info_list_free (list);	
+}
+
+static void
+preferences_item_update_theme (const NautilusPreferencesItem *item)
+{
+	char *current_value;
+
+	g_assert (NAUTILUS_IS_PREFERENCES_ITEM (item));
+	g_assert (item->details->item_type == NAUTILUS_PREFERENCE_ITEM_THEME);
+
+	current_value = nautilus_preferences_get (item->details->preference_name);
+
+	g_assert (current_value != NULL);
+	
+	nautilus_string_picker_set_selected_string (NAUTILUS_STRING_PICKER (item->details->child), current_value);
+
+	g_free (current_value);
 }
 
 static void
@@ -611,7 +706,6 @@ preferences_item_create_theme (NautilusPreferencesItem	*item,
 				     const char 	*preference_name)
 {
 	char			*description;
-	char			*current_value;
 	NautilusStringList	*theme_list;
 
 	g_assert (item != NULL);
@@ -633,22 +727,13 @@ preferences_item_create_theme (NautilusPreferencesItem	*item,
 	add_icon_themes(theme_list, "i-directory");
 	
 	nautilus_string_picker_set_string_list (NAUTILUS_STRING_PICKER (item->details->child), theme_list);
-
-	current_value = nautilus_preferences_get (item->details->preference_name);
-
-	g_assert (current_value != NULL);
-	g_assert (nautilus_string_list_contains (theme_list, current_value));
-	
-	nautilus_string_picker_set_selected_string (NAUTILUS_STRING_PICKER (item->details->child), current_value);
-
-	g_free (current_value);
-
 	nautilus_string_list_free (theme_list);
 	
- 	gtk_signal_connect (GTK_OBJECT (item->details->child),
- 			    "changed",
- 			    GTK_SIGNAL_FUNC (text_item_changed_callback),
- 			    (gpointer) item);
+	item->details->change_signal_ID = 
+		gtk_signal_connect (GTK_OBJECT (item->details->child),
+	 			    "changed",
+	 			    GTK_SIGNAL_FUNC (text_item_changed_callback),
+			    	    (gpointer) item);
 }
 
 /* NautilusPreferencesItem public methods */
@@ -709,7 +794,6 @@ static void
 text_item_changed_callback (GtkWidget *button, gpointer user_data)
 {
 	NautilusPreferencesItem	*item;
-	char			*text;
 
 	g_assert (user_data != NULL);
 	g_assert (NAUTILUS_IS_PREFERENCES_ITEM (user_data));
@@ -719,21 +803,13 @@ text_item_changed_callback (GtkWidget *button, gpointer user_data)
 	g_assert (item->details->child != NULL);
 	g_assert (NAUTILUS_IS_STRING_PICKER (item->details->child));
 
-	text = nautilus_string_picker_get_selected_string (NAUTILUS_STRING_PICKER (item->details->child));
-
-	if (text != NULL)
-	{
-		nautilus_preferences_set (item->details->preference_name, text);
-		
-		g_free (text);
-	}
+	preferences_item_update_text_settings_at_idle (item);
 }
 
 static void
 editable_string_changed_callback (GtkWidget *button, gpointer user_data)
 {
 	NautilusPreferencesItem	*item;
-	char			*text;
 	
 	g_assert (user_data != NULL);
 	g_assert (NAUTILUS_IS_PREFERENCES_ITEM (user_data));
@@ -743,22 +819,13 @@ editable_string_changed_callback (GtkWidget *button, gpointer user_data)
 	g_assert (item->details->child != NULL);
 	g_assert (NAUTILUS_IS_TEXT_CAPTION (item->details->child));
 
-	text = nautilus_text_caption_get_text (NAUTILUS_TEXT_CAPTION (item->details->child));
-	
-	if (text != NULL)
-	{
-		nautilus_preferences_set (item->details->preference_name, text);
-		
-		g_free (text);
-	}
+	preferences_item_update_text_settings_at_idle (item);
 }
 
 static void
 integer_changed_callback (GtkWidget *button, gpointer user_data)
 {
 	NautilusPreferencesItem	*item;
-	char			*text;
-	int value = 0;
 	
 	g_assert (user_data != NULL);
 	g_assert (NAUTILUS_IS_PREFERENCES_ITEM (user_data));
@@ -768,13 +835,7 @@ integer_changed_callback (GtkWidget *button, gpointer user_data)
 	g_assert (item->details->child != NULL);
 	g_assert (NAUTILUS_IS_TEXT_CAPTION (item->details->child));
 
-	text = nautilus_text_caption_get_text (NAUTILUS_TEXT_CAPTION (item->details->child));
-
-	if (text != NULL) {
-		nautilus_eat_str_to_int (text, &value);
-	}
-
-	nautilus_preferences_set_integer (item->details->preference_name, value);
+	preferences_item_update_integer_settings_at_idle (item);
 }
 
 char *
@@ -784,3 +845,116 @@ nautilus_preferences_item_get_name (const NautilusPreferencesItem *preferences_i
 
 	return g_strdup (preferences_item->details->preference_name);
 }
+
+void
+nautilus_preferences_item_update_displayed_value (const NautilusPreferencesItem *item)
+{
+	NautilusPreferencesItemType item_type;
+	
+	g_return_if_fail (NAUTILUS_IS_PREFERENCES_ITEM (item));
+
+	item_type = item->details->item_type;
+
+	g_return_if_fail (item->details->item_type != PREFERENCES_ITEM_UNDEFINED_ITEM);
+
+	/* Block the change signal while we update the widget to match the preference */
+	g_assert (item->details->change_signal_ID != 0);
+	g_assert (GTK_IS_WIDGET (item->details->child));
+	gtk_signal_handler_block (GTK_OBJECT (item->details->child),
+				  item->details->change_signal_ID);
+
+	/* Update the child widget according to the item type */
+	switch (item_type)
+	{
+	case NAUTILUS_PREFERENCE_ITEM_BOOLEAN:
+		preferences_item_update_boolean (item);
+		break;
+		
+	case NAUTILUS_PREFERENCE_ITEM_ENUM:
+		preferences_item_update_enum (item);
+		break;
+
+	case NAUTILUS_PREFERENCE_ITEM_SHORT_ENUM:
+		preferences_item_update_short_enum (item);
+		break;
+
+	case NAUTILUS_PREFERENCE_ITEM_FONT_FAMILY:
+		preferences_item_update_font_family (item);
+		break;
+	
+	case NAUTILUS_PREFERENCE_ITEM_THEME:
+		preferences_item_update_theme (item);
+		break;
+	case NAUTILUS_PREFERENCE_ITEM_EDITABLE_STRING:
+		preferences_item_update_editable_string (item);
+		break;	
+	case NAUTILUS_PREFERENCE_ITEM_INTEGER:
+		preferences_item_update_integer (item);
+		break;
+	default:
+		g_assert_not_reached ();
+	}
+
+	gtk_signal_handler_unblock (GTK_OBJECT (item->details->child),
+				    item->details->change_signal_ID);
+}
+
+static gboolean
+update_text_settings_at_idle (NautilusPreferencesItem *preferences_item)
+{
+	char *text;
+
+	text = nautilus_text_caption_get_text (NAUTILUS_TEXT_CAPTION (preferences_item->details->child));
+
+	if (text != NULL)
+	{
+		nautilus_preferences_set (preferences_item->details->preference_name, text);
+		
+		g_free (text);
+	}
+	
+	text_idle_handler = FALSE;
+
+	return FALSE;
+}
+
+static void
+preferences_item_update_text_settings_at_idle (NautilusPreferencesItem *preferences_item)
+{
+	g_return_if_fail (NAUTILUS_IS_PREFERENCES_ITEM (preferences_item));
+
+	if (text_idle_handler == FALSE) {
+		gtk_idle_add ((GtkFunction) update_text_settings_at_idle, preferences_item);
+		text_idle_handler = TRUE;
+	}
+}
+
+static gboolean
+update_integer_settings_at_idle (NautilusPreferencesItem *preferences_item)
+{
+	int value = 0;
+	char *text;
+
+	text = nautilus_text_caption_get_text (NAUTILUS_TEXT_CAPTION (preferences_item->details->child));
+
+	if (text != NULL) {
+		nautilus_eat_str_to_int (text, &value);
+	}
+	
+	nautilus_preferences_set_integer (preferences_item->details->preference_name, value);
+
+	integer_idle_handler = FALSE;
+
+	return FALSE;
+}
+
+static void
+preferences_item_update_integer_settings_at_idle (NautilusPreferencesItem *preferences_item)
+{
+	g_return_if_fail (NAUTILUS_IS_PREFERENCES_ITEM (preferences_item));
+
+	if (integer_idle_handler == FALSE) {
+		gtk_idle_add ((GtkFunction) update_integer_settings_at_idle, preferences_item);
+		integer_idle_handler = TRUE;
+	}
+}
