@@ -82,7 +82,7 @@ static const char *user_level_names_for_storage[] =
 {
 	"novice",
 	"intermediate",
-	"hacker"
+	"advanced"
 };
 
 static char *       preferences_get_path                            (void);
@@ -438,7 +438,7 @@ nautilus_preferences_get_boolean (const char *name)
 	char *key;
 	
 	g_return_val_if_fail (name != NULL, FALSE);
-	
+
 	key = preferences_key_make_for_getter (name);
 	result = nautilus_gconf_get_boolean (key);
 	g_free (key);
@@ -451,14 +451,19 @@ nautilus_preferences_set_integer (const char *name,
 				  int int_value)
 {
 	char *key;
+	int old_value;
 
 	g_return_if_fail (name != NULL);
 	
 	key = preferences_key_make (name);
-	nautilus_gconf_set_integer (key, int_value);
-	g_free (key);
+	old_value = nautilus_preferences_get_integer (name);
 
-	nautilus_gconf_suggest_sync ();
+	if (int_value != old_value) {
+		nautilus_gconf_set_integer (key, int_value);
+
+		nautilus_gconf_suggest_sync ();
+	}
+	g_free (key);
 }
 
 int
@@ -482,14 +487,19 @@ nautilus_preferences_set (const char *name,
 			  const char *string_value)
 {
 	char *key;
+	char *old_value;
 
 	g_return_if_fail (name != NULL);
-	
-	key = preferences_key_make (name);
-	nautilus_gconf_set_string (key, string_value);
-	g_free (key);
 
-	nautilus_gconf_suggest_sync ();
+	key = preferences_key_make (name);
+	old_value = nautilus_preferences_get (name);
+
+	if (strcmp (string_value, old_value) != 0) {
+		nautilus_gconf_set_string (key, string_value);
+		
+		nautilus_gconf_suggest_sync ();
+	}
+	g_free (key);
 }
 
 char *
@@ -513,7 +523,7 @@ nautilus_preferences_get (const char *name)
 
 void
 nautilus_preferences_set_string_list (const char *name,
-				      GSList *string_list_value)
+				      GList *string_list_value)
 {
 	char *key;
 
@@ -526,10 +536,10 @@ nautilus_preferences_set_string_list (const char *name,
 	nautilus_gconf_suggest_sync ();
 }
 
-GSList *
+GList *
 nautilus_preferences_get_string_list (const char *name)
 {
- 	GSList *result;
+ 	GList *result;
 	char *key;
 	
 	g_return_val_if_fail (name != NULL, NULL);
@@ -564,7 +574,7 @@ nautilus_preferences_get_user_level (void)
 	user_level = nautilus_gconf_get_string (key);
 	g_free (key);
 
-	if (nautilus_str_is_equal (user_level, "hacker")) {
+	if (nautilus_str_is_equal (user_level, "advanced")) {
 		result = 2;
 	} else if (nautilus_str_is_equal (user_level, "intermediate")) {
 		result = 1;
@@ -682,7 +692,7 @@ nautilus_preferences_default_get_string (const char *name,
 void
 nautilus_preferences_default_set_string_list (const char *name,
 					      int user_level,
-					      GSList *string_list_value)
+					      GList *string_list_value)
 {
 	char *default_key;
 	
@@ -693,11 +703,11 @@ nautilus_preferences_default_set_string_list (const char *name,
 	g_free (default_key);
 }
 
-GSList *
+GList *
 nautilus_preferences_default_get_string_list (const char *name,
 					      int user_level)
 {
- 	GSList *result;
+ 	GList *result;
 	char *default_key;
 	
 	g_return_val_if_fail (name != NULL, NULL);
@@ -1147,71 +1157,6 @@ nautilus_preferences_add_callback_while_alive (const char *name,
 			    data);
 }
 
-static GList *remove_list = NULL;
-
-static void
-preferences_while_process_running_remover (void)
-{
-	GList *iterator;
-
-	for (iterator = remove_list; iterator != NULL; iterator = iterator->next) {
-		WhileAliveData *data;
-		
-		data = iterator->data;
-		g_assert (data != NULL);
-
-		nautilus_preferences_remove_callback (data->name,
-						      data->callback,
-						      data->callback_data);
-
-		g_free (data->name);
-		g_free (data);
-	}
-
-	g_list_free (remove_list);
-	remove_list = NULL;
-}
-
-/*
- * nautilus_preferences_add_callback_while_process_is_running:
- *
- * @name: Preference name.
- * @callback: Callback function.
- * @callback_data: Data for callback function.
- *
- * Add a preference callback for the duration of the currently
- * running process.  The callback will be automatically removed
- * at exit time.  This is useful for preference values that need
- * to be stored be stored statically
- */
-void
-nautilus_preferences_add_callback_while_process_is_running (const char *name,
-							    NautilusPreferencesCallback callback,
-							    gpointer callback_data)
-{
-	static gboolean while_process_running_remover_installed = FALSE;
-	WhileAliveData *data;
-
-	g_return_if_fail (name != NULL);
-	g_return_if_fail (callback != NULL);
-
-	/* Setup the at exit disconnector once */
-	if (while_process_running_remover_installed == FALSE) {
-		g_atexit (preferences_while_process_running_remover);
-		while_process_running_remover_installed = TRUE;
-	}
-
-	data = g_new (WhileAliveData, 1);
-	data->name = g_strdup (name);
-	data->callback = callback;
-	data->callback_data = callback_data;
-	
-	nautilus_preferences_add_callback (name, callback, callback_data);
-
-	remove_list = g_list_append (remove_list, data);
-}
-
-
 void
 nautilus_preferences_remove_callback (const char *name,
 				      NautilusPreferencesCallback callback,
@@ -1341,7 +1286,7 @@ nautilus_preferences_get_user_level_name_for_display (int user_level)
 {
 	user_level = preferences_user_level_check_range (user_level);
 	
-	return g_strdup (user_level_names_for_display[user_level]);
+	return g_strdup (gettext (user_level_names_for_display[user_level]));
 }
 
 char *
