@@ -89,6 +89,9 @@ typedef struct {
  */
 #define NAME_COLUMN_INITIAL_WIDTH	 200
 
+/* Progamr name of the mime type capplet */
+#define FILE_TYPES_CAPPLET_NAME 	"file-types-capplet"
+
 /* Forward declarations as needed */
 static gboolean program_file_pair_is_default_for_file_type 	 (ProgramFilePair *pair);
 static gboolean program_file_pair_is_default_for_file 		 (ProgramFilePair *pair);
@@ -182,8 +185,7 @@ program_file_pair_get_program_name_for_display (ProgramFilePair *pair)
 		  || pair->view_identifier != NULL);
 
 	if (pair->action_type == GNOME_VFS_MIME_ACTION_TYPE_COMPONENT) {
-		return g_strdup_printf
-			(_("View as %s"), pair->view_identifier->name);
+		return g_strdup (pair->view_identifier->view_as_label);
 	}
 
 	return g_strdup (pair->application->name);	
@@ -358,6 +360,9 @@ repopulate_program_list (GnomeDialog *program_chooser,
 	}
 
 	gtk_clist_sort (clist);
+
+	/* Start with first item selected, rather than some arbitrary item */
+	gtk_clist_select_row (clist, 0, 0);
 }
 
 static NautilusFile *
@@ -889,25 +894,40 @@ set_default_for_item (ProgramFilePair *pair)
 }
 
 static void
-launch_mime_capplet (GtkWidget *button, gpointer ignored)
+launch_mime_capplet (GtkWidget *button, gpointer callback_data)
 {
+	char *command;
+		
 	g_assert (GTK_IS_WIDGET (button));
-
-	nautilus_launch_application_from_command ("nautilus-mime-type-capplet", NULL, FALSE);
+	
+	command = g_strdup_printf ("%s %s", FILE_TYPES_CAPPLET_NAME, (char *)callback_data);
+		
+	nautilus_launch_application_from_command (command, NULL, FALSE);
+	
+	g_free (command);
 }
 
 static void
 launch_mime_capplet_and_close_dialog (GtkWidget *button, gpointer callback_data)
 {
+	ProgramFilePair *file_pair;
+	char *mime_type;
+	
 	g_assert (GTK_IS_WIDGET (button));
 	g_assert (GNOME_IS_DIALOG (callback_data));
 
-	launch_mime_capplet (button, callback_data);
+	file_pair = get_selected_program_file_pair (GNOME_DIALOG (callback_data));
+		
+	mime_type = nautilus_file_get_mime_type (file_pair->file);
+
+	launch_mime_capplet (button, mime_type);
 
 	/* Don't leave a nested modal dialogs in the wake of switching
 	 * user's attention to the capplet.
 	 */	
 	gnome_dialog_close (GNOME_DIALOG (callback_data));
+	
+	g_free (mime_type);
 }
 
 static void
@@ -1438,11 +1458,11 @@ nautilus_program_chooser_show_no_choices_message (GnomeVFSMimeActionType action_
 	file_name = nautilus_file_get_name (file);
 
 	if (action_type == GNOME_VFS_MIME_ACTION_TYPE_COMPONENT) {
-		unavailable_message = g_strdup_printf ("No viewers are available for %s.", file_name);		
+		unavailable_message = g_strdup_printf ("No viewers are available for \"%s\".", file_name);		
 		dialog_title = g_strdup ("No Viewers Available");
 	} else {
 		g_assert (action_type == GNOME_VFS_MIME_ACTION_TYPE_APPLICATION);
-		unavailable_message = g_strdup_printf ("No applications are available for %s.", file_name);		
+		unavailable_message = g_strdup_printf ("No applications are available for \"%s\".", file_name);		
 		dialog_title = g_strdup ("No Applications Available");
 	}
 
@@ -1455,10 +1475,10 @@ nautilus_program_chooser_show_no_choices_message (GnomeVFSMimeActionType action_
 				  "for which file types with the \"File Types and "
 				  "Programs\" part of the GNOME Control Center. Do "
 				  "you want to go there now?", unavailable_message);
-	dialog = nautilus_yes_no_dialog 
+	dialog = nautilus_show_yes_no_dialog 
 		(prompt, dialog_title, GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, parent_window);
 
-	gnome_dialog_button_connect (dialog, GNOME_OK, launch_mime_capplet, NULL);
+	gnome_dialog_button_connect (dialog, GNOME_OK, launch_mime_capplet, nautilus_file_get_mime_type (file));
 
 	g_free (unavailable_message);
 	g_free (file_name);
