@@ -1,3 +1,4 @@
+
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 
 /*
@@ -23,6 +24,7 @@
  * Author: Maciej Stachowiak <mjs@eazel.com>
  *         Ettore Perazzoli <ettore@gnu.org>
  *         Michael Meeks <michael@nuclecu.unam.mx>
+ *	   Andy Hertzfeld <andy@eazel.com>
  *
  */
 
@@ -62,6 +64,18 @@ static const char untranslated_go_to_label[] = N_("Go To:");
 #define LOCATION_LABEL _(untranslated_location_label)
 #define GO_TO_LABEL _(untranslated_go_to_label)
 
+struct NautilusLocationBarDetails {
+	GtkLabel *label;
+	GtkEntry *entry;
+	
+	char *last_location;
+	
+	char *current_directory;
+	GList *file_info_list;
+	
+	uint idle_id;
+};
+
 enum {
 	NAUTILUS_DND_MC_DESKTOP_ICON,
 	NAUTILUS_DND_URI_LIST,
@@ -139,7 +153,7 @@ drag_data_received_callback (GtkWidget *widget,
 		prompt = g_strdup_printf (_("Do you want to view these %d locations "
 					  "in separate windows?"), 
 					  name_count);
-		new_windows_for_extras = nautilus_simple_dialog 
+		new_windows_for_extras = nautilus_run_simple_dialog 
 			(GTK_WIDGET (nautilus_location_bar_get_window (widget)),
 			 TRUE,
 			 prompt,
@@ -164,7 +178,7 @@ drag_data_received_callback (GtkWidget *widget,
 		application = nautilus_location_bar_get_window (widget)->application;
 		for (node = names->next; node != NULL; node = node->next) {
 			new_window = nautilus_application_create_window (application);
-			nautilus_window_goto_uri (new_window, node->data);
+			nautilus_window_go_to (new_window, node->data);
 		}
 	}
 						  
@@ -246,18 +260,43 @@ accumulate_name(char *full_name, char *candidate_name)
 	return result_name;
 }
 
+/* utility routine to load the file info list for the current directory, if necessary */
+static void
+get_file_info_list (NautilusLocationBar *bar, const char* dir_name)
+{
+	GnomeVFSResult result;
+	if (nautilus_strcmp (bar->details->current_directory, dir_name) != 0) {
+		g_free (bar->details->current_directory);
+		if (bar->details->file_info_list) {
+			gnome_vfs_file_info_list_free (bar->details->file_info_list);	
+			bar->details->file_info_list = NULL;		
+		}
+		
+		bar->details->current_directory = g_strdup (dir_name);
+		result = gnome_vfs_directory_list_load (&bar->details->file_info_list, dir_name,
+							GNOME_VFS_FILE_INFO_DEFAULT, NULL);
+		if (result != GNOME_VFS_OK) {
+			if (bar->details->file_info_list) {
+				gnome_vfs_file_info_list_free (bar->details->file_info_list);	
+				bar->details->file_info_list = NULL;			
+			}
+		}
+	}
+}
+
 /* routine that performs the tab expansion using gnome-vfs.  Extract the directory name and
   incomplete basename, then iterate through the directory trying to complete it.  If we
   find something, add it to the entry */
   
-static void
-try_to_expand_path (GtkEditable *editable)
+static gboolean
+try_to_expand_path (NautilusLocationBar *bar)
 {
-	GnomeVFSResult result;
 	GnomeVFSFileInfo *current_file_info;
-	GnomeVFSDirectoryList *list;
+	GList *element;
 	GnomeVFSURI *uri;
-	int base_length;
+	GtkEditable *editable;
+	
+	uint base_length;
 	int current_path_length;
 	int offset;
 	const char *base_name_ptr;
@@ -267,24 +306,30 @@ try_to_expand_path (GtkEditable *editable)
 	char *dir_name;
 	char *expand_text;
 	char *expand_name;
-	
+	char *tilde_expand_name;
+	char *expanded_name;
+
+	editable = GTK_EDITABLE (bar->details->entry);
 	user_location = gtk_editable_get_chars (editable, 0, -1);
+	bar->details->idle_id = 0;
 	
 	/* if it's just '~' or '~/', don't expand because slash shouldn't be appended */
 	if (nautilus_strcmp (user_location, "~") == 0
 	    || nautilus_strcmp (user_location, "~/") == 0) {
-		return;
+		return FALSE;
 	}
+	gnome_vfs_expand_initial_tilde (user_location);
 	current_path = nautilus_make_uri_from_input (user_location);
 
 	if (!nautilus_istr_has_prefix (current_path, "file://")) {
 		g_free (current_path);
-		return;
+		return FALSE;
 	}
 
 	current_path_length = strlen(current_path);	
 	offset = current_path_length - strlen(user_location);
 
+	gnome_vfs_expand_initial_tilde (current_path);
 	uri = gnome_vfs_uri_new (current_path);
 	
 	base_name_ptr = gnome_vfs_uri_get_basename (uri);
@@ -297,30 +342,28 @@ try_to_expand_path (GtkEditable *editable)
 	if (base_name == NULL) {
 		gnome_vfs_uri_unref (uri);
 		g_free (current_path);
-		return;	
+		return FALSE;	
 	}
-	
+
 	base_length = strlen (base_name);
 	dir_name = gnome_vfs_uri_extract_dirname (uri);
 
-	/* get file info for the directory */
-
-	result = gnome_vfs_directory_list_load (&list, dir_name,
-						GNOME_VFS_FILE_INFO_DEFAULT, NULL);
-	if (result != GNOME_VFS_OK) {
+	/* get file info for the directory, if it hasn't changed since last time */
+	get_file_info_list (bar, dir_name);	
+	if (bar->details->file_info_list == NULL) {
 		g_free (dir_name);
 		g_free (base_name);
 		gnome_vfs_uri_unref (uri);
 		g_free (current_path);
-		return;
+		return FALSE;
 	}
 
 	/* iterate through the directory, keeping the intersection of all the names that
 	   have the current basename as a prefix. */
 
-	current_file_info = gnome_vfs_directory_list_first(list);
 	expand_text = NULL;
-	while (current_file_info != NULL) {
+	for (element = bar->details->file_info_list; element != NULL; element = element->next) {
+		current_file_info = element->data;
 		if (nautilus_str_has_prefix (current_file_info->name, base_name)) {
 			if (current_file_info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
 				expand_name = g_strconcat (current_file_info->name, "/", NULL);
@@ -330,22 +373,30 @@ try_to_expand_path (GtkEditable *editable)
 			expand_text = accumulate_name (expand_text, expand_name);
 			g_free (expand_name);
 		}
-		current_file_info = gnome_vfs_directory_list_next (list);
 	}
 	
 	/* if we've got something, add it to the entry */	
-	if (expand_text && !nautilus_str_has_suffix (current_path, expand_text)) {
+	if (expand_text && !nautilus_str_has_suffix (current_path, expand_text)
+	    && base_length < strlen (expand_text)) {
 		gtk_entry_append_text (GTK_ENTRY (editable), expand_text + base_length);
- 		gtk_entry_select_region (GTK_ENTRY (editable), current_path_length - offset,
+		gtk_entry_select_region (GTK_ENTRY (editable), current_path_length - offset,
 					 current_path_length + strlen (expand_text) - base_length - offset);
-		g_free (expand_text);
 	}
-	
+	g_free (expand_text);
+
+	tilde_expand_name = gtk_entry_get_text (GTK_ENTRY (editable));
+	if (*tilde_expand_name == '~') {
+		expanded_name = gnome_vfs_expand_initial_tilde (tilde_expand_name);
+		gtk_entry_set_text (GTK_ENTRY (editable), expanded_name);
+		g_free (expanded_name);
+	}
+
 	g_free (dir_name);
 	g_free (base_name);
 	g_free (current_path);
 	g_free (user_location);
-	gnome_vfs_directory_list_destroy (list);
+
+	return FALSE;
 }
 
 /* Until we have a more elegant solution, this is how we figure out if
@@ -394,10 +445,13 @@ editable_key_press_callback (GtkObject *object,
 	int position;
 	gboolean *return_value_location;
 	NautilusLocationBar *bar;
+	char *expanded_text;
 	
 	g_assert (n_args == 1);
 	g_assert (args != NULL);
 
+	bar = NAUTILUS_LOCATION_BAR (data);
+
 	editable = GTK_EDITABLE (object);
 	event = GTK_VALUE_POINTER (args[0]);
 	return_value_location = GTK_RETLOC_BOOL (args[1]);
@@ -409,14 +463,22 @@ editable_key_press_callback (GtkObject *object,
 	}
 	
 	/* Only do an expand if we just handled a key that inserted
-	 * characters.
+	 * characters. Do the expand at idle time to avoid slowing down typing
+	 * when the directory is large.
 	 */
 	if (*return_value_location
+	    && bar->details->idle_id <= 0
 	    && entry_would_have_inserted_characters (event))  {
-		try_to_expand_path (editable);
+        	bar->details->idle_id = gtk_idle_add ( (GtkFunction) try_to_expand_path, bar);		
+	}
+
+	if (event->keyval == GDK_slash) {
+		expanded_text = gnome_vfs_expand_initial_tilde (gtk_entry_get_text (GTK_ENTRY (editable)));
+
+		gtk_entry_set_text (GTK_ENTRY (editable), expanded_text);
+		g_free (expanded_text);
 	}
 	
-	bar = NAUTILUS_LOCATION_BAR (data);
 	nautilus_location_bar_update_label (bar);
 }
 
@@ -428,7 +490,7 @@ real_activate (NautilusNavigationBar *navigation_bar)
 	bar = NAUTILUS_LOCATION_BAR (navigation_bar);
 
 	/* Put the keyboard focus in the text field when switching to this mode */
-	gtk_widget_grab_focus (GTK_WIDGET (bar->entry));
+	gtk_widget_grab_focus (GTK_WIDGET (bar->details->entry));
 }
 
 static void
@@ -438,9 +500,22 @@ destroy (GtkObject *object)
 
 	bar = NAUTILUS_LOCATION_BAR (object);
 	
-	g_free (bar->last_location);
-
-	NAUTILUS_CALL_PARENT_CLASS (GTK_OBJECT_CLASS, destroy, (object));
+	/* cancel the pending idle call, if any */
+	if (bar->details->idle_id > 0) {
+		gtk_idle_remove (bar->details->idle_id);
+	}
+	
+	if (bar->details->file_info_list) {
+		gnome_vfs_file_info_list_free (bar->details->file_info_list);	
+	
+	}
+	
+	g_free (bar->details->current_directory);
+	
+	g_free (bar->details->last_location);
+	g_free (bar->details);
+	
+	NAUTILUS_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
 static void
@@ -467,6 +542,8 @@ nautilus_location_bar_initialize (NautilusLocationBar *bar)
 	GtkWidget *event_box;
 	GtkWidget *hbox;
 
+	bar->details = g_new0 (NautilusLocationBarDetails, 1);
+
 	hbox = gtk_hbox_new (0, FALSE);
 
 	event_box = gtk_event_box_new ();
@@ -521,8 +598,8 @@ nautilus_location_bar_initialize (NautilusLocationBar *bar)
 
 	gtk_widget_show_all (hbox);
 
-	bar->label = GTK_LABEL (label);
-	bar->entry = GTK_ENTRY (entry);	
+	bar->details->label = GTK_LABEL (label);
+	bar->details->entry = GTK_ENTRY (entry);	
 }
 
 
@@ -537,7 +614,7 @@ nautilus_location_bar_new (NautilusWindow *window)
 
 	/* Clipboard */
 	nautilus_clipboard_set_up_editable
-		(GTK_EDITABLE (location_bar->entry),
+		(GTK_EDITABLE (location_bar->details->entry),
 		 nautilus_window_get_ui_container (window),
 		 TRUE);
 
@@ -560,14 +637,25 @@ nautilus_location_bar_set_location (NautilusNavigationBar *navigation_bar,
 	 * thus should not emit the LOCATION_CHANGED signal.*/
 	
 	formatted_location = nautilus_format_uri_for_display (location);
-	nautilus_entry_set_text (NAUTILUS_ENTRY (bar->entry),
+	nautilus_entry_set_text (NAUTILUS_ENTRY (bar->details->entry),
 				 formatted_location);
 	g_free (formatted_location);
 
+	/* free up the cached file info from the previous location */
+	if (bar->details->current_directory) {
+		g_free (bar->details->current_directory);
+		bar->details->current_directory = NULL;
+	}
+	
+	if (bar->details->file_info_list) {
+		gnome_vfs_file_info_list_free (bar->details->file_info_list);	
+		bar->details->file_info_list = NULL;			
+	}
+	
 	/* remember the original location for later comparison */
 	
-	g_free (bar->last_location);
-	bar->last_location = g_strdup (location);
+	g_free (bar->details->last_location);
+	bar->details->last_location = g_strdup (location);
 	nautilus_location_bar_update_label (bar);
 }
 
@@ -591,7 +679,7 @@ nautilus_location_bar_get_location (NautilusNavigationBar *navigation_bar)
 
 	bar = NAUTILUS_LOCATION_BAR (navigation_bar);
 	
-	user_location = gtk_editable_get_chars (GTK_EDITABLE (bar->entry), 0, -1);
+	user_location = gtk_editable_get_chars (GTK_EDITABLE (bar->details->entry), 0, -1);
 	best_uri = nautilus_make_uri_from_input (user_location);
 	g_free (user_location);
 	return best_uri;
@@ -608,13 +696,13 @@ nautilus_location_bar_update_label (NautilusLocationBar *bar)
 {
 	char *current_text, *current_location;
 	
-	current_text = gtk_entry_get_text (GTK_ENTRY (bar->entry));
+	current_text = gtk_entry_get_text (GTK_ENTRY (bar->details->entry));
 	current_location = nautilus_make_uri_from_input (current_text);
 	
-	if (nautilus_uris_match (bar->last_location, current_location)) {
-		gtk_label_set_text (GTK_LABEL (bar->label), LOCATION_LABEL);
+	if (nautilus_uris_match (bar->details->last_location, current_location)) {
+		gtk_label_set_text (GTK_LABEL (bar->details->label), LOCATION_LABEL);
 	} else {		 
-		gtk_label_set_text (GTK_LABEL (bar->label), GO_TO_LABEL);
+		gtk_label_set_text (GTK_LABEL (bar->details->label), GO_TO_LABEL);
 	}	
 	g_free (current_location);
 }
