@@ -4,7 +4,7 @@
  *  Nautilus
  *
  *  Copyright (C) 1999, 2000 Red Hat, Inc.
- *  Copyright (C) 1999, 2000 Eazel, Inc.
+ *  Copyright (C) 1999, 2000, 2001 Eazel, Inc.
  *
  *  Nautilus is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License as
@@ -20,7 +20,8 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *  Author: Elliot Lee <sopwith@redhat.com>
+ *  Authors: Elliot Lee <sopwith@redhat.com>
+ *           Darin Adler <darin@eazel.com>
  *
  */
 
@@ -35,6 +36,13 @@
 #include <gtk/gtksignal.h>
 #include <libnautilus/nautilus-bonobo-workarounds.h>
 #include <libnautilus/nautilus-view.h>
+#include <libnautilus-extensions/nautilus-gtk-extensions.h>
+
+typedef struct {
+	char *location;
+	GList *selection;
+	char *title;
+} LocationPlus;
 
 static void impl_Nautilus_ViewFrame_open_location_in_this_window         (PortableServer_Servant  servant,
 									  Nautilus_URI            location,
@@ -46,6 +54,11 @@ static void impl_Nautilus_ViewFrame_open_location_force_new_window       (Portab
 									  Nautilus_URI            location,
 									  const Nautilus_URIList *selection,
 									  CORBA_Environment      *ev);
+static void impl_Nautilus_ViewFrame_report_location_change               (PortableServer_Servant  servant,
+									  Nautilus_URI            location,
+									  const Nautilus_URIList *selection,
+									  const CORBA_char       *title,
+									  CORBA_Environment      *ev);
 static void impl_Nautilus_ViewFrame_report_selection_change              (PortableServer_Servant  servant,
 									  const Nautilus_URIList *selection,
 									  CORBA_Environment      *ev);
@@ -64,6 +77,8 @@ static void impl_Nautilus_ViewFrame_report_load_failed                   (Portab
 static void impl_Nautilus_ViewFrame_set_title                            (PortableServer_Servant  servant,
 									  const CORBA_char       *title,
 									  CORBA_Environment      *ev);
+static void impl_Nautilus_ViewFrame_go_back                              (PortableServer_Servant  servant,
+									  CORBA_Environment      *ev);
 
 POA_Nautilus_ViewFrame__epv impl_Nautilus_ViewFrame_epv =
 {
@@ -71,6 +86,7 @@ POA_Nautilus_ViewFrame__epv impl_Nautilus_ViewFrame_epv =
 	&impl_Nautilus_ViewFrame_open_location_in_this_window,
 	&impl_Nautilus_ViewFrame_open_location_prefer_existing_window,
 	&impl_Nautilus_ViewFrame_open_location_force_new_window,
+	&impl_Nautilus_ViewFrame_report_location_change,
 	&impl_Nautilus_ViewFrame_report_selection_change,
 	&impl_Nautilus_ViewFrame_report_status,
 	&impl_Nautilus_ViewFrame_report_load_underway,
@@ -78,6 +94,7 @@ POA_Nautilus_ViewFrame__epv impl_Nautilus_ViewFrame_epv =
 	&impl_Nautilus_ViewFrame_report_load_complete,
 	&impl_Nautilus_ViewFrame_report_load_failed,
 	&impl_Nautilus_ViewFrame_set_title,
+	&impl_Nautilus_ViewFrame_go_back,
 };
 
 static PortableServer_ServantBase__epv base_epv;
@@ -88,14 +105,6 @@ POA_Nautilus_ViewFrame__vepv impl_Nautilus_ViewFrame_vepv =
 	&impl_Nautilus_ViewFrame_epv
 };
 
-static void
-view_widget_gone (GtkObject *object,
-		  impl_POA_Nautilus_ViewFrame *servant)
-{
-	g_assert (servant->view == NAUTILUS_VIEW_FRAME (object));
-	servant->view = NULL;
-}
-
 static void
 impl_Nautilus_ViewFrame__destroy (BonoboObject *object,
 				  impl_POA_Nautilus_ViewFrame *servant)
@@ -105,12 +114,7 @@ impl_Nautilus_ViewFrame__destroy (BonoboObject *object,
 	
 	CORBA_exception_init (&ev);
 
-	if (servant->view != NULL) {
-		gtk_signal_disconnect_by_func (GTK_OBJECT (servant->view),
-					       view_widget_gone,
-					       servant);
-	}
-	
+	nautilus_nullify_cancel (&servant->view);
 	object_id = PortableServer_POA_servant_to_id (bonobo_poa (), servant, &ev);
 	PortableServer_POA_deactivate_object (bonobo_poa (), object_id, &ev);
 	CORBA_free (object_id);
@@ -140,24 +144,136 @@ impl_Nautilus_ViewFrame__create (NautilusViewFrame *view, CORBA_Environment *ev)
 	gtk_signal_connect (GTK_OBJECT (bonobo_object), "destroy",
 			    GTK_SIGNAL_FUNC (impl_Nautilus_ViewFrame__destroy), servant);
 
-	gtk_signal_connect (GTK_OBJECT (view), "destroy",
-			    view_widget_gone, servant);
+	nautilus_nullify_when_destroyed (&servant->view);
   
 	return bonobo_object;
 }
 
+static void
+list_free_deep_callback (gpointer callback_data)
+{
+	nautilus_g_list_free_deep (callback_data);
+}
+
+static void
+free_location_plus_callback (gpointer callback_data)
+{
+	LocationPlus *location_plus;
+
+	location_plus = callback_data;
+	g_free (location_plus->location);
+	nautilus_g_list_free_deep (location_plus->selection);
+	g_free (location_plus->title);
+	g_free (location_plus);
+}
+
+static void
+open_in_this_window (NautilusViewFrame *view,
+		     gpointer callback_data)
+{
+	nautilus_view_frame_open_location_in_this_window (view, callback_data);
+}
+
+static void
+open_prefer_existing_window (NautilusViewFrame *view,
+			     gpointer callback_data)
+{
+	nautilus_view_frame_open_location_prefer_existing_window (view, callback_data);
+}
+
+static void
+open_force_new_window (NautilusViewFrame *view,
+		       gpointer callback_data)
+{
+	LocationPlus *location_plus;
+
+	location_plus = callback_data;
+	nautilus_view_frame_open_location_force_new_window
+		(view,
+		 location_plus->location,
+		 location_plus->selection);
+}
+
+static void
+report_location_change (NautilusViewFrame *view,
+			gpointer callback_data)
+{
+	LocationPlus *location_plus;
+
+	location_plus = callback_data;
+	nautilus_view_frame_report_location_change
+		(view,
+		 location_plus->location,
+		 location_plus->selection,
+		 location_plus->title);
+}
+
+static void
+report_selection_change (NautilusViewFrame *view,
+			 gpointer callback_data)
+{
+	nautilus_view_frame_report_selection_change (view, callback_data);
+}
+
+static void
+report_status (NautilusViewFrame *view,
+	       gpointer callback_data)
+{
+	nautilus_view_frame_report_status (view, callback_data);
+}
+
+static void
+report_load_underway (NautilusViewFrame *view,
+		      gpointer callback_data)
+{
+	nautilus_view_frame_report_load_underway (view);
+}
+
+static void
+report_load_progress (NautilusViewFrame *view,
+		      gpointer callback_data)
+{
+	nautilus_view_frame_report_load_progress (view, * (float *) callback_data);
+}
+
+static void
+report_load_complete (NautilusViewFrame *view,
+		      gpointer callback_data)
+{
+	nautilus_view_frame_report_load_complete (view);
+}
+
+static void
+report_load_failed (NautilusViewFrame *view,
+		      gpointer callback_data)
+{
+	nautilus_view_frame_report_load_failed (view);
+}
+
+static void
+set_title (NautilusViewFrame *view,
+	   gpointer callback_data)
+{
+	nautilus_view_frame_set_title (view, callback_data);
+}
+
+static void
+go_back (NautilusViewFrame *view,
+	 gpointer callback_data)
+{
+	nautilus_view_frame_go_back (view);
+}
+
 static void
 impl_Nautilus_ViewFrame_open_location_in_this_window (PortableServer_Servant servant,
 						      Nautilus_URI location,
 						      CORBA_Environment *ev)
 {
-	NautilusViewFrame *view;
-
-	view = ((impl_POA_Nautilus_ViewFrame *) servant)->view;
-	if (view == NULL) {
-		return;
-	}
-	nautilus_view_frame_open_location_in_this_window (view, location);
+	nautilus_view_frame_queue_incoming_call
+		(servant,
+		 open_in_this_window,
+		 g_strdup (location),
+		 g_free);
 }
 
 static void
@@ -165,13 +281,11 @@ impl_Nautilus_ViewFrame_open_location_prefer_existing_window (PortableServer_Ser
 							      Nautilus_URI location,
 							      CORBA_Environment *ev)
 {
-	NautilusViewFrame *view;
-
-	view = ((impl_POA_Nautilus_ViewFrame *) servant)->view;
-	if (view == NULL) {
-		return;
-	}
-	nautilus_view_frame_open_location_prefer_existing_window (view, location);
+	nautilus_view_frame_queue_incoming_call
+		(servant,
+		 open_prefer_existing_window,
+		 g_strdup (location),
+		 g_free);
 }
 
 static void
@@ -180,17 +294,38 @@ impl_Nautilus_ViewFrame_open_location_force_new_window (PortableServer_Servant s
 							const Nautilus_URIList *selection,
 							CORBA_Environment *ev)
 {
-	NautilusViewFrame *view;
-	GList *selection_as_g_list;
+	LocationPlus *location_plus;
 
-	view = ((impl_POA_Nautilus_ViewFrame *) servant)->view;
-	if (view == NULL) {
-		return;
-	}
-	selection_as_g_list = nautilus_shallow_g_list_from_uri_list (selection);
-	nautilus_view_frame_open_location_force_new_window
-		(view, location, selection_as_g_list);
-	g_list_free (selection_as_g_list);
+	location_plus = g_new0 (LocationPlus, 1);
+	location_plus->location = g_strdup (location);
+	location_plus->selection = nautilus_g_list_from_uri_list (selection);
+
+	nautilus_view_frame_queue_incoming_call
+		(servant,
+		 open_force_new_window,
+		 location_plus,
+		 free_location_plus_callback);
+}
+
+static void
+impl_Nautilus_ViewFrame_report_location_change (PortableServer_Servant servant,
+						Nautilus_URI location,
+						const Nautilus_URIList *selection,
+						const CORBA_char *title,
+						CORBA_Environment *ev)
+{
+	LocationPlus *location_plus;
+
+	location_plus = g_new0 (LocationPlus, 1);
+	location_plus->location = g_strdup (location);
+	location_plus->selection = nautilus_g_list_from_uri_list (selection);
+	location_plus->title = g_strdup (title);
+
+	nautilus_view_frame_queue_incoming_call
+		(servant,
+		 report_location_change,
+		 location_plus,
+		 free_location_plus_callback);
 }
 
 static void
@@ -198,17 +333,11 @@ impl_Nautilus_ViewFrame_report_selection_change (PortableServer_Servant servant,
 						 const Nautilus_URIList *selection,
 						 CORBA_Environment *ev)
 {
-	GList *selection_as_g_list;
-	NautilusViewFrame *view;
-
-	view = ((impl_POA_Nautilus_ViewFrame *) servant)->view;
-	if (view == NULL) {
-		return;
-	}
-	selection_as_g_list = nautilus_shallow_g_list_from_uri_list (selection);
-	nautilus_view_frame_report_selection_change
-		(view, selection_as_g_list);
-	g_list_free (selection_as_g_list);
+	nautilus_view_frame_queue_incoming_call
+		(servant,
+		 report_selection_change,
+		 nautilus_g_list_from_uri_list (selection),
+		 list_free_deep_callback);
 }
 
 static void
@@ -216,26 +345,22 @@ impl_Nautilus_ViewFrame_report_status (PortableServer_Servant servant,
 				       const CORBA_char *status,
 				       CORBA_Environment *ev)
 {
-	NautilusViewFrame *view;
-
-	view = ((impl_POA_Nautilus_ViewFrame *) servant)->view;
-	if (view == NULL) {
-		return;
-	}
-	nautilus_view_frame_report_status (view, status);
+	nautilus_view_frame_queue_incoming_call
+		(servant,
+		 report_status,
+		 g_strdup (status),
+		 g_free);
 }
 
 static void
 impl_Nautilus_ViewFrame_report_load_underway (PortableServer_Servant servant,
 					      CORBA_Environment *ev)
 {
-	NautilusViewFrame *view;
-
-	view = ((impl_POA_Nautilus_ViewFrame *) servant)->view;
-	if (view == NULL) {
-		return;
-	}
-	nautilus_view_frame_report_load_underway (view);
+	nautilus_view_frame_queue_incoming_call
+		(servant,
+		 report_load_underway,
+		 NULL,
+		 NULL);
 }
 
 static void
@@ -243,39 +368,37 @@ impl_Nautilus_ViewFrame_report_load_progress (PortableServer_Servant servant,
 					      CORBA_float fraction_done,
 					      CORBA_Environment *ev)
 {
-	NautilusViewFrame *view;
+	float *copy;
 
-	view = ((impl_POA_Nautilus_ViewFrame *) servant)->view;
-	if (view == NULL) {
-		return;
-	}
-	nautilus_view_frame_report_load_progress (view, fraction_done);
+	copy = g_new (float, 1);
+	*copy = fraction_done;
+	nautilus_view_frame_queue_incoming_call
+		(servant,
+		 report_load_progress,
+		 copy,
+		 g_free);
 }
 
 static void
 impl_Nautilus_ViewFrame_report_load_complete (PortableServer_Servant servant,
 					      CORBA_Environment *ev)
 {
-	NautilusViewFrame *view;
-
-	view = ((impl_POA_Nautilus_ViewFrame *) servant)->view;
-	if (view == NULL) {
-		return;
-	}
-	nautilus_view_frame_report_load_complete (view);
+	nautilus_view_frame_queue_incoming_call
+		(servant,
+		 report_load_complete,
+		 NULL,
+		 NULL);
 }
 
 static void
 impl_Nautilus_ViewFrame_report_load_failed (PortableServer_Servant servant,
 					    CORBA_Environment *ev)
 {
-	NautilusViewFrame *view;
-
-	view = ((impl_POA_Nautilus_ViewFrame *) servant)->view;
-	if (view == NULL) {
-		return;
-	}
-	nautilus_view_frame_report_load_failed (view);
+	nautilus_view_frame_queue_incoming_call
+		(servant,
+		 report_load_failed,
+		 NULL,
+		 NULL);
 }
 
 static void
@@ -283,11 +406,17 @@ impl_Nautilus_ViewFrame_set_title (PortableServer_Servant servant,
 				   const CORBA_char *title,
 				   CORBA_Environment *ev)
 {
-	NautilusViewFrame *view;
+	nautilus_view_frame_queue_incoming_call
+		(servant,
+		 set_title,
+		 g_strdup (title),
+		 g_free);
+}
 
-	view = ((impl_POA_Nautilus_ViewFrame *) servant)->view;
-	if (view == NULL) {
-		return;
-	}
-	nautilus_view_frame_set_title (view, title);
+static void
+impl_Nautilus_ViewFrame_go_back (PortableServer_Servant servant,
+				 CORBA_Environment *ev)
+{
+	nautilus_view_frame_queue_incoming_call
+		(servant, go_back, NULL, NULL);
 }
